@@ -1,5 +1,5 @@
 /*
- *  Driver for generic CS4235/CS4236B/CS4237B/CS4238B/CS4239 chips
+ *  Driver for generic CS4232/CS4235/CS4236B/CS4237B/CS4238B/CS4239 chips
  *  Copyright (c) by Jaroslav Kysela <perex@suse.cz>
  *
  *
@@ -20,11 +20,19 @@
  */
 
 #define SND_MAIN_OBJECT_FILE
-#include "driver.h"
-#include "cs4231.h"
-#include "mpu401.h"
-#include "opl3.h"
-#include "initval.h"
+#include "../include/driver.h"
+#include "../include/cs4231.h"
+#include "../include/mpu401.h"
+#include "../include/opl3.h"
+#include "../include/initval.h"
+
+#if defined(CS4232)
+#define IDENT "CS4232"
+#else
+#define IDENT "CS4236+"
+#undef CS4236
+#define CS4236
+#endif
 
 int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 1-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;	/* ID for this card */
@@ -39,34 +47,35 @@ int snd_dma1[SND_CARDS] = SND_DEFAULT_DMA;	/* 0,1,3,5,6,7 */
 int snd_dma1_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
 int snd_dma2[SND_CARDS] = SND_DEFAULT_DMA;	/* 0,1,3,5,6,7 */
 int snd_dma2_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
-#ifdef MODULE_PARM
+int snd_isapnp[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 1};
 MODULE_PARM(snd_index, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_index, "Index value for CS4236+ soundcard.");
+MODULE_PARM_DESC(snd_index, "Index value for " IDENT " soundcard.");
 MODULE_PARM(snd_id, "1-" __MODULE_STRING(SND_CARDS) "s");
-MODULE_PARM_DESC(snd_id, "ID string for CS4236+ soundcard.");
+MODULE_PARM_DESC(snd_id, "ID string for " IDENT " soundcard.");
 MODULE_PARM(snd_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_port, "Port # for CS4236+ driver.");
+MODULE_PARM_DESC(snd_port, "Port # for " IDENT " driver.");
 MODULE_PARM(snd_cport, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_cport, "Control port # for CS4236+ driver.");
+MODULE_PARM_DESC(snd_cport, "Control port # for " IDENT " driver.");
 MODULE_PARM(snd_mpu_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for CS4236+ driver.");
+MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for " IDENT " driver.");
 MODULE_PARM(snd_fm_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_fm_port, "FM port # for CS4236+ driver.");
+MODULE_PARM_DESC(snd_fm_port, "FM port # for " IDENT " driver.");
 MODULE_PARM(snd_sb_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_sb_port, "SB port # for CS4236+ driver (optional).");
+MODULE_PARM_DESC(snd_sb_port, "SB port # for " IDENT " driver (optional).");
 MODULE_PARM(snd_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_irq, "IRQ # for CS4236+ driver.");
+MODULE_PARM_DESC(snd_irq, "IRQ # for " IDENT " driver.");
 MODULE_PARM(snd_mpu_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_mpu_irq, "MPU-401 IRQ # for CS4232 driver.");
+MODULE_PARM_DESC(snd_mpu_irq, "MPU-401 IRQ # for " IDENT " driver.");
 MODULE_PARM(snd_dma1, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1, "DMA1 # for CS4236+ driver.");
+MODULE_PARM_DESC(snd_dma1, "DMA1 # for " IDENT " driver.");
 MODULE_PARM(snd_dma1_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1_size, "Size of DMA1 # for CS4236+ driver.");
+MODULE_PARM_DESC(snd_dma1_size, "Size of DMA1 # for " IDENT " driver.");
 MODULE_PARM(snd_dma2, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma2, "DMA2 # for CS4236+ driver.");
+MODULE_PARM_DESC(snd_dma2, "DMA2 # for " IDENT " driver.");
 MODULE_PARM(snd_dma2_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma2_size, "Size of DMA2 # for CS4236+ driver.");
-#endif
+MODULE_PARM_DESC(snd_dma2_size, "Size of DMA2 # for " IDENT " driver.");
+MODULE_PARM(snd_isapnp, "1-" __MODULE_STRING(SND_CARDS) "i");
+MODULE_PARM_DESC(snd_isapnp, "Enable/disable ISA PnP for " IDENT " driver.");
 
 static struct snd_card_cs4236 {
 	snd_irq_t *irqptr;
@@ -77,7 +86,7 @@ static struct snd_card_cs4236 {
 	snd_pcm_t *pcm;
 	snd_kmixer_t *mixer;
 	snd_rawmidi_t *rmidi;
-	snd_synth_t *synth;
+	snd_hwdep_t *synth;
 	cs4231_t *codec;
 	unsigned short pcm_status_reg;
 #ifdef CONFIG_ISAPNP
@@ -88,12 +97,39 @@ static struct snd_card_cs4236 {
 } *snd_card_cs4236_cards[SND_CARDS] = SND_DEFAULT_PTR;
 
 #ifdef CONFIG_ISAPNP
-static unsigned int snd_card_cs4236_pnpids[] = {
+#ifdef CS4232
+static unsigned int snd_card_pnpids[] = {
+	/* Turtle Beach TBS-2000 (CS4232) */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x7532),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0xb006), /* MPU */	
+	/* Turtle Beach Tropez Plus (CS4232) */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x7632),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('P','N','P')<<16)|ISAPNP_FUNCTION(0xb006), /* MPU */
+	/* --- */
+	0	/* end */
+};
+#endif
+#ifdef CS4236
+static unsigned int snd_card_pnpids[] = {
+	/* Genius Sound Maker 3DJ - CS4237B */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x0437),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
 	/* some uknown CS4236B */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x0b35),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */	
+	/* HP 6330 Onboard sound */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x1525),	  /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0100), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0110), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0103), /* MPU */
 	/* ACER AW37 - CS4235 */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x4236),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
@@ -104,6 +140,11 @@ static unsigned int snd_card_cs4236_pnpids[] = {
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* Dell Optiplex GX1 - CS4236B */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x6835),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
 	/* Turtle Beach Malibu - CS4237B */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x7537),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
@@ -128,6 +169,7 @@ static unsigned int snd_card_cs4236_pnpids[] = {
 	0	/* end */
 };
 #endif
+#endif
 
 static void snd_card_cs4236_use_inc(snd_card_t * card)
 {
@@ -151,36 +193,42 @@ static snd_pcm_t *snd_card_cs4236_detect(snd_card_t * card,
 {
 	snd_pcm_t *pcm;
 
-	if (snd_register_ioport(card, port, 4, "CS4236+", NULL) < 0)
+	if (snd_register_ioport(card, port, 4, IDENT, NULL) < 0)
 		return NULL;
-	if (snd_register_ioport(card, cport, 8, "CS4236+ - control", NULL) < 0) {
+	if (snd_register_ioport(card, cport, 8, IDENT " - control", NULL) < 0) {
 		snd_unregister_ioports(card);
 		return NULL;
 	}
 	if (mpu_port != SND_AUTO_PORT) {
-		if (snd_register_ioport(card, mpu_port, 2, "CS4236+ - MPU-401", NULL) < 0) {
+		if (snd_register_ioport(card, mpu_port, 2, IDENT " - MPU-401", NULL) < 0) {
 			snd_unregister_ioports(card);
-			snd_printk("ports for CS4236+ MPU-401 are already used, try another\n");
+			snd_printk("ports for " IDENT " MPU-401 are already used, try another\n");
 			return NULL;
 		}
 	}
 	if (fm_port != SND_AUTO_PORT) {
-		if (snd_register_ioport(card, fm_port, 4, "CS4236+ - FM", NULL) < 0) {
+		if (snd_register_ioport(card, fm_port, 4, IDENT " - FM", NULL) < 0) {
 			snd_unregister_ioports(card);
-			snd_printk("ports for CS4236+ FM are already used, try another\n");
+			snd_printk("ports for " IDENT " FM are already used, try another\n");
 			return NULL;
 		}
 	}
 	if (sb_port != SND_AUTO_PORT) {
-		if (snd_register_ioport(card, sb_port, 16, "CS4236+ - SB", NULL) < 0) {
+		if (snd_register_ioport(card, sb_port, 16, IDENT " - SB", NULL) < 0) {
 			snd_unregister_ioports(card);
-			snd_printk("ports for CS4236+ SB are already used, try another\n");
+			snd_printk("ports for " IDENT " SB are already used, try another\n");
 			return NULL;
 		}
 	}
+#ifdef CS4236
 	pcm = snd_cs4236_new_device(card, port, cport,
 				    irqptr, dma1ptr, dma2ptr,
 				    CS4231_HW_DETECT3, 0);
+#else
+	pcm = snd_cs4231_new_device(card, port,
+				    irqptr, dma1ptr, dma2ptr,
+				    CS4231_HW_DETECT, 0);
+#endif
 	if (!pcm) {
 		snd_unregister_ioports(card);
 		return NULL;
@@ -218,7 +266,7 @@ static int snd_card_cs4236_resources(int dev, struct snd_card_cs4236 *acard,
 	static int possible_dmas[] = {-1};
 	int err;
 
-	if ((err = snd_register_interrupt(card, "CS4236+",
+	if ((err = snd_register_interrupt(card, IDENT,
 			snd_irq[dev], SND_IRQ_TYPE_ISA,
 			snd_card_cs4236_interrupt, acard,
 			possible_irqs, &acard->irqptr)) < 0)
@@ -226,25 +274,25 @@ static int snd_card_cs4236_resources(int dev, struct snd_card_cs4236 *acard,
 	acard->mpuirqptr = NULL;
 	if (snd_mpu_port[dev] >= 0 && snd_mpu_port[dev] != SND_AUTO_PORT && snd_mpu_irq[dev] >= 0) {
 		if (snd_register_interrupt(card,
-				"CS4236+ - MPU401", snd_mpu_irq[dev],
+				IDENT " - MPU401", snd_mpu_irq[dev],
 				SND_IRQ_TYPE_ISA,
 				snd_card_cs4236_midi_interrupt,
 				acard, possible_irqs, &acard->mpuirqptr) < 0) {
 			snd_mpu_port[dev] = SND_AUTO_PORT;
-			snd_printk("warning: can't allocate IRQ for MPU401, midi port is disabled\n");
+			snd_printk("warning: cannot allocate IRQ for MPU401, midi port is disabled\n");
 		}
 	} else {
 		snd_mpu_port[dev] = SND_AUTO_PORT;
 		acard->mpuirqptr = NULL;
 	}
 	if ((err = snd_register_dma_channel(card,
-			"CS4236+ - DMA1", snd_dma1[dev],
+			IDENT " - DMA1", snd_dma1[dev],
 			SND_DMA_TYPE_ISA, snd_dma1_size[dev],
 			possible_dmas, &acard->dma1ptr)) < 0)
 		return err;
 	if (snd_dma2[dev] >= 0 && snd_dma2[dev] < 8) {
 		if ((err = snd_register_dma_channel(card,
-				"CS4236+ - DMA2", snd_dma2[dev],
+				IDENT " - DMA2", snd_dma2[dev],
 				SND_DMA_TYPE_ISA, snd_dma2_size[dev],
 				possible_dmas, &acard->dma2ptr)) < 0)
 			return err;
@@ -266,35 +314,35 @@ static int snd_card_cs4236_isapnp(int dev, struct snd_card_cs4236 *acard)
 	idx = 0;
       __again:
       	pdev = NULL;
-	while ((tmp = snd_card_cs4236_pnpids[idx]) != 0) {
+	while ((tmp = snd_card_pnpids[idx]) != 0) {
 		pdev = isapnp_find_device(tmp >> 16, tmp & 0xffff, dev);
 		if (pdev)
 			break;
 		idx += 4;
 	}
 	if (!pdev) {
-		snd_printdd("isapnp failed for CS4236+\n");
+		snd_printdd("isapnp failed for " IDENT "\n");
 		return -ENODEV;
 	}
-	tmp = snd_card_cs4236_pnpids[idx+1];
+	tmp = snd_card_pnpids[idx+1];
 	acard->wss = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
 	if (!acard->wss) {
-		snd_printdd("isapnp failed for CS4236+ (WSS) - %i\n", idx);
+		snd_printdd("isapnp failed for " IDENT " (WSS) - %i\n", idx);
 		idx += 5;
 		goto __again;		/* maybe we have another config */
 	}
-	tmp = snd_card_cs4236_pnpids[idx+2];
+	tmp = snd_card_pnpids[idx+2];
 	acard->ctrl = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
 	if (!acard->ctrl) {
-		snd_printdd("isapnp failed for CS4236+ (CTRL) - %i\n", idx);
+		snd_printdd("isapnp failed for " IDENT " (CTRL) - %i\n", idx);
 		idx += 5;
 		goto __again;		/* maybe we have another config */
 	}
 	if (snd_mpu_port[dev] > 0) {
-		tmp = snd_card_cs4236_pnpids[idx+3];
+		tmp = snd_card_pnpids[idx+3];
 		acard->mpu = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
 		if (!acard->mpu) {
-			snd_printdd("isapnp failed for CS4236+ (MPU) - %i\n", idx);
+			snd_printdd("isapnp failed for " IDENT " (MPU) - %i\n", idx);
 			idx += 5;
 			goto __again;		/* maybe we have another config */
 		}
@@ -320,7 +368,7 @@ static int snd_card_cs4236_isapnp(int dev, struct snd_card_cs4236 *acard)
 	if (snd_dma2[dev] != SND_AUTO_DMA)
 		cfg.dma[1] = snd_dma2[dev] < 0 ? 254 : snd_dma2[dev];
 	if (isapnp_configure(&cfg)<0) {
-		snd_printk("CS4236+ isapnp configure failed (out of resources?)\n");
+		snd_printk(IDENT " isapnp configure failed (out of resources?)\n");
 		isapnp_cfg_end();
 		return -EBUSY;
 	}
@@ -346,7 +394,7 @@ static int snd_card_cs4236_isapnp(int dev, struct snd_card_cs4236 *acard)
 	cfg.port_disable[2] = snd_sb_port[dev];
 	cfg.port_disable_size[2] = 16;
 	if (isapnp_configure(&cfg)<0) {
-		snd_printk("CS4236+ isapnp configure failed (out of resources?)\n");
+		snd_printk(IDENT " isapnp configure failed (out of resources?)\n");
 		isapnp_cfg_end();
 		return -EBUSY;
 	}
@@ -385,7 +433,7 @@ static int snd_card_cs4236_isapnp(int dev, struct snd_card_cs4236 *acard)
 	if (snd_mpu_port[dev] != SND_AUTO_PORT)
 		isapnp_activate(acard->mpu->number);
 	isapnp_cfg_end();
-	return -ENODEV;
+	return 0;
 }
 
 static void snd_card_cs4236_deactivate(struct snd_card_cs4236 *acard)
@@ -407,17 +455,15 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 {
 	snd_card_t *card;
 	snd_pcm_t *pcm = NULL;
-	snd_pcm1_t *pcm1;
 	snd_kmixer_t *mixer = NULL;
 	snd_rawmidi_t *rmidi = NULL;
-	snd_synth_t *synth = NULL;
+	snd_hwdep_t *synth = NULL;
 	cs4231_t *codec;
 
 #ifndef CONFIG_ISAPNP
 	if (snd_port[dev] == SND_AUTO_PORT ||
 	    snd_cport[dev] == SND_AUTO_PORT) {
-		snd_printk("probing for CS4236+ isn't supported\n");
-		snd_printk("port = 0x%x, cport = 0x%x\n", snd_port[dev], snd_cport[dev]);
+		snd_printk("probing for " IDENT " is not supported\n");
 		return -ENODEV;
 	}
 #endif
@@ -425,12 +471,22 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 		       snd_card_cs4236_use_inc, snd_card_cs4236_use_dec);
 	if (!card)
 		return -ENOMEM;
+#ifdef CS4236
 	card->type = SND_CARD_TYPE_CS4236;
+#endif
+#ifdef CS4232
+	card->type = SND_CARD_TYPE_CS4232;
+#endif
 #ifdef CONFIG_ISAPNP
-	if (snd_card_cs4236_isapnp(dev, acard)<0) {
+	if (snd_isapnp[dev]) {
+		if (snd_card_cs4236_isapnp(dev, acard)<0) {
+			snd_printk("isapnp detection failed and probing for " IDENT " is not supported\n");
+			snd_card_free(card);
+			return -EBUSY;
+		}
+	} else {
 		if (snd_port[dev] == SND_AUTO_PORT ||
 		    snd_cport[dev] == SND_AUTO_PORT) {
-			snd_printk("isapnp detection failed and probing for CS4236+ isn't supported\n");
 			snd_card_free(card);
 			return -EBUSY;
 		}
@@ -458,12 +514,15 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 		snd_card_free(card);
 		return -ENODEV;
 	}
-	pcm1 = (snd_pcm1_t *) pcm->private_data;
-	codec = (cs4231_t *) pcm1->private_data;
+	codec = (cs4231_t *) pcm->private_data;
 	acard->codec = codec;
 	acard->pcm_status_reg = codec->port + 2;
 
+#ifdef CS4236
 	mixer = snd_cs4236_new_mixer(pcm, 0);
+#else
+	mixer = snd_cs4231_new_mixer(pcm, 0);
+#endif
 	if (!mixer)
 		goto __nodev;
 
@@ -481,7 +540,7 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 		if (snd_rawmidi_register(rmidi, 0) < 0)
 			goto __nodev;
 	}
-	if (synth && snd_synth_register(synth) < 0) {
+	if (synth && snd_hwdep_register(synth, 0) < 0) {
 		if (rmidi)
 			snd_rawmidi_unregister(rmidi);
 		rmidi = NULL;
@@ -489,7 +548,7 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 	}
 	if (snd_mixer_register(mixer, 0) < 0) {
 		if (synth)
-			snd_synth_unregister(synth);
+			snd_hwdep_unregister(synth);
 		synth = NULL;
 		if (rmidi)
 			snd_rawmidi_unregister(rmidi);
@@ -498,7 +557,7 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 	}
 	if (snd_pcm_register(pcm, 0)) {
 		if (synth)
-			snd_synth_unregister(synth);
+			snd_hwdep_unregister(synth);
 		synth = NULL;
 		if (rmidi)
 			snd_rawmidi_unregister(rmidi);
@@ -507,7 +566,12 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 		mixer = NULL;
 		goto __nodev;
 	}
+#ifdef CS4236
 	strcpy(card->abbreviation, "CS4236");
+#endif
+#ifdef CS4232
+	strcpy(card->abbreviation, "CS4232");
+#endif
 	strcpy(card->shortname, pcm->name);
 	sprintf(card->longname, "%s at 0x%x, irq %i, dma %i",
 		pcm->name,
@@ -525,7 +589,7 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 		acard->synth = synth;
 		return 0;
 	}
-	snd_synth_unregister(synth);
+	snd_hwdep_unregister(synth);
 	snd_mixer_unregister(mixer);
 	snd_pcm_unregister(pcm);
 	snd_card_free(card);
@@ -533,7 +597,7 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 
       __nodev:
 	if (synth)
-		snd_synth_free(synth);
+		snd_hwdep_free(synth);
 	if (rmidi)
 		snd_rawmidi_free(rmidi);
 	if (mixer)
@@ -549,12 +613,9 @@ int init_module(void)
 	int dev, cards;
 	struct snd_card_cs4236 *acard;
 
-#ifndef LINUX_2_1
-	register_symtab(NULL);
-#endif
 	for (dev = cards = 0; dev < SND_CARDS && snd_port[dev] > 0; dev++) {
 		acard = (struct snd_card_cs4236 *)
-				snd_malloc(sizeof(struct snd_card_cs4236));
+				snd_kcalloc(sizeof(struct snd_card_cs4236), GFP_KERNEL);
 		if (!acard)
 			continue;
 		memset(acard, 0, sizeof(struct snd_card_cs4236));
@@ -562,17 +623,17 @@ int init_module(void)
 #ifdef CONFIG_ISAPNP
 			snd_card_cs4236_deactivate(acard);
 #endif
-			snd_free(acard, sizeof(struct snd_card_cs4236));
+			snd_kfree(acard);
 			if (snd_port[dev] == SND_AUTO_PORT)
 				break;
-			snd_printk("CS4236+ soundcard #%i not found at 0x%x or device busy\n", dev + 1, snd_port[dev]);
+			snd_printk(IDENT " soundcard #%i not found at 0x%x or device busy\n", dev + 1, snd_port[dev]);
 			continue;
 		}
 		snd_card_cs4236_cards[dev] = acard;
 		cards++;
 	}
 	if (!cards) {
-		snd_printk("CS4236+ soundcard #%i not found or device busy\n", dev + 1);
+		snd_printk(IDENT " soundcard #%i not found or device busy\n", dev + 1);
 		return -ENODEV;
 	}
 	return 0;
@@ -588,7 +649,7 @@ void cleanup_module(void)
 		if (acard) {
 			snd_card_unregister(acard->card);
 			if (acard->synth)
-				snd_synth_unregister(acard->synth);
+				snd_hwdep_unregister(acard->synth);
 			if (acard->rmidi)
 				snd_rawmidi_unregister(acard->rmidi);
 			if (acard->mixer)
@@ -600,7 +661,7 @@ void cleanup_module(void)
 #ifdef CONFIG_ISAPNP
 			snd_card_cs4236_deactivate(acard);
 #endif
-			snd_free(acard, sizeof(struct snd_card_cs4236));
+			snd_kfree(acard);
 		}
 	}
 }
