@@ -44,14 +44,14 @@
 #define AD1848_AUX2_RIGHT_INPUT	0x05	/* right AUX2 input control */
 #define AD1848_LEFT_OUTPUT	0x06	/* left output control register */
 #define AD1848_RIGHT_OUTPUT	0x07	/* right output control register */
-#define AD1848_DATA_FORMAT	0x08	/* clock and data format - playback/record - bits 7-0 MCE */
+#define AD1848_DATA_FORMAT	0x08	/* clock and data format - playback/capture - bits 7-0 MCE */
 #define AD1848_IFACE_CTRL	0x09	/* interface control - bits 7-2 MCE */
 #define AD1848_PIN_CTRL		0x0a	/* pin control */
 #define AD1848_TEST_INIT	0x0b	/* test and initialization */
 #define AD1848_MISC_INFO	0x0c	/* miscellaneaous information */
 #define AD1848_LOOPBACK		0x0d	/* loopback control */
-#define AD1848_DATA_UPR_CNT	0x0e	/* playback/record upper base count */
-#define AD1848_DATA_LWR_CNT	0x0f	/* playback/record lower base count */
+#define AD1848_DATA_UPR_CNT	0x0e	/* playback/capture upper base count */
+#define AD1848_DATA_LWR_CNT	0x0f	/* playback/capture lower base count */
 
 /* definitions for codec register select port - CODECP( REGSEL ) */
 
@@ -85,12 +85,12 @@
 
 /* definitions for interface control register - AD1848_IFACE_CTRL */
 
-#define AD1848_RECORD_PIO	0x80	/* record PIO enable */
+#define AD1848_CAPTURE_PIO	0x80	/* capture PIO enable */
 #define AD1848_PLAYBACK_PIO	0x40	/* playback PIO enable */
 #define AD1848_CALIB_MODE	0x18	/* calibration mode bits */
 #define AD1848_AUTOCALIB	0x08	/* auto calibrate */
 #define AD1848_SINGLE_DMA	0x04	/* use single DMA channel */
-#define AD1848_RECORD_ENABLE	0x02	/* record enable */
+#define AD1848_CAPTURE_ENABLE	0x02	/* capture enable */
 #define AD1848_PLAYBACK_ENABLE	0x01	/* playback enable */
 
 /* definitions for pin control register - AD1848_PIN_CTRL */
@@ -116,9 +116,9 @@ struct snd_stru_ad1848_freq {
 
 #define AD1848_MODE_NONE	0x0000
 #define AD1848_MODE_PLAY	0x0001
-#define AD1848_MODE_RECORD	0x0002
+#define AD1848_MODE_CAPTURE	0x0002
 #define AD1848_MODE_TIMER	0x0004
-#define AD1848_MODE_OPEN	(AD1848_MODE_PLAY|AD1848_MODE_RECORD|AD1848_MODE_TIMER)
+#define AD1848_MODE_OPEN	(AD1848_MODE_PLAY|AD1848_MODE_CAPTURE|AD1848_MODE_TIMER)
 
 /* defines for codec.hardware */
 
@@ -126,6 +126,7 @@ struct snd_stru_ad1848_freq {
 #define AD1848_HW_AD1847	0x0001	/* AD1847 chip */
 #define AD1848_HW_AD1848	0x0002	/* AD1848 chip */
 #define AD1848_HW_CS4248	0x0003	/* CS4248 chip */
+#define AD1848_HW_CMI8330	0x0004	/* CMI8330 chip */
 
 struct snd_stru_ad1848 {
 	unsigned short port;		/* i/o port */
@@ -139,6 +140,10 @@ struct snd_stru_ad1848 {
 	unsigned short single_dma:1;	/* forced single DMA mode (GUS 16-bit daughter board) or dma1 == dma2 */
 
 	snd_pcm_t *pcm;
+	snd_pcm_subchn_t *playback_subchn;
+	snd_pcm1_subchn_t *playback_subchn1;
+	snd_pcm_subchn_t *capture_subchn;
+	snd_pcm1_subchn_t *capture_subchn1; 
 	snd_card_t *card;
 	snd_kmixer_t *mixer;
 
@@ -146,20 +151,36 @@ struct snd_stru_ad1848 {
 	int mce_bit;
 	int calibrate_mute;
 
+	spinlock_t reg_lock;
+	struct semaphore open_mutex;
+
 	snd_kmixer_element_t *me_mux_line1;
 	snd_kmixer_element_t *me_mux_aux1;
 	snd_kmixer_element_t *me_mux_line2;
 	snd_kmixer_element_t *me_mux_mix;
 
-	snd_spin_define(reg);
-	snd_mutex_define(open);
-	snd_sleep_define(mce);
+	snd_kmixer_element_t *me_mux;
+	snd_kmixer_element_t *me_accu;
+	snd_kmixer_element_t *me_dig_accu;
+	snd_kmixer_element_t *me_vol_aux1;
+	snd_kmixer_element_t *me_sw_aux1;
+	snd_kmixer_element_t *me_vol_aux2;
+	snd_kmixer_element_t *me_sw_aux2;
+	snd_kmixer_element_t *me_vol_igain;
+	snd_kmixer_element_t *me_capture;
+	snd_kmixer_element_t *me_vol_loop;
+	snd_kmixer_element_t *me_sw_loop;
+	snd_kmixer_element_t *me_playback;
+	snd_kmixer_element_t *me_vol_pcm;
+	snd_kmixer_element_t *me_sw_pcm;
 };
 
 typedef struct snd_stru_ad1848 ad1848_t;
 
 /* exported functions */
 
+void snd_ad1848_out(ad1848_t * codec, unsigned char reg, unsigned char value);
+
 void snd_ad1848_interrupt(snd_pcm_t * pcm, unsigned char status);
 
 extern snd_pcm_t *snd_ad1848_new_device(snd_card_t * card,
@@ -170,18 +191,18 @@ extern snd_pcm_t *snd_ad1848_new_device(snd_card_t * card,
 
 snd_kmixer_t *snd_ad1848_new_mixer(snd_pcm_t * pcm, int pcm_dev);
 
-int snd_ad1848_mixer_stereo_volume(int w_flag, int *voices, ad1848_t *codec,
+int snd_ad1848_mixer_stereo_volume(void *private_data, int w_flag, int *voices,
 					int bit, int invert, int shift,
 					unsigned char left_reg,
 					unsigned char right_reg);
-int snd_ad1848_mixer_mono_volume(int w_flag, int *voices, ad1848_t *codec,
+int snd_ad1848_mixer_mono_volume(void *private_data, int w_flag, int *voices,
 					int bit, int invert, int shift,
 					unsigned char reg);
-int snd_ad1848_mixer_stereo_switch(int w_flag, unsigned int *bitmap, ad1848_t *codec,
+int snd_ad1848_mixer_stereo_switch(void *private_data, int w_flag, unsigned int *bitmap,
 					int bit, int invert,
 					unsigned char left_reg,
 					unsigned char right_reg);
-int snd_ad1848_mixer_mono_switch(int w_flag, unsigned int *bitmap, ad1848_t *codec,
+int snd_ad1848_mixer_mono_switch(void *private_data, int w_flag, int *value,
 					int bit, int invert, unsigned char reg);
 
 #ifdef CONFIG_SND_DEBUG
