@@ -36,6 +36,7 @@
 
 	/* system messages */
 #define SND_SEQ_EVENT_SYSTEM		0
+#define SND_SEQ_EVENT_RESULT		1
 
 	/* note messages */
 #define SND_SEQ_EVENT_NOTE		1
@@ -80,37 +81,38 @@
 #define SND_SEQ_EVENT_PORT_EXIT		64	/* port was deleted from system */
 #define SND_SEQ_EVENT_PORT_CHANGE	65	/* port status/info has changed */
 
+#define SND_SEQ_EVENT_OSS		70	/* OSS raw event */
+#define SND_SEQ_EVENT_NORMAL_CONTROLS	80
+
 	/* synthesizer events */	
 #define SND_SEQ_EVENT_SAMPLE		80	/* sample select */
-#define SND_SEQ_EVENT_SAMPLE_START	81	/* voice start */
-#define SND_SEQ_EVENT_SAMPLE_STOP	82	/* voice stop */
-#define SND_SEQ_EVENT_SAMPLE_FREQ	83	/* playback frequency */
-#define SND_SEQ_EVENT_SAMPLE_VOLUME	84	/* volume and balance */
-#define SND_SEQ_EVENT_SAMPLE_LOOP	85	/* sample loop */
-#define SND_SEQ_EVENT_SAMPLE_POSITION	86	/* sample position */
+#define SND_SEQ_EVENT_SAMPLE_CLUSTER	81	/* sample cluster select */
+#define SND_SEQ_EVENT_SAMPLE_START	82	/* voice start */
+#define SND_SEQ_EVENT_SAMPLE_STOP	83	/* voice stop */
+#define SND_SEQ_EVENT_SAMPLE_FREQ	84	/* playback frequency */
+#define SND_SEQ_EVENT_SAMPLE_VOLUME	85	/* volume and balance */
+#define SND_SEQ_EVENT_SAMPLE_LOOP	86	/* sample loop */
+#define SND_SEQ_EVENT_SAMPLE_POSITION	87	/* sample position */
+#define SND_SEQ_EVENT_SAMPLE_PRIVATE1	88	/* private (hardware dependent) event */
 
 	/* instrument layer */
 #define SND_SEQ_EVENT_INSTR_BEGIN	100	/* begin of instrument management */
-#define SND_SEQ_EVENT_INSTR_BEGIN_RESULT 101	/* result */
 #define SND_SEQ_EVENT_INSTR_END		102	/* end of instrument management */
-#define SND_SEQ_EVENT_INSTR_END_RESULT	103	/* result */
-#define SND_SEQ_EVENT_INSTR_PUT		104	/* put instrument to port */
-#define SND_SEQ_EVENT_INSTR_PUT_RESULT	105	/* result */
-#define SND_SEQ_EVENT_INSTR_GET		106	/* get instrument from port */
-#define SND_SEQ_EVENT_INSTR_GET_RESULT	107	/* result */
-#define SND_SEQ_EVENT_INSTR_FREE	108	/* free instrument(s) */
-#define SND_SEQ_EVENT_INSTR_FREE_RESULT	109	/* result */
-#define SND_SEQ_EVENT_INSTR_LIST	110	/* instrument list */
-#define SND_SEQ_EVENT_INSTR_LIST_RESULT 111	/* result */
-#define SND_SEQ_EVENT_INSTR_RESET	112	/* reset instrument memory */
-#define SND_SEQ_EVENT_INSTR_RESET_RESULT 113	/* result */
-#define SND_SEQ_EVENT_INSTR_INFO	114	/* instrument interface info */
-#define SND_SEQ_EVENT_INSTR_INFO_RESULT 115	/* result */
-#define SND_SEQ_EVENT_INSTR_STATUS	116	/* instrument interface status */
-#define SND_SEQ_EVENT_INSTR_STATUS_RESULT 117	/* result */
-#define SND_SEQ_EVENT_INSTR_CLUSTER	118	/* cluster parameters */
-#define SND_SEQ_EVENT_INSTR_CLUSTER_RESULT 119	/* result */
-#define SND_SEQ_EVENT_INSTR_CHANGE	120	/* instrument change */
+#define SND_SEQ_EVENT_INSTR_PUT		103	/* put instrument to port */
+#define SND_SEQ_EVENT_INSTR_GET		104	/* get instrument from port */
+#define SND_SEQ_EVENT_INSTR_GET_RESULT	105	/* result */
+#define SND_SEQ_EVENT_INSTR_FREE	106	/* free instrument(s) */
+#define SND_SEQ_EVENT_INSTR_LIST	107	/* instrument list */
+#define SND_SEQ_EVENT_INSTR_LIST_RESULT 108	/* result */
+#define SND_SEQ_EVENT_INSTR_RESET	109	/* reset instrument memory */
+#define SND_SEQ_EVENT_INSTR_INFO	110	/* instrument interface info */
+#define SND_SEQ_EVENT_INSTR_INFO_RESULT 111	/* result */
+#define SND_SEQ_EVENT_INSTR_STATUS	112	/* instrument interface status */
+#define SND_SEQ_EVENT_INSTR_STATUS_RESULT 113	/* result */
+#define SND_SEQ_EVENT_INSTR_CLUSTER	114	/* cluster parameters */
+#define SND_SEQ_EVENT_INSTR_CLUSTER_GET	115	/* get cluster parameters */
+#define SND_SEQ_EVENT_INSTR_CLUSTER_RESULT 116	/* result */
+#define SND_SEQ_EVENT_INSTR_CHANGE	117	/* instrument change */
 
 	/* hardware specific events - range 192-255 */
 
@@ -153,6 +155,7 @@ typedef struct {
 #define SND_SEQ_DEST_DIRECT		(1<<5)	/* bypass enqueing */
 #define SND_SEQ_DEST_MASK		(1<<5)
 
+
 	/* note event */
 typedef struct {
 	unsigned char note;
@@ -160,57 +163,93 @@ typedef struct {
 	unsigned int duration;
 } snd_seq_ev_note;
 
-
 	/* controller event */
 typedef struct {
 	unsigned int param;
 	signed int value;
 } snd_seq_ev_ctrl;
 
-
 	/* generic set of bytes (8x8 bit) */
 typedef struct {
 	unsigned char d[8];	/* 8 bit value */
 } snd_seq_ev_raw8;
 
-
 	/* generic set of integers (2x32 bit) */
 typedef struct {
 	unsigned int d[2];	/* 32 bit value */
 } snd_seq_ev_raw32;
 
-
 	/* external stored data */
 typedef struct {
 	int len;		/* length of data */
 	void *ptr;		/* pointer to data (note: maybe 64-bit) */
 } snd_seq_ev_ext;
 
-
 	/* external stored data - IPC shared memory */
 typedef struct {
 	int len;		/* length of data */
 	key_t ipc;		/* IPC key */
 } snd_seq_ev_ipcshm;
 
-
 /* Instrument cluster type */
 typedef unsigned long snd_seq_instr_cluster_t;
 
 /* Instrument type */
 typedef struct {
 	snd_seq_instr_cluster_t cluster;
-	unsigned int std;
+	unsigned int std;		/* the upper byte means a private instrument (owner - client #) */
 	unsigned short bank;
 	unsigned short prg;
 } snd_seq_instr_t;
 
+	/* sample number */
+typedef struct {
+	unsigned int std;
+	unsigned short bank;
+	unsigned short prg;
+} snd_seq_ev_sample;
+
+	/* sample cluster */
+typedef struct {
+	snd_seq_instr_cluster_t cluster;
+} snd_seq_ev_cluster;
+
+	/* sample position */
+typedef unsigned int snd_seq_position_t; /* playback position (in samples) * 16 */
+
+	/* sample stop mode */
+typedef enum {
+	SAMPLE_STOP_IMMEDIATELY = 0,	/* terminate playing immediately */
+	SAMPLE_STOP_VENVELOPE = 1,	/* finish volume envelope */
+	SAMPLE_STOP_LOOP = 2		/* terminate loop and finish wave */
+} snd_seq_stop_mode_t;
+
+	/* sample frequency */
+typedef int snd_seq_frequency_t; /* playback frequency in HZ * 16 */
+
+	/* sample volume control; if any value is set to -1 == do not change */
+typedef struct {
+	signed short volume;	/* range: 0-16383 */
+	signed short lr;	/* left-right balance; range: 0-16383 */
+	signed short fr;	/* front-rear balance; range: 0-16383 */
+	signed short du;	/* down-up balance; range: 0-16383 */
+} snd_seq_ev_volume;
+
+	/* simple loop redefinition */
+typedef struct {
+	unsigned int start;	/* loop start (in samples) * 16 */
+	unsigned int end;	/* loop end (in samples) * 16 */
+} snd_seq_ev_loop;
 
 /* INSTR_BEGIN event */
 typedef struct {
 	long timeout;	/* zero = forever, otherwise timeout in ms */
 } snd_seq_ev_instr_begin_t;
 
+typedef struct {
+	int event;		/* processed event type */
+	int result;
+} snd_seq_result_t;
 
 typedef struct {
 	long int tv_sec;	/* seconds */
@@ -248,8 +287,15 @@ typedef struct snd_seq_event_t {
 			snd_seq_real_time_t real;
 		} time;
 		snd_seq_addr_t addr;
-		int result;
+		snd_seq_result_t result;
 		snd_seq_ev_instr_begin_t instr_begin;
+		snd_seq_ev_sample sample;
+		snd_seq_ev_cluster cluster;
+		snd_seq_position_t position;
+		snd_seq_stop_mode_t stop_mode;
+		snd_seq_frequency_t frequency;
+		snd_seq_ev_volume volume;
+		snd_seq_ev_loop loop;
 	} data;
 } snd_seq_event_t;
 
@@ -259,6 +305,7 @@ typedef struct {
 	int clients;			/* maximum clients count */
 	int ports;			/* maximum ports per client */
 	int channels;			/* maximum channels per port */
+	char reserved[32];
 } snd_seq_system_info_t;
 
 
@@ -289,6 +336,29 @@ typedef struct {
 } snd_seq_client_info_t;
 
 
+/* client pool size */
+typedef struct {
+	int client;			/* client number to inquire */
+	int output_pool;		/* outgoing (write) pool size */
+	int input_pool;			/* incoming (read) pool size */
+	int output_room;		/* minimum free pool size for select/blocking mode */
+	int output_free;		/* unused size */
+	int input_free;			/* unused size */
+	char reserved[64];
+} snd_seq_client_pool_t;
+
+
+/* reset read/write pool */
+#define SND_SEQ_RESET_POOL_NONE		0	/* do not remove events */
+#define SND_SEQ_RESET_POOL_ALL		1	/* purge all events in pool */
+
+typedef struct {
+	int reset_output;		/* purge write pool */
+	snd_seq_addr_t output_match;	/* matching client/port */
+	int reset_input;		/* purge read pool */ 
+	snd_seq_addr_t input_match;	/* matching client/port */
+} snd_seq_reset_pool_t;
+
 	/* known port numbers */
 #define SND_SEQ_PORT_SYSTEM_TIMER	0
 #define SND_SEQ_PORT_SYSTEM_ANNOUNCE	1
@@ -327,6 +397,7 @@ typedef struct {
 	unsigned int capability;	/* port capability bits */
 	unsigned int type;		/* port type bits */
 	int midi_channels;		/* channels per MIDI port */
+	int midi_voices;		/* voices per MIDI port */
 	int synth_voices;		/* voices per SYNTH port */
 
 	int out_use;			/* R/O: subscribers for output (this port->sequencer) */
@@ -433,8 +504,6 @@ typedef struct {
 	int client;			/* sequencer client */
 	int used;			/* queue is used with this client (must be set for accepting events) */
 	/* per client watermarks */
-	int low;			/* low watermark for wakeup (minimum free events in queue) */
-	int high;			/* high watermark for wakeup (maximum used events in queue) */
 	char reserved[64];		/* for future use */
 } snd_seq_queue_client_t;
 
@@ -443,6 +512,9 @@ typedef struct {
 	snd_seq_addr_t dest;		/* destination address */
 	int exclusive: 1,		/* exclusive mode */
 	    realtime: 1;		/* realtime timestamp */
+	int midi_channels;		/* midi channels setup, zero = do not care */
+	int midi_voices;		/* midi voices setup, zero = do not care */
+	int synth_voices;		/* synth voices setup, zero = do not care */
 	char reserved[32];		/* for future use */
 } snd_seq_port_subscribe_t;
 
@@ -459,6 +531,7 @@ typedef struct {
 /* instrument ASCII identifiers */
 #define SND_SEQ_INSTR_ID_DLS1		"DLS1"
 #define SND_SEQ_INSTR_ID_DLS2		"DLS2"
+#define SND_SEQ_INSTR_ID_SIMPLE		"Simple Wave"
 #define SND_SEQ_INSTR_ID_SOUNDFONT	"SoundFont"
 #define SND_SEQ_INSTR_ID_GUS_PATCH	"GUS Patch"
 #define SND_SEQ_INSTR_ID_INTERWAVE	"InterWave FFFF"
@@ -469,13 +542,34 @@ typedef struct {
 /* instrument types */
 #define SND_SEQ_INSTR_TYPE0_DLS1	(1<<0)	/* MIDI DLS v1 */
 #define SND_SEQ_INSTR_TYPE0_DLS2	(1<<1)	/* MIDI DLS v2 */
-#define SND_SEQ_INSTR_TYPE1_SOUNDFONT	(1<<0)	/* EMU SoundFont */
-#define SND_SEQ_INSTR_TYPE1_GUS_PATCH	(1<<1)	/* Gravis UltraSound Patch */
-#define SND_SEQ_INSTR_TYPE1_INTERWAVE	(1<<2)	/* InterWave FFFF */
+#define SND_SEQ_INSTR_TYPE1_SIMPLE	(1<<0)	/* Simple Wave */
+#define SND_SEQ_INSTR_TYPE1_SOUNDFONT	(1<<1)	/* EMU SoundFont */
+#define SND_SEQ_INSTR_TYPE1_GUS_PATCH	(1<<2)	/* Gravis UltraSound Patch */
+#define SND_SEQ_INSTR_TYPE1_INTERWAVE	(1<<3)	/* InterWave FFFF */
 #define SND_SEQ_INSTR_TYPE2_OPL2	(1<<0)	/* Yamaha OPL2 */
 #define SND_SEQ_INSTR_TYPE2_OPL3	(1<<1)	/* Yamaha OPL3 */
 #define SND_SEQ_INSTR_TYPE2_OPL4	(1<<2)	/* Yamaha OPL4 */
 
+/* put commands */
+#define SND_SEQ_INSTR_PUT_CMD_CREATE	0
+#define SND_SEQ_INSTR_PUT_CMD_REPLACE	1
+#define SND_SEQ_INSTR_PUT_CMD_MODIFY	2
+#define SND_SEQ_INSTR_PUT_CMD_ADD	3
+#define SND_SEQ_INSTR_PUT_CMD_REMOVE	4
+
+/* get commands */
+#define SND_SEQ_INSTR_GET_CMD_FULL	0
+#define SND_SEQ_INSTR_GET_CMD_PARTIAL	1
+
+/* query flags */
+#define SND_SEQ_INSTR_QUERY_FOLLOW_ALIAS (1<<0)
+
+/* free commands */
+#define SND_SEQ_INSTR_FREE_CMD_ALL	0
+#define SND_SEQ_INSTR_FREE_CMD_PRIVATE	1
+#define SND_SEQ_INSTR_FREE_CMD_CLUSTER	2
+#define SND_SEQ_INSTR_FREE_CMD_SINGLE	3
+
 /* instrument data */
 typedef struct {
 	char name[32];			/* instrument name */
@@ -491,6 +585,7 @@ typedef struct {
 
 typedef struct {
 	snd_seq_instr_t id;		/* instrument identifier */
+	int cmd;			/* put command */
 	char reserved[16];		/* for the future */
 	long len;			/* real instrument data length (without header) */
 	snd_seq_instr_data_t data;	/* instrument data */
@@ -500,9 +595,10 @@ typedef struct {
 
 typedef struct {
 	snd_seq_instr_t id;		/* instrument identifier */
+	unsigned int flags;		/* query flags */
+	int cmd;			/* query command */
 	char reserved[16];		/* reserved for the future use */
 	long len;			/* real instrument data length (without header) */
-	snd_seq_instr_data_t data;	/* instrument data */
 } snd_seq_instr_get_t;
 
 typedef struct {
@@ -517,7 +613,7 @@ typedef struct {
 
 typedef struct {
 	char reserved[16];		/* for the future use */
-	unsigned int type;              /* SND_SEQ_INSTR_FTYPE_* */
+	unsigned int cmd;               /* SND_SEQ_INSTR_FREE_CMD_* */
 	union {
 		snd_seq_instr_t instr;
 		snd_seq_instr_cluster_t cluster;
@@ -594,5 +690,8 @@ typedef struct {
 #define SND_SEQ_IOCTL_SET_QUEUE_SYNC	_IOW ('S', 0x48, snd_seq_queue_sync_t)
 #define SND_SEQ_IOCTL_GET_QUEUE_CLIENT	_IOWR('S', 0x49, snd_seq_queue_client_t)
 #define SND_SEQ_IOCTL_SET_QUEUE_CLIENT	_IOW ('S', 0x4a, snd_seq_queue_client_t)
+#define SND_SEQ_IOCTL_GET_CLIENT_POOL	_IOWR('S', 0x4b, snd_seq_client_pool_t)
+#define SND_SEQ_IOCTL_SET_CLIENT_POOL	_IOW ('S', 0x4c, snd_seq_client_pool_t)
+#define SND_SEQ_IOCTL_RESET_POOL	_IOW ('S', 0x4d, snd_seq_reset_pool_t)
 
 #endif /* __SND_ASEQUENCER_H */
