@@ -2,7 +2,7 @@
  *  Advanced Linux Sound Architecture - ALSA - Driver
  *
  *  The interface file between the ALSA driver & the user space
- *  Copyright (c) 1994-98 by Jaroslav Kysela <perex@suse.cz>
+ *  Copyright (c) 1994-99 by Jaroslav Kysela <perex@suse.cz>
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -42,14 +42,14 @@
  *  protocol version
  */
 
-#define SND_PROTOCOL_VERSION( major, minor, subminor ) (((major)<<16)|((minor)<<8)|(subminor))
-#define SND_PROTOCOL_MAJOR( version ) (((version)>>16)&0xffff)
-#define SND_PROTOCOL_MINOR( version ) (((version)>>8)&0xff)
-#define SND_PROTOCOL_SUBMINOR( version ) ((version)&0xff)
-#define SND_PROTOCOL_UNCOMPATIBLE( kversion, uversion ) \
-	( SND_PROTOCOL_MAJOR(kversion) != SND_PROTOCOL_MAJOR(uversion) || \
-	 ( SND_PROTOCOL_MAJOR(kversion) == SND_PROTOCOL_MAJOR(uversion) && \
-	   SND_PROTOCOL_MINOR(kversion) != SND_PROTOCOL_MINOR(uversion) ) )
+#define SND_PROTOCOL_VERSION(major, minor, subminor) (((major)<<16)|((minor)<<8)|(subminor))
+#define SND_PROTOCOL_MAJOR(version) (((version)>>16)&0xffff)
+#define SND_PROTOCOL_MINOR(version) (((version)>>8)&0xff)
+#define SND_PROTOCOL_MICRO(version) ((version)&0xff)
+#define SND_PROTOCOL_INCOMPATIBLE(kversion, uversion) \
+	(SND_PROTOCOL_MAJOR(kversion) != SND_PROTOCOL_MAJOR(uversion) || \
+	 (SND_PROTOCOL_MAJOR(kversion) == SND_PROTOCOL_MAJOR(uversion) && \
+	   SND_PROTOCOL_MINOR(kversion) != SND_PROTOCOL_MINOR(uversion)))
 
 /*
  *  hardware independent conversion
@@ -113,7 +113,7 @@ typedef struct snd_switch {
  *                                                                          *
  ****************************************************************************/
 
-#define SND_CTL_VERSION			SND_PROTOCOL_VERSION( 1, 0, 0 )
+#define SND_CTL_VERSION			SND_PROTOCOL_VERSION(2, 0, 0)
 
 #define SND_CTL_SW_JOYSTICK		"Joystick"
 #define SND_CTL_SW_JOYSTICK_ADDRESS	"Joystick Address"
@@ -121,6 +121,7 @@ typedef struct snd_switch {
 
 typedef struct snd_ctl_hw_info {
 	unsigned int type;	/* type of card - look to SND_CARD_TYPE_XXXX */
+	unsigned int hwdepdevs;	/* count of hardware dependent devices (0 to N) */
 	unsigned int pcmdevs;	/* count of PCM devices (0 to N) */
 	unsigned int mixerdevs;	/* count of MIXER devices (0 to N) */
 	unsigned int mididevs;	/* count of raw MIDI devices (0 to N) */
@@ -138,6 +139,8 @@ typedef struct snd_ctl_hw_info {
 #define SND_CTL_IOCTL_SWITCH_LIST	_IOWR('U', 0x02, snd_switch_list_t)
 #define SND_CTL_IOCTL_SWITCH_READ	_IOWR('U', 0x03, snd_switch_t)
 #define SND_CTL_IOCTL_SWITCH_WRITE	_IOWR('U', 0x04, snd_switch_t)
+#define SND_CTL_IOCTL_HWDEP_DEVICE	_IOWR('U', 0x08, int)
+#define SND_CTL_IOCTL_HWDEP_INFO	_IOR ('U', 0x09, snd_hwdep_info_t)
 #define SND_CTL_IOCTL_MIXER_DEVICE	_IOWR('U', 0x10, int)
 #define SND_CTL_IOCTL_MIXER_INFO	_IOR ('U', 0x10, snd_mixer_info_t)
 #define SND_CTL_IOCTL_MIXER_SWITCH_LIST	_IOWR('U', 0x11, snd_switch_list_t)
@@ -146,7 +149,7 @@ typedef struct snd_ctl_hw_info {
 #define SND_CTL_IOCTL_PCM_DEVICE	_IOWR('U', 0x20, int)
 #define SND_CTL_IOCTL_PCM_INFO		_IOR ('U', 0x21, snd_pcm_info_t)
 #define SND_CTL_IOCTL_PCM_PLAYBACK_INFO	_IOR ('U', 0x22, snd_pcm_playback_info_t)
-#define SND_CTL_IOCTL_PCM_RECORD_INFO	_IOR ('U', 0x23, snd_pcm_record_info_t)
+#define SND_CTL_IOCTL_PCM_CAPTURE_INFO	_IOR ('U', 0x23, snd_pcm_capture_info_t)
 #define SND_CTL_IOCTL_PCM_PSWITCH_LIST	_IOWR('U', 0x24, snd_switch_list_t)
 #define SND_CTL_IOCTL_PCM_PSWITCH_READ  _IOWR('U', 0x25, snd_switch_t)
 #define SND_CTL_IOCTL_PCM_PSWITCH_WRITE _IOWR('U', 0x26, snd_switch_t)
@@ -177,7 +180,7 @@ typedef struct snd_ctl_hw_info {
 #define SND_CTL_IFACE_CONTROL		0
 #define SND_CTL_IFACE_MIXER		1
 #define SND_CTL_IFACE_PCM_PLAYBACK	2
-#define SND_CTL_IFACE_PCM_RECORD	3
+#define SND_CTL_IFACE_PCM_CAPTURE	3
 #define SND_CTL_IFACE_RAWMIDI_OUTPUT	4
 #define SND_CTL_IFACE_RAWMIDI_INPUT	5
 
@@ -192,13 +195,41 @@ typedef struct snd_ctl_read {
 	} data;
 } snd_ctl_read_t;
 
+/****************************************************************************
+ *                                                                          *
+ *        Section for driver control interface - /dev/snd/control?          *
+ *                                                                          *
+ ****************************************************************************/
+
+#define SND_HWDEP_VERSION		SND_PROTOCOL_VERSION(1, 0, 0)
+
+#define SND_HWDEP_TYPE_OPL2		0
+#define SND_HWDEP_TYPE_OPL3		1
+#define SND_HWDEP_TYPE_OPL4		2
+#define SND_HWDEP_TYPE_EMU8000		3
+#define SND_HWDEP_TYPE_YSS225           4      /* Yamaha FX processor */
+#define SND_HWDEP_TYPE_ICS2115          5      /* Wavetable synth */
+/* --- */
+#define SND_HWDEP_TYPE_LAST             5
+
+typedef struct snd_hwdep_info {
+	unsigned int type;	/* type of card - look to SND_CARD_TYPE_XXXX */
+	unsigned char id[32];	/* ID of this hardware dependent device */
+	unsigned char name[80];	/* name of this hardware dependent device */
+	unsigned int hw_type;	/* hardware depedent device type */
+	unsigned char reserved[64];	/* reserved for future */
+} snd_hwdep_info_t;
+
+#define SND_HWDEP_IOCTL_PVERSION	_IOR ('H', 0x00, int)
+#define SND_HWDEP_IOCTL_INFO		_IOR ('H', 0x01, snd_hwdep_info_t)
+
 /****************************************************************************
  *                                                                          *
  *                  MIXER interface - /dev/snd/mixer??                      *
  *                                                                          *
  ****************************************************************************/
 
-#define SND_MIXER_VERSION		SND_PROTOCOL_VERSION(2, 0, 0)
+#define SND_MIXER_VERSION		SND_PROTOCOL_VERSION(2, 1, 1)
 
 /* inputs */				/* max 24 chars */
 #define SND_MIXER_IN_SYNTHESIZER	"Synth"
@@ -215,6 +246,9 @@ typedef struct snd_ctl_read {
 #define SND_MIXER_IN_MONO		"Mono"
 #define SND_MIXER_IN_SPEAKER		"PC Speaker"
 #define SND_MIXER_IN_AUX		"Aux"
+#define SND_MIXER_IN_CENTER		"Center Input"
+#define SND_MIXER_IN_WOOFER		"Woofer Input"
+#define SND_MIXER_IN_SURROUND		"Surround Input"
 
 /* outputs */				/* max 24 chars */
 #define SND_MIXER_OUT_MASTER		"Master"
@@ -222,11 +256,14 @@ typedef struct snd_ctl_read {
 #define SND_MIXER_OUT_MASTER_DIGITAL	"Master Digital"
 #define SND_MIXER_OUT_HEADPHONE		"Headphone"
 #define SND_MIXER_OUT_PHONE		"Phone Output"
+#define SND_MIXER_OUT_CENTER		"Center"
+#define SND_MIXER_OUT_WOOFER		"Woofer"
 #define SND_MIXER_OUT_SURROUND		"Surround"
 #define SND_MIXER_OUT_DSP		"DSP Output"
 
 /* groups */				/* max 24 chars */
-#define SND_MIXER_GRP_TONE_CONTROL	"Tone Control"
+#define SND_MIXER_GRP_BASS		"Bass"
+#define SND_MIXER_GRP_TREBLE		"Treble"
 #define SND_MIXER_GRP_EQUALIZER		"Equalizer"
 #define SND_MIXER_GRP_FADER		"Fader"
 #define SND_MIXER_GRP_EFFECT		"Effect"
@@ -238,6 +275,7 @@ typedef struct snd_ctl_read {
 #define SND_MIXER_GRP_DIGITAL_LOOPBACK	"Digital Loopback"
 
 /* others */
+#define SND_MIXER_ELEMENT_TONE_CONTROL	"Tone Control"
 #define SND_MIXER_ELEMENT_INPUT_MUX	"Input MUX"
 #define SND_MIXER_ELEMENT_DIGITAL_ACCU	"Digital Accumulator"
 #define SND_MIXER_ELEMENT_OUTPUT_ACCU	"Output Accumulator"
@@ -311,6 +349,7 @@ typedef struct snd_ctl_read {
 #define SND_MIXER_VOICE_CENTER		4
 #define SND_MIXER_VOICE_REAR_LEFT	5
 #define SND_MIXER_VOICE_REAR_RIGHT	6
+#define SND_MIXER_VOICE_WOOFER		7
 
 typedef struct {
 	unsigned short voice: 15,
@@ -378,12 +417,58 @@ typedef struct snd_mixer_groups {
 	snd_mixer_gid_t *pgroups; /* array */
 } snd_mixer_groups_t;
 
+typedef enum {
+	SND_MIXER_CHN_FRONT_LEFT,
+	SND_MIXER_CHN_FRONT_RIGHT,
+	SND_MIXER_CHN_FRONT_CENTER,
+	SND_MIXER_CHN_REAR_LEFT,
+	SND_MIXER_CHN_REAR_RIGHT,
+	SND_MIXER_CHN_WOOFER,
+	SND_MIXER_CHN_LAST = 31,
+	SND_MIXER_CHN_MONO = SND_MIXER_CHN_FRONT_LEFT
+} snd_mixer_channel_t;
+
+#define SND_MIXER_CHN_MASK_MONO		(1<<SND_MIXER_CHN_MONO)
+#define SND_MIXER_CHN_MASK_FRONT_LEFT	(1<<SND_MIXER_CHN_FRONT_LEFT)
+#define SND_MIXER_CHN_MASK_FRONT_RIGHT	(1<<SND_MIXER_CHN_FRONT_RIGHT)
+#define SND_MIXER_CHN_MASK_FRONT_CENTER	(1<<SND_MIXER_CHN_FRONT_CENTER)
+#define SND_MIXER_CHN_MASK_REAR_LEFT	(1<<SND_MIXER_CHN_REAR_LEFT)
+#define SND_MIXER_CHN_MASK_REAR_RIGHT	(1<<SND_MIXER_CHN_REAR_RIGHT)
+#define SND_MIXER_CHN_MASK_WOOFER	(1<<SND_MIXER_CHN_WOOFER)
+#define SND_MIXER_CHN_MASK_STEREO	(SND_MIXER_CHN_MASK_FRONT_LEFT|SND_MIXER_CHN_MASK_FRONT_RIGHT)
+
+#define SND_MIXER_GRPCAP_VOLUME		(1<<0)
+#define SND_MIXER_GRPCAP_JOINTLY_VOLUME	(1<<1)
+#define SND_MIXER_GRPCAP_MUTE		(1<<2)
+#define SND_MIXER_GRPCAP_JOINTLY_MUTE	(1<<3)
+#define SND_MIXER_GRPCAP_CAPTURE	(1<<4)
+#define SND_MIXER_GRPCAP_JOINTLY_CAPTURE (1<<5)
+#define SND_MIXER_GRPCAP_EXCL_CAPTURE	(1<<6)
+
 typedef struct snd_mixer_group {
 	snd_mixer_gid_t gid;
-	int elements_size;	/* size in element identifiers */
-	int elements;		/* count of filled element identifiers */
-	int elements_over;	/* missing element identifiers */
-	snd_mixer_eid_t *pelements; /* array */
+	int elements_size;		/* size in element identifiers */
+	int elements;			/* count of filled element identifiers */
+	int elements_over;		/* missing element identifiers */
+	snd_mixer_eid_t *pelements;	/* array */
+	unsigned int caps;		/* capabilities */
+	unsigned int channels;		/* bitmap of active channels */	
+	unsigned int mute;		/* bitmap of muted channels */
+	unsigned int capture;		/* bitmap of capture channels */
+	int capture_group;		/* capture group (for exclusive capture source) */
+	int min;			/* minimum value */
+	int max;			/* maximum value */
+	union {
+		struct {
+			int front_left;		/* front left value */
+			int front_right;	/* front right value */
+			int front_center;	/* front center */
+			int rear_left;		/* left rear */
+			int rear_right;		/* right rear */
+			int woofer;		/* woofer */
+		} names;
+		int values[32];
+	} volume;
 } snd_mixer_group_t;
 
 /*
@@ -654,6 +739,8 @@ struct snd_mixer_element_tone_control1 {
 #define SND_MIXER_EFF1_DELAY		(1<<7)
 #define SND_MIXER_EFF1_FEEDBACK		(1<<8)
 
+#define SND_MIXER_EFF1_DEPTH_REAR	(1<<9)
+
 struct snd_mixer_element_3d_effect1_info {
 	unsigned int effect;		/* bitmap of SND_MIXER_EFF1_* */
 	int min_wide, max_wide;		/* 3D wide */
@@ -663,6 +750,7 @@ struct snd_mixer_element_3d_effect1_info {
 	int min_depth, max_depth;	/* 3D depth */
 	int min_delay, max_delay;	/* 3D delay */
 	int min_feedback, max_feedback;	/* 3D feedback */
+	int min_depth_rear, max_depth_rear;	/* 3D depth rear */
 };
 
 struct snd_mixer_element_3d_effect1 {
@@ -676,6 +764,7 @@ struct snd_mixer_element_3d_effect1 {
 	int depth;			/* 3D depth */
 	int delay;			/* 3D delay */
 	int feedback;			/* 3D feedback */
+	int depth_rear;			/* 3D depth rear */
 };
 
 /*
@@ -755,16 +844,22 @@ typedef struct snd_mixer_element {
 	} data;
 } snd_mixer_element_t;
 
+typedef struct snd_mixer_filter {
+	unsigned int read_cmds[8];
+} snd_mixer_filter_t;
+
 /* ioctl commands */
 #define SND_MIXER_IOCTL_PVERSION	_IOR ('R', 0x00, int)
 #define SND_MIXER_IOCTL_INFO		_IOWR('R', 0x01, snd_mixer_info_t)
 #define SND_MIXER_IOCTL_ELEMENTS	_IOWR('R', 0x10, snd_mixer_elements_t)
 #define SND_MIXER_IOCTL_ROUTES		_IOWR('R', 0x11, snd_mixer_routes_t)
 #define SND_MIXER_IOCTL_GROUPS		_IOWR('R', 0x12, snd_mixer_groups_t)
-#define SND_MIXER_IOCTL_GROUP		_IOWR('R', 0x13, snd_mixer_group_t)
+#define SND_MIXER_IOCTL_GROUP_READ	_IOWR('R', 0x13, snd_mixer_group_t)
+#define SND_MIXER_IOCTL_GROUP_WRITE	_IOWR('R', 0x14, snd_mixer_group_t)
 #define SND_MIXER_IOCTL_ELEMENT_INFO	_IOWR('R', 0x20, snd_mixer_element_info_t)
 #define SND_MIXER_IOCTL_ELEMENT_READ	_IOWR('R', 0x21, snd_mixer_element_t)
 #define SND_MIXER_IOCTL_ELEMENT_WRITE	_IOWR('R', 0x22, snd_mixer_element_t)
+#define SND_MIXER_IOCTL_READ_FILTER	_IOW ('R', 0x30, snd_mixer_filter_t)
 
 /*
  *  Read interface.
@@ -776,9 +871,10 @@ typedef struct snd_mixer_element {
 #define SND_MIXER_READ_ELEMENT_ROUTE	3	/* the element route was changed */
 #define SND_MIXER_READ_ELEMENT_ADD	4	/* the element was added */
 #define SND_MIXER_READ_ELEMENT_REMOVE	5	/* the element was removed */
-#define SND_MIXER_READ_GROUP_CHANGE	6	/* the group was changed */
-#define SND_MIXER_READ_GROUP_ADD	7	/* the group was added */
-#define SND_MIXER_READ_GROUP_REMOVE	8	/* the group was removed */
+#define SND_MIXER_READ_GROUP_VALUE	6	/* the group value was changed */
+#define SND_MIXER_READ_GROUP_CHANGE	7	/* the group was changed */
+#define SND_MIXER_READ_GROUP_ADD	8	/* the group was added */
+#define SND_MIXER_READ_GROUP_REMOVE	9	/* the group was removed */
 
 typedef struct snd_mixer_read {
 	unsigned int cmd;		/* command - SND_MIXER_READ_* */
@@ -790,12 +886,12 @@ typedef struct snd_mixer_read {
 } snd_mixer_read_t;
 
 /*
- *  Obsolete interface compatible with Open Sound System API
+ *  Interface compatible with Open Sound System API
  */
 
 #ifdef __SND_OSS_COMPAT__
 
-#define SND_MIXER_OSS_CAP_EXCL_INPUT	0x00000001	/* only one recording source at moment */
+#define SND_MIXER_OSS_CAP_EXCL_INPUT	0x00000001	/* only one capture source at moment */
 
 #define SND_MIXER_OSS_DEVS	25
 #define SND_MIXER_OSS_VOLUME	0
@@ -837,15 +933,15 @@ struct snd_oss_mixer_info_obsolete {
 	char name[32];
 };
 
-#define SND_MIXER_OSS_SET_RECSRC _IOWR( 'M', 255, int )
-#define SND_MIXER_OSS_RECSRC	_IOR ( 'M', 255, int )
-#define SND_MIXER_OSS_DEVMASK	_IOR ( 'M', 254, int )
-#define SND_MIXER_OSS_RECMASK	_IOR ( 'M', 253, int )
-#define SND_MIXER_OSS_CAPS	_IOR ( 'M', 252, int )
-#define SND_MIXER_OSS_STEREODEVS _IOR ( 'M', 251, int )
-#define SND_MIXER_OSS_INFO      _IOR ( 'M', 101, struct snd_oss_mixer_info )
-#define SND_MIXER_OSS_OLD_INFO	_IOR ( 'M', 101, struct snd_oss_mixer_info_obsolete )
-#define SND_OSS_GETVERSION	_IOR ( 'M', 118, int )
+#define SND_MIXER_OSS_SET_RECSRC _IOWR('M', 255, int)
+#define SND_MIXER_OSS_RECSRC	_IOR ('M', 255, int)
+#define SND_MIXER_OSS_DEVMASK	_IOR ('M', 254, int)
+#define SND_MIXER_OSS_RECMASK	_IOR ('M', 253, int)
+#define SND_MIXER_OSS_CAPS	_IOR ('M', 252, int)
+#define SND_MIXER_OSS_STEREODEVS _IOR ('M', 251, int)
+#define SND_MIXER_OSS_INFO      _IOR ('M', 101, struct snd_oss_mixer_info)
+#define SND_MIXER_OSS_OLD_INFO	_IOR ('M', 101, struct snd_oss_mixer_info_obsolete)
+#define SND_OSS_GETVERSION	_IOR ('M', 118, int)
 
 #endif				/* __SND_OSS_COMPAT__ */
 
@@ -855,7 +951,7 @@ struct snd_oss_mixer_info_obsolete {
  *                                                                           *
  *****************************************************************************/
 
-#define SND_PCM_VERSION			SND_PROTOCOL_VERSION( 1, 0, 1 )
+#define SND_PCM_VERSION			SND_PROTOCOL_VERSION(1, 0, 2)
 
 #define SND_PCM_SFMT_MU_LAW		0
 #define SND_PCM_SFMT_A_LAW		1
@@ -892,10 +988,10 @@ struct snd_oss_mixer_info_obsolete {
 #define SND_PCM_FMT_U16_BE		(1 << SND_PCM_SFMT_U16_BE)
 #define SND_PCM_FMT_MPEG		(1 << SND_PCM_SFMT_MPEG)
 #define SND_PCM_FMT_GSM			(1 << SND_PCM_SFMT_GSM)
-#define SND_PCM_FMT_S24_LE		(1 << SND_PCM_SFMT_S32_LE)
-#define SND_PCM_FMT_S24_BE		(1 << SND_PCM_SFMT_S32_BE)
-#define SND_PCM_FMT_U24_LE		(1 << SND_PCM_SFMT_U32_LE)
-#define SND_PCM_FMT_U24_BE		(1 << SND_PCM_SFMT_U32_BE)
+#define SND_PCM_FMT_S24_LE		(1 << SND_PCM_SFMT_S24_LE)
+#define SND_PCM_FMT_S24_BE		(1 << SND_PCM_SFMT_S24_BE)
+#define SND_PCM_FMT_U24_LE		(1 << SND_PCM_SFMT_U24_LE)
+#define SND_PCM_FMT_U24_BE		(1 << SND_PCM_SFMT_U24_BE)
 #define SND_PCM_FMT_S32_LE		(1 << SND_PCM_SFMT_S32_LE)
 #define SND_PCM_FMT_S32_BE		(1 << SND_PCM_SFMT_S32_BE)
 #define SND_PCM_FMT_U32_LE		(1 << SND_PCM_SFMT_U32_LE)
@@ -908,9 +1004,9 @@ struct snd_oss_mixer_info_obsolete {
 #define SND_PCM_INFO_DSP		SND_PCM_INFO_CODEC
 #define SND_PCM_INFO_MMAP		0x00000002	/* for compatibility with OSS, this flag shouldn't be used with native applications */
 #define SND_PCM_INFO_PLAYBACK		0x00000100
-#define SND_PCM_INFO_RECORD		0x00000200
+#define SND_PCM_INFO_CAPTURE		0x00000200
 #define SND_PCM_INFO_DUPLEX		0x00000400
-#define SND_PCM_INFO_DUPLEX_LIMIT	0x00000800	/* rate for playback & record channels must be same!!! */
+#define SND_PCM_INFO_DUPLEX_LIMIT	0x00000800	/* rate for playback & capture channels must be same!!! */
 #define SND_PCM_INFO_DUPLEX_MONO	0x00001000	/* in duplex mode - only mono (one channel) is supported */
 
 #define SND_PCM_PINFO_BATCH		0x00000001	/* double buffering */
@@ -922,16 +1018,11 @@ struct snd_oss_mixer_info_obsolete {
 #define SND_PCM_RINFO_16BITONLY		0x00000004	/* hardware supports only 16-bit samples, but driver does conversions from 8-bit to 16-bit */
 #define SND_PCM_RINFO_OVERRANGE		0x00010000	/* hardware supports ADC overrange detection */
 
-#define SND_PCM_MASK_PLAYBACK		0x0001
-#define SND_PCM_MASK_RECORD		0x0002
-#define SND_PCM_MASK_DUPLEX		(SND_PCM_MASK_PLAYBACK|SND_PCM_MASK_RECORD)
-#define SND_PCM_MASK_BOTH		SND_PCM_MASK_DUPLEX
-
 /*
  * Things to know:
  *   1) Real fragment size can be aligned by driver if hardware needs.
- *      Current fragment value can be taken from status structure.
- *   2) If fragments_max in playback_params structure is -N, value
+ *      Current fragment value can be taken from the status structure.
+ *   2) If fragments_max in the playback_params structure is -N, the value
  *      means total fragments - N.
  */
 
@@ -940,7 +1031,9 @@ typedef struct snd_pcm_info {
 	unsigned int flags;		/* see to SND_PCM_INFO_XXXX */
 	unsigned char id[32];		/* ID of this PCM device */
 	unsigned char name[80];		/* name of this device */
-	unsigned char reserved[64];	/* reserved for future... */
+	unsigned int playback;		/* playback subdevices - 1 */
+	unsigned int capture;		/* capture subdevices - 1 */
+	unsigned char reserved[56];	/* reserved for future... */
 } snd_pcm_info_t;
 
 typedef struct snd_pcm_playback_info {
@@ -955,25 +1048,25 @@ typedef struct snd_pcm_playback_info {
 	unsigned int max_fragment_size;	/* max fragment size in bytes */
 	unsigned int fragment_align;	/* align fragment value */
 	unsigned int hw_formats;	/* formats supported by hardware */
-	unsigned int switches;		/* count of switches */
+	unsigned int subdevice;		/* subdevice number */
 	unsigned char reserved[56];	/* reserved for future... */
 } snd_pcm_playback_info_t;
 
-typedef struct snd_pcm_record_info {
+typedef struct snd_pcm_capture_info {
 	unsigned int flags;		/* see to SND_PCM_RINFO_XXXX */
 	unsigned int formats;		/* supported formats */
 	unsigned int min_rate;		/* min rate (in Hz) */
 	unsigned int max_rate;		/* max rate (in Hz) */
 	unsigned int min_channels;	/* min channels (probably always 1) */
 	unsigned int max_channels;	/* max channels */
-	unsigned int buffer_size;	/* record buffer size */
+	unsigned int buffer_size;	/* capture buffer size */
 	unsigned int min_fragment_size;	/* min fragment size in bytes */
 	unsigned int max_fragment_size;	/* max fragment size in bytes */
 	unsigned int fragment_align;	/* align fragment value */
 	unsigned int hw_formats;	/* formats supported by hardware */
-	unsigned int switches;		/* count of switches */
+	unsigned int subdevice;		/* subdevice number */
 	unsigned char reserved[56];	/* reserved for future... */
-} snd_pcm_record_info_t;
+} snd_pcm_capture_info_t;
 
 typedef struct snd_pcm_format {
 	unsigned int format;		/* SND_PCM_SFMT_XXXX */
@@ -990,11 +1083,11 @@ typedef struct snd_pcm_playback_params {
 	unsigned char reserved[16];	/* must be filled with zero */
 } snd_pcm_playback_params_t;
 
-typedef struct snd_pcm_record_params {
+typedef struct snd_pcm_capture_params {
 	int fragment_size;		/* requested size of fragment in bytes */
 	int fragments_min;		/* minimum number of filled fragments for wakeup */
 	unsigned char reserved[16];	/* must be filled with zero */
-} snd_pcm_record_params_t;
+} snd_pcm_capture_params_t;
 
 typedef struct snd_pcm_playback_status {
 	unsigned int rate;	/* real used rate */
@@ -1009,7 +1102,7 @@ typedef struct snd_pcm_playback_status {
 	unsigned char reserved[16];
 } snd_pcm_playback_status_t;
 
-typedef struct snd_pcm_record_status {
+typedef struct snd_pcm_capture_status {
 	unsigned int rate;	/* real used rate */
 	int fragments;		/* allocated fragments */
 	int fragment_size;	/* current fragment size in bytes */
@@ -1017,34 +1110,34 @@ typedef struct snd_pcm_record_status {
 	int free;		/* bytes in buffer still free */
 	int overrun;		/* count of overruns from last status */
 	struct timeval time;	/* time the next read was taken */
-	struct timeval stime;	/* time when record was started */
-	int scount;		/* number of bytes processed from record start */
+	struct timeval stime;	/* time when capture was started */
+	int scount;		/* number of bytes processed from capture start */
 	int overrange;		/* ADC overrange detection */
 	unsigned char reserved[12];
-} snd_pcm_record_status_t;
+} snd_pcm_capture_status_t;
 
 #define SND_PCM_IOCTL_PVERSION		_IOR ('A', 0x00, int)
 #define SND_PCM_IOCTL_INFO		_IOR ('A', 0x01, snd_pcm_info_t)
 #define SND_PCM_IOCTL_PLAYBACK_INFO	_IOR ('A', 0x02, snd_pcm_playback_info_t)
-#define SND_PCM_IOCTL_RECORD_INFO	_IOR ('A', 0x03, snd_pcm_record_info_t)
+#define SND_PCM_IOCTL_CAPTURE_INFO	_IOR ('A', 0x03, snd_pcm_capture_info_t)
 #define SND_PCM_IOCTL_PLAYBACK_FORMAT	_IOWR('A', 0x10, snd_pcm_format_t)
-#define SND_PCM_IOCTL_RECORD_FORMAT	_IOWR('A', 0x11, snd_pcm_format_t)
+#define SND_PCM_IOCTL_CAPTURE_FORMAT	_IOWR('A', 0x11, snd_pcm_format_t)
 #define SND_PCM_IOCTL_PLAYBACK_PARAMS	_IOWR('A', 0x12, snd_pcm_playback_params_t)
-#define SND_PCM_IOCTL_RECORD_PARAMS	_IOWR('A', 0x13, snd_pcm_record_params_t)
+#define SND_PCM_IOCTL_CAPTURE_PARAMS	_IOWR('A', 0x13, snd_pcm_capture_params_t)
 #define SND_PCM_IOCTL_PLAYBACK_STATUS	_IOR ('A', 0x20, snd_pcm_playback_status_t)
-#define SND_PCM_IOCTL_RECORD_STATUS	_IOR ('A', 0x21, snd_pcm_record_status_t)
+#define SND_PCM_IOCTL_CAPTURE_STATUS	_IOR ('A', 0x21, snd_pcm_capture_status_t)
 #define SND_PCM_IOCTL_DRAIN_PLAYBACK	_IO  ('A', 0x30)
 #define SND_PCM_IOCTL_FLUSH_PLAYBACK	_IO  ('A', 0x31)
-#define SND_PCM_IOCTL_FLUSH_RECORD	_IO  ('A', 0x32)
+#define SND_PCM_IOCTL_FLUSH_CAPTURE	_IO  ('A', 0x32)
 #define SND_PCM_IOCTL_PLAYBACK_PAUSE	_IOWR('A', 0x33, int)
 #define SND_PCM_IOCTL_PLAYBACK_TIME	_IOWR('A', 0x40, int)
-#define SND_PCM_IOCTL_RECORD_TIME	_IOWR('A', 0x41, int)
+#define SND_PCM_IOCTL_CAPTURE_TIME	_IOWR('A', 0x41, int)
 
 /*
  *  Loopback interface
  */
 
-#define SND_PCM_LB_VERSION		SND_PROTOCOL_VERSION( 1, 0, 0 )
+#define SND_PCM_LB_VERSION		SND_PROTOCOL_VERSION(1, 0, 0)
 
 #define SND_PCM_LB_STREAM_MODE_RAW	0
 #define SND_PCM_LB_STREAM_MODE_PACKET	1
@@ -1062,12 +1155,12 @@ typedef struct snd_pcm_loopback_header {
 #define SND_PCM_LB_IOCTL_FORMAT		_IOR ( 'L', 0x02, struct snd_pcm_format )
 
 /*
- *  Obsolete interface compatible with Open Sound System API
+ *  Interface compatible with Open Sound System API
  */
 
 #ifdef __SND_OSS_COMPAT__
 
-#define SND_PCM_ENABLE_RECORD		0x00000001
+#define SND_PCM_ENABLE_CAPTURE		0x00000001
 #define SND_PCM_ENABLE_PLAYBACK		0x00000002
 
 #define SND_PCM_CAP_REVISION		0x000000ff
@@ -1130,7 +1223,6 @@ struct snd_pcm_buffer_description {
 #define SND_PCM_IOCTL_OSS_DUPLEX	_IO  ('P', 22)
 #define SND_PCM_IOCTL_OSS_GETODELAY	_IOR ('P', 23, int)
 #define SND_PCM_IOCTL_OSS_PROFILE	_IOW ('P', 23, int)
-#define SND_PCM_IOCTL_OSS_MASK		_IOW ('X', 0, int)
 
 #endif				/* __SND_OSS_COMPAT__ */
 
@@ -1249,7 +1341,7 @@ struct snd_pcm_buffer_description {
  *  Raw MIDI section - /dev/snd/midi??
  */
 
-#define SND_RAWMIDI_VERSION		SND_PROTOCOL_VERSION( 1, 0, 0 )
+#define SND_RAWMIDI_VERSION		SND_PROTOCOL_VERSION(1, 0, 0)
 
 #define SND_RAWMIDI_INFO_OUTPUT		0x00000001
 #define SND_RAWMIDI_INFO_INPUT		0x00000002
@@ -1325,29 +1417,31 @@ typedef struct snd_rawmidi_input_status {
 #define SND_TIMER_TYPE_MAX		(7<<28)
 
 /* type */
-#define SND_TIMER_TYPE(tmr)		((tmr >> 28) & 0x3f)
+#define SND_TIMER_TYPE(tmr)		(((tmr) >> 28) & 0x3f)
 /* global number */
 #define SND_TIMER_GLOBAL_MAX		0x000003ff
-#define SND_TIMER_GLOBAL(tmr)		(tmr & SND_TIMER_GLOBAL_MAX)
+#define SND_TIMER_GLOBAL(tmr)		((tmr) & SND_TIMER_GLOBAL_MAX)
 #define SND_TIMER_GLOBAL_SYSTEM		0	/* system timer number */
 /* soundcard number */
 #define SND_TIMER_SOUNDCARD_CARD_MAX	(SND_CARDS-1)
 #define SND_TIMER_SOUNDCARD_CARD_SHIFT	22
-#define SND_TIMER_SOUNDCARD_CARD(tmr)	((tmr >> SND_TIMER_SOUNDCARD_CARD_SHIFT) & SND_TIMER_SOUNDCARD_CARD_MAX)
+#define SND_TIMER_SOUNDCARD_CARD(tmr)	(((tmr) >> SND_TIMER_SOUNDCARD_CARD_SHIFT) & SND_TIMER_SOUNDCARD_CARD_MAX)
 #define SND_TIMER_SOUNDCARD_DEV_MAX	0x003fffff
-#define SND_TIMER_SOUNDCARD_DEV(tmr)	(tmr & SND_TIMER_SOUNDCARD_DEV_MAX)
+#define SND_TIMER_SOUNDCARD_DEV(tmr)	((tmr) & SND_TIMER_SOUNDCARD_DEV_MAX)
+#define SND_TIMER_SOUNDCARD(card,dev)	(SND_TIMER_TYPE_SOUNDCARD|(((card)&SND_TIMER_SOUNDCARD_CARD_MAX)<<SND_TIMER_SOUNDCARD_CARD_SHIFT)|((dev)&SND_TIMER_SOUNDCARD_DEV_MAX))
 /* PCM slave timer numbers */
 #if SND_CARDS > 64
 #error "There is not enough space for the timer identifier."
 #endif
 #define SND_TIMER_PCM_CARD_MAX		(SND_CARDS-1)
 #define SND_TIMER_PCM_CARD_SHIFT	22
-#define SND_TIMER_PCM_CARD(tmr)		((tmr >> SND_TIMER_PCM_CARD_SHIFT) & SND_TIMER_PCM_CARD_MAX)
+#define SND_TIMER_PCM_CARD(tmr)		(((tmr) >> SND_TIMER_PCM_CARD_SHIFT) & SND_TIMER_PCM_CARD_MAX)
 #define SND_TIMER_PCM_DEV_MAX		0x000003ff
 #define SND_TIMER_PCM_DEV_SHIFT		12
-#define SND_TIMER_PCM_DEV(tmr)		((tmr >> SND_TIMER_PCM_DEV_SHIFT) & SND_TIMER_PCM_DEV_MAX)
+#define SND_TIMER_PCM_DEV(tmr)		(((tmr) >> SND_TIMER_PCM_DEV_SHIFT) & SND_TIMER_PCM_DEV_MAX)
 #define SND_TIMER_PCM_SUBDEV_MAX	0x00000fff
 #define SND_TIMER_PCM_SUBDEV(tmr)	(tmr & SND_TIMER_PCM_SUBDEV_MAX)
+#define SND_TIMER_PCM(card,dev,subdev)	(SND_TIMER_TYPE_PCM|(((card)&SND_TIMER_PCM_CARD_MAX)<<SND_TIMER_PCM_CARD_SHIFT)|(((dev)&SND_TIMER_PCM_DEV_MAX)<<SND_TIMER_PCM_DEV_SHIFT)|((subdev)&SND_TIMER_PCM_SUBDEV_MAX))
 
 /* slave timer types */
 #define SND_TIMER_STYPE_NONE		0
