@@ -29,16 +29,14 @@ typedef struct snd_stru_ctl_read {
 
 struct snd_stru_control {
 	snd_card_t *card;
+	unsigned int hwdep_device;
 	unsigned int mixer_device;
 	unsigned int pcm_device;
 	unsigned int rawmidi_device;
-	snd_spin_define(change_lock);
-	snd_sleep_define(change);
-	snd_spin_define(read_lock);
+	wait_queue_head_t change_sleep;
+	spinlock_t read_lock;
 	int read_active: 1,		/* read interface is activated */
 	    rebuild: 1;			/* rebuild the structure */
-	int pool_size;			/* size of pool in items */
-	snd_kctl_read_t *pool;		/* atomic pool */
 	snd_kctl_read_t *first_item;
 	snd_kctl_read_t *last_item;
 	struct snd_stru_control *next;
