@@ -22,7 +22,9 @@
  *
  */
 
+#ifdef ALSA_BUILD
 #include "config.h"
+#endif
 
 #define SND_CARDS		8	/* number of supported soundcards - don't change - minor numbers */
 
@@ -38,21 +40,18 @@
  *  ==========================================================================
  */
 
+#ifdef ALSA_BUILD
 #define __KERNEL__
 #define MODULE
+#endif
 
 #define LinuxVersionCode(v, p, s) (((v)<<16)|((p)<<8)|(s))
 
 #include <linux/config.h>
 #include <linux/version.h>
 
-#if LinuxVersionCode(2, 0, 0) > LINUX_VERSION_CODE
-#error "This driver is designed only for Linux 2.0.0 and highter."
-#endif
-#if LinuxVersionCode(2, 1, 0) <= LINUX_VERSION_CODE
-#define LINUX_2_1
-#if LinuxVersionCode(2, 1, 127) > LINUX_VERSION_CODE
-#error "This driver requires Linux 2.1.127 and highter."
+#if LinuxVersionCode(2, 2, 3) > LINUX_VERSION_CODE
+#error "This driver requires Linux 2.2.3 and highter."
 #endif
 #if LinuxVersionCode(2, 2, 0) <= LINUX_VERSION_CODE
 #define LINUX_2_2
@@ -60,22 +59,28 @@
 #if LinuxVersionCode(2, 3, 1) <= LINUX_VERSION_CODE
 #define LINUX_2_3
 #endif
+#if LinuxVersionCode(2, 3, 11) <= LINUX_VERSION_CODE
+#define NEW_RESOURCE
+#endif
+#if LinuxVersionCode(2, 3, 13) <= LINUX_VERSION_CODE
+#define NEW_PCI
 #endif
 
+#ifdef ALSA_BUILD
 #if defined(CONFIG_MODVERSIONS) && !defined(__GENKSYMS__) && !defined(__DEPEND__)
 #define MODVERSIONS
 #include <linux/modversions.h>
 #include "sndversions.h"
 #endif
-#ifndef SND_MAIN_OBJECT_FILE
-#define __NO_VERSION__
-#endif
 #ifdef SND_NO_MODVERS
 #undef MODVERSIONS
-#ifdef LINUX_2_1
 #undef _set_ver
 #endif
 #endif
+
+#ifndef SND_MAIN_OBJECT_FILE
+#define __NO_VERSION__
+#endif
 #include <linux/module.h>
 
 #include <linux/utsname.h>
@@ -90,26 +95,19 @@
 #include <asm/irq.h>
 #include <asm/dma.h>
 #include <asm/segment.h>
-#ifdef LINUX_2_1
 #include <asm/uaccess.h>
-#endif
 #include <asm/system.h>
 #include <asm/string.h>
 
-#include <linux/interrupt.h>
-#include <linux/fs.h>
-#include <linux/fcntl.h>
-#ifdef LINUX_2_1
-#include <linux/vmalloc.h>
-#else
-#ifdef CONFIG_PCI
-#include <linux/bios32.h>
-#endif
-#endif
-#include <linux/proc_fs.h>
 #ifdef CONFIG_PCI
 #include <linux/pci.h>
 #endif
+#include <linux/interrupt.h>
+#include <linux/fs.h>
+#include <linux/fcntl.h>
+#include <linux/vmalloc.h>
+#include <linux/proc_fs.h>
+#include <linux/poll.h>
 
 #ifdef CONFIG_ISAPNP
 #ifdef CONFIG_ISAPNP_KERNEL
@@ -119,40 +117,12 @@
 #endif
 #endif
 
-#undef SND_POLL
-#if LinuxVersionCode(2, 1, 72) <= LINUX_VERSION_CODE
-#define SND_POLL
-#endif
-#ifdef __MAX_POLL_TABLE_ENTRIES	/* new kernel with poll() */
-#define SND_POLL
-#endif
-#ifdef SND_POLL
-#include <linux/poll.h>
-#endif
-
-#if LinuxVersionCode(2, 1, 35) >= LINUX_VERSION_CODE
-#define in_interrupt() intr_count
-#endif
-
-#if LinuxVersionCode(2, 1, 118) <= LINUX_VERSION_CODE
-#define SND_FOPS_FLUSH
-#endif
-
+#ifndef ALSA_BUILD
+#include <linux/asound.h>
+#include <linux/asoundid.h>
+#else
 #include "asound.h"
 #include "asoundid.h"
-
-#ifndef LINUX_2_1
-#define mm_segment_t unsigned int
-#define copy_from_user memcpy_fromfs
-#define copy_to_user memcpy_tofs
-#define test_and_set_bit set_bit
-#define test_and_clear_bit clear_bit
-#define ioremap vremap
-#define iounmap vfree
-#endif
-
-#ifndef DMA_MODE_AUTOINIT
-#define DMA_MODE_AUTOINIT	0x10
 #endif
 
 static inline mm_segment_t snd_enter_user(void)
@@ -166,13 +136,18 @@ static inline void snd_leave_user(mm_segment_t fs)
 	set_fs(fs);
 }
 
+#ifndef LINUX_2_3
+#define init_MUTEX(x) *(x) = MUTEX
+#define DECLARE_MUTEX(x) struct semaphore x = MUTEX
+typedef struct wait_queue wait_queue_t;
+typedef struct wait_queue * wait_queue_head_t;
+#define init_waitqueue_head(x) *(x) = NULL
+#endif
+
 /*
  *  ==========================================================================
  */
 
-#include "sndspinlock.h"
-#include "schedule.h"
-
 /* auto values */
 
 #define SND_AUTO_PORT		0x00ff
@@ -187,23 +162,32 @@ static inline void snd_leave_user(mm_segment_t fs)
 #define SND_DMA_TYPE_PCI_16MB	2	/* PCI DMA (must be in low 16MB memory) */
 #define SND_DMA_TYPE_HARDWARE	3	/* buffer mapped from device */
 
-typedef struct snd_stru_dma {
-	int type;			/* dma type - see SND_DMA_TYPE_XXXX */
+typedef struct snd_stru_dma_area snd_dma_area_t;
+typedef struct snd_stru_dma snd_dma_t;
+
+struct snd_stru_dma_area {
 	volatile unsigned short
 	 mmaped:1,			/* mmaped area to user space */
 	 mmap_free:1;			/* free mmaped buffer */
-	int dma;			/* DMA number */
-	char *name;			/* pointer to name */
 	unsigned char *buf;		/* pointer to DMA buffer */
 	long size;			/* real size of DMA buffer */
-	long rsize;			/* requested size of DMA buffer */
 	char *owner;			/* owner of this DMA channel */
-	char *soft_owner;		/* owner of soft DMA channel */
 	struct vm_area_struct *vma;	/* virtual memory area */
-	snd_mutex_define(lock);		/* mutex for locking */
-	snd_mutex_define(mutex);	/* snd_dma_malloc/free */
+	snd_dma_t *dma;
+	snd_dma_area_t *next;
+};
+
+struct snd_stru_dma {
+	int type;			/* dma type - see SND_DMA_TYPE_XXXX */
+	int multi: 1;			/* multi area support */
+	int dma;			/* DMA number */
+	char *name;			/* pointer to name */
+	long rsize;			/* requested size of DMA buffer */
+	char *multi_match[2];		/* allowed owners for multi alloc */
+	struct semaphore mutex;		/* snd_dma_malloc/free */
+	struct snd_stru_dma_area *areas;  /* DMA areas */
 	struct snd_stru_dma *next;
-} snd_dma_t;
+};
 
 #define SND_IRQ_TYPE_ISA	0	/* ISA IRQ */
 #define SND_IRQ_TYPE_PCI	1	/* PCI IRQ (shareable) */
@@ -221,6 +205,9 @@ typedef struct snd_stru_port {
 	unsigned short port;
 	unsigned short size;
 	char *name;
+#ifdef NEW_RESOURCE
+	struct resource *res;
+#endif
 	struct snd_stru_port *next;
 } snd_port_t;
 
@@ -238,7 +225,7 @@ typedef struct snd_stru_rawmidi snd_rawmidi_t;
 typedef struct snd_stru_control snd_control_t;
 typedef struct snd_stru_timer snd_timer_t;
 typedef struct snd_stru_timer_instance snd_timer_instance_t;
-typedef struct snd_stru_synth snd_synth_t;
+typedef struct snd_stru_hwdep snd_hwdep_t;
 
 /* main structure for soundcard */
 
@@ -261,7 +248,7 @@ struct snd_stru_card {
 	void (*use_inc) (snd_card_t * card);	/* increment use count */
 	void (*use_dec) (snd_card_t * card);	/* decrement use count */
 
-	snd_mutex_define(control);		/* control card mutex */
+	struct semaphore control;		/* control card mutex */
 
 	void *private_data;			/* private data for soundcard */
 	void (*private_free) (void *private);	/* callback for freeing of private data */
@@ -285,11 +272,7 @@ typedef long (snd_read_t) (struct file * file, char *buf, long count);
 typedef long (snd_write_t) (struct file * file, const char *buf, long count);
 typedef int (snd_open_t) (unsigned short minor, int cardnum, int device, struct file * file);
 typedef int (snd_release_t) (unsigned short minor, int cardnum, int device, struct file * file);
-#ifdef SND_POLL
 typedef unsigned int (snd_poll_t) (struct file * file, poll_table * wait);
-#else
-typedef int (snd_select_t) (struct file * file, int sel_type, select_table * wait);
-#endif
 typedef int (snd_ioctl_t) (struct file * file, unsigned int cmd, unsigned long arg);
 typedef int (snd_mmap_t) (struct inode * inode, struct file * file, struct vm_area_struct * vma);
 
@@ -304,11 +287,7 @@ struct snd_stru_minor {
 	snd_write_t *write;
 	snd_open_t *open;
 	snd_release_t *release;
-#ifdef SND_POLL
 	snd_poll_t *poll;
-#else
-	snd_select_t *select;
-#endif
 	snd_ioctl_t *ioctl;
 	snd_mmap_t *mmap;
 };
@@ -322,6 +301,8 @@ extern int snd_device_mode;
 extern int snd_device_gid;
 extern int snd_device_uid;
 
+extern void snd_request_card(int card);
+
 extern int snd_register_device(int type, snd_card_t * card, int dev, snd_minor_t * reg, const char *name);
 extern int snd_unregister_device(int type, snd_card_t * card, int dev);
 
@@ -350,23 +331,22 @@ void snd_oss_cleanup_module(void);
 
 /* memory.c */
 
-extern void snd_malloc_init(void);
-extern void snd_malloc_done(void);
-extern void *snd_malloc(unsigned long size);
-extern void *snd_calloc(unsigned long size);
-extern void snd_free(void *obj, unsigned long size);
-extern char *snd_malloc_strdup(char *string);
-extern void snd_free_str(char *string);
+extern void snd_memory_init(void);
+extern void snd_memory_done(void);
+extern void *snd_kmalloc(size_t size, int flags);
+extern void snd_kfree(void *obj);
+extern void *snd_kcalloc(size_t size, int flags);
+extern char *snd_kmalloc_strdup(const char *string, int flags);
 extern char *snd_malloc_pages(unsigned long size, int *pg, int dma);
+extern void *snd_vmalloc(unsigned long size);
+extern void snd_vfree(void *obj);
 extern void snd_free_pages(char *ptr, unsigned long size);
-extern int snd_dma_malloc(snd_card_t * card, snd_dma_t * dma, char *owner, int soft);
-extern void snd_dma_free(snd_card_t * card, snd_dma_t * dma, int soft);
-extern int snd_dma_soft_grab(snd_card_t * card, snd_dma_t * dma);
-extern void snd_dma_soft_release(snd_card_t * card, snd_dma_t * dma);
+extern int snd_dma_malloc(snd_card_t * card, snd_dma_t * dma, char *owner, snd_dma_area_t **rarea);
+extern void snd_dma_free(snd_card_t * card, snd_dma_area_t *area);
 extern void snd_dma_notify_vma_close(struct vm_area_struct *area);
+#ifdef CONFIG_SND_DEBUG_MEMORY
 extern int snd_memory_info_init(void);
 extern int snd_memory_info_done(void);
-#ifdef CONFIG_SND_DEBUG_MEMORY
 extern void snd_memory_debug1(void);
 #endif
 
@@ -411,6 +391,7 @@ extern int snd_unregister_interrupts(snd_card_t * card);
 #define DMA_MODE_NO_ENABLE	0x0100
 
 extern void snd_dma_program(int dma, const void *buf, unsigned int size, unsigned short mode);
+extern void snd_dma_disable(int dma);
 extern unsigned int snd_dma_residue(int dma);
 
 /* misc.c */
