@@ -23,7 +23,6 @@
  *
  */
 
-#include "sndpci.h"
 #include "pcm1.h"
 #include "mixer.h"
 #include "midi.h"
@@ -81,8 +80,11 @@
 
 #define SLSB_REG_STATUS 0x0e
 
+#define SL_PCI_COMMAND 0x4
+#define SL_PCI_LEGACYCONTROL 0x40
 #define SL_PCI_CONFIG 0x50
 #define SL_PCI_DDMACONTROL 0x60
+#define SL_PCI_DDMA2CONTROL 0xF0
 
 #define SL_CMD_EXTSAMPLERATE 0xa1
 #define SL_CMD_FILTERDIV 0xa2
@@ -102,6 +104,7 @@
 #define SL_CMD_STOPAUDIO1 0xd3
 #define SL_CMD_AUDIO1STATUS 0xd8
 #define SL_CMD_CONTDMA 0xd4
+#define SL_CMD_TESTIRQ 0xf2
 
 #define SL_RECSRC_MIC 0
 #define SL_RECSRC_AUXACD 2
@@ -139,19 +142,23 @@ struct snd_stru_solo {
 
 	unsigned int mode;
 
-	struct snd_pci_dev *pci;
+	struct pci_dev *pci;
 	snd_card_t *card;
 	snd_pcm_t *pcm;
 	snd_pcm_t *pcm2;  
+	snd_pcm_subchn_t *playback_subchn;
+	snd_pcm1_subchn_t *playback_subchn1;
+	snd_pcm_subchn_t *playback2_subchn;
+	snd_pcm1_subchn_t *playback2_subchn1;
 	snd_kmixer_t *mixer;
 	snd_rawmidi_t *rmidi;
-	snd_synth_t *synth;	/* S3FM */
+	snd_hwdep_t *fmsynth;	/* FM */
 
-	 snd_spin_define(reg);
+	spinlock_t reg_lock;
 	snd_info_entry_t *proc_entry;
 };
 
-esssolo_t *snd_solo_create(snd_card_t * card, struct snd_pci_dev *pci,
+esssolo_t *snd_solo_create(snd_card_t * card, struct pci_dev *pci,
 			   snd_dma_t * dma1ptr,
 			   snd_dma_t * dma2ptr,
 			   snd_irq_t * irqptr,
