@@ -21,7 +21,11 @@
 #ifndef __SND_SEQ_KERNEL_H
 #define __SND_SEQ_KERNEL_H
 
+#ifndef ALSA_BUILD
+#include <linux/asequencer.h>
+#else
 #include "asequencer.h"
+#endif
 
 /* maximum number of events dequeued per schedule interval */
 #define SND_SEQ_MAX_DEQUEUE		50
@@ -33,10 +37,16 @@
 #define SND_SEQ_MAX_CLIENTS 		192
 
 /* max number of events in memory pool */
-#define SND_SEQ_MAX_EVENTS		500
+#define SND_SEQ_MAX_EVENTS		2000
+
+/* default number of events in memory pool */
+#define SND_SEQ_DEFAULT_EVENTS		500
 
 /* max number of events in memory pool for one client (outqueue) */
-#define SND_SEQ_MAX_CLIENT_EVENTS	200
+#define SND_SEQ_MAX_CLIENT_EVENTS	2000
+
+/* default number of events in memory pool for one client (outqueue) */
+#define SND_SEQ_DEFAULT_CLIENT_EVENTS	200
 
 /* typedefs */
 struct snd_seq_stru_user_client;
@@ -59,19 +69,19 @@ typedef struct {
 } snd_seq_client_callback_t;
 
 /* call-backs for kernel port */
-typedef int (snd_seq_kernel_port_subscribe_t)(void *private_data);
-typedef int (snd_seq_kernel_port_unsubscribe_t)(void *private_data);
-typedef int (snd_seq_kernel_port_use_t)(void *private_data);
-typedef int (snd_seq_kernel_port_unuse_t)(void *private_data);
+typedef int (snd_seq_kernel_port_subscribe_t)(void *private_data, snd_seq_port_subscribe_t *info);
+typedef int (snd_seq_kernel_port_unsubscribe_t)(void *private_data, snd_seq_port_subscribe_t *info);
+typedef int (snd_seq_kernel_port_use_t)(void *private_data, snd_seq_port_subscribe_t *info);
+typedef int (snd_seq_kernel_port_unuse_t)(void *private_data, snd_seq_port_subscribe_t *info);
 typedef int (snd_seq_kernel_port_input_t)(snd_seq_event_t *ev, int direct, void *private_data);
 typedef void (snd_seq_kernel_port_private_free_t)(void *private_data);
 
 typedef struct {
 	void *private_data;
 	snd_seq_kernel_port_subscribe_t *subscribe;
-	snd_seq_kernel_port_use_t *unsubscribe;
-	snd_seq_kernel_port_subscribe_t *use;
-	snd_seq_kernel_port_use_t *unuse;
+	snd_seq_kernel_port_unsubscribe_t *unsubscribe;
+	snd_seq_kernel_port_use_t *use;
+	snd_seq_kernel_port_unuse_t *unuse;
 	snd_seq_kernel_port_input_t *event_input;
 	snd_seq_kernel_port_private_free_t *private_free;
 	/*...*/
