@@ -19,11 +19,15 @@
  *
  */
 
-#include "driver.h"
-#include "minors.h"
-#include "info.h"
-#include "sndpersist.h"
+#include "../include/driver.h"
+#include "../include/minors.h"
+#include "../include/info.h"
+#include "../include/sndpersist.h"
+#include "../include/version.h"
 #include <stdarg.h>
+#ifdef CONFIG_DEVFS_FS
+#include <linux/devfs_fs_kernel.h>
+#endif
 
 /*
 
@@ -33,7 +37,9 @@ int snd_info_check_reserved_words(const char *str)
 {
 	static char *reserved[] =
 	{
+#ifndef CONFIG_DEVFS_FS
 		"dev",
+#endif
 		"version",
 		"meminfo",
 		"memdebug",
@@ -62,7 +68,7 @@ int snd_info_check_reserved_words(const char *str)
 extern int snd_major;
 extern struct file_operations snd_fops;
 
-snd_mutex_define_static(info);
+static DECLARE_MUTEX(info_mutex);
 
 struct snd_info_private_data {
 	snd_info_buffer_t *rbuffer;
@@ -107,198 +113,11 @@ int snd_iprintf(snd_info_buffer_t * buffer, char *fmt,...)
  */
 
 struct proc_dir_entry *snd_proc_root = NULL;
+#ifndef CONFIG_DEVFS_FS
 struct proc_dir_entry *snd_proc_dev = NULL;
+#endif
 struct proc_dir_entry *snd_proc_seq = NULL;
 
-#ifndef LINUX_2_1
-
-static struct inode *snd_info_get_inode(struct super_block *s, int ino, struct proc_dir_entry *de)
-{
-	struct inode *inode = iget(s, ino);
-	if (inode && inode->i_sb == s) {
-		inode->u.generic_ip = (void *) de;
-		if (de) {
-			if (de->mode) {
-				inode->i_mode = de->mode;
-				inode->i_uid = de->uid;
-				inode->i_gid = de->gid;
-			}
-			if (de->size)
-				inode->i_size = de->size;
-			if (de->ops)
-				inode->i_op = de->ops;
-			if (de->nlink)
-				inode->i_nlink = de->nlink;
-			if (de->fill_inode)
-				de->fill_inode(inode);
-		}
-	}
-	return inode;
-}
-
-static int snd_info_match(int len, const char *name, struct proc_dir_entry *de)
-{
-	if (!de || !de->low_ino)
-		return 0;
-	/* "" means "." ---> so paths like "/usr/lib//libc.a" work */
-	if (!len && (de->name[0] == '.') && (de->name[1] == '\0'))
-		return 1;
-	if (de->namelen != len)
-		return 0;
-	return !memcmp(name, de->name, len);
-}
-
-static int snd_info_lookup(struct inode *dir, const char *name, int len,
-			   struct inode **result)
-{
-	struct proc_dir_entry *de;
-	int ino;
-
-	*result = NULL;
-	if (!dir || !S_ISDIR(dir->i_mode)) {
-		iput(dir);
-		return -ENOTDIR;
-	}
-	de = (struct proc_dir_entry *) dir->u.generic_ip;
-	if (!de) {
-		iput(dir);
-		return -EINVAL;
-	}
-	/* Special case "." and "..": they aren't on the directory list */
-	*result = dir;
-	if (!len)
-		return 0;
-	if (name[0] == '.') {
-		if (len == 1)
-			return 0;
-		if (name[1] == '.' && len == 2) {
-			struct inode *inode;
-			inode = snd_info_get_inode(dir->i_sb,
-						   de->parent->low_ino,
-						   de->parent);
-			iput(dir);
-			if (!inode)
-				return -EINVAL;
-			*result = inode;
-			return 0;
-		}
-	}
-	*result = NULL;
-	for (de = de->subdir; de; de = de->next) {
-		if (snd_info_match(len, name, de))
-			break;
-	}
-	if (!de) {
-		iput(dir);
-		return -ENOENT;
-	}
-	ino = de->low_ino | (dir->i_ino & ~(0xffff));
-
-	if (!(*result = snd_info_get_inode(dir->i_sb, ino, de))) {
-		iput(dir);
-		return -EINVAL;
-	}
-	iput(dir);
-	return 0;
-}
-
-/*
- * This returns non-zero if at EOF, so that the /proc
- * root directory can use this and check if it should
- * continue with the <pid> entries..
- *
- * Note that the VFS-layer doesn't care about the return
- * value of the readdir() call, as long as it's non-negative
- * for success..
- */
-static int snd_info_readdir(struct inode *inode, struct file *filp,
-			    void *dirent, filldir_t filldir)
-{
-	struct proc_dir_entry *de;
-	unsigned int ino;
-	int i;
-
-	if (!inode || !S_ISDIR(inode->i_mode))
-		return -ENOTDIR;
-	ino = inode->i_ino;
-	de = (struct proc_dir_entry *) inode->u.generic_ip;
-	if (!de)
-		return -EINVAL;
-	i = filp->f_pos;
-	switch (i) {
-	case 0:
-		if (filldir(dirent, ".", 1, i, ino) < 0)
-			return 0;
-		i++;
-		filp->f_pos++;
-		/* fall through */
-	case 1:
-		if (filldir(dirent, "..", 2, i, de->parent->low_ino) < 0)
-			return 0;
-		i++;
-		filp->f_pos++;
-		/* fall through */
-	default:
-		ino &= ~0xffff;
-		de = de->subdir;
-		i -= 2;
-		for (;;) {
-			if (!de)
-				return 1;
-			if (!i)
-				break;
-			de = de->next;
-			i--;
-		}
-
-		do {
-			if (filldir(dirent, de->name, de->namelen,
-				    filp->f_pos, ino | de->low_ino) < 0)
-				return 0;
-			filp->f_pos++;
-			de = de->next;
-		} while (de);
-	}
-	return 1;
-}
-
-static struct file_operations snd_info_dir_operations =
-{
-	NULL,			/* lseek - default */
-	NULL,			/* read - bad */
-	NULL,			/* write - bad */
-	snd_info_readdir,	/* readdir */
-	NULL,			/* select - default */
-	NULL,			/* ioctl - default */
-	NULL,			/* mmap */
-	NULL,			/* no special open code */
-	NULL,			/* no special release code */
-	NULL			/* can't fsync */
-};
-
-static struct inode_operations snd_info_dir_inode_operations =
-{
-	&snd_info_dir_operations,	/* default net directory file-ops */
-	NULL,				/* create */
-	snd_info_lookup,		/* lookup */
-	NULL,				/* link */
-	NULL,				/* unlink */
-	NULL,				/* symlink */
-	NULL,				/* mkdir */
-	NULL,				/* rmdir */
-	NULL,				/* mknod */
-	NULL,				/* rename */
-	NULL,				/* readlink */
-	NULL,				/* follow_link */
-	NULL,				/* readpage */
-	NULL,				/* writepage */
-	NULL,				/* bmap */
-	NULL,				/* truncate */
-	NULL				/* permission */
-};
-
-#else /* LINUX_2_1 */
-
 static void snd_info_fill_inode(struct inode *inode, int fill)
 {
 	if (fill)
@@ -307,8 +126,6 @@ static void snd_info_fill_inode(struct inode *inode, int fill)
 		MOD_DEC_USE_COUNT;
 }
 
-#endif
-
 static snd_info_entry_t *snd_info_find_entry(unsigned short ino)
 {
 	int tmp;
@@ -353,12 +170,7 @@ static snd_card_t *snd_info_find_card(unsigned short ino)
 	return NULL;
 }
 
-#ifdef LINUX_2_1
 static loff_t snd_info_entry_lseek(struct file *file, loff_t offset, int orig)
-#else
-static int snd_info_entry_lseek(struct inode *inode, struct file *file,
-				off_t offset, int orig)
-#endif
 {
 	struct snd_info_private_data *data;
 	struct snd_info_entry *entry;
@@ -391,13 +203,8 @@ static int snd_info_entry_lseek(struct inode *inode, struct file *file,
 	return -ENXIO;
 }
 
-#ifdef LINUX_2_1
 static ssize_t snd_info_entry_read(struct file *file, char *buffer,
 				   size_t count, loff_t * offset)
-#else
-static int snd_info_entry_read(struct inode *inode, struct file *file,
-			       char *buffer, int count)
-#endif
 {
 	struct snd_info_private_data *data;
 	struct snd_info_entry *entry;
@@ -419,9 +226,8 @@ static int snd_info_entry_read(struct inode *inode, struct file *file,
 		size1 = buf->size - file->f_pos;
 		if (size1 < size)
 			size = size1;
-		if (verify_area(VERIFY_WRITE, buffer, size))
+		if (copy_to_user(buffer, buf->buffer + file->f_pos, size))
 			return -EFAULT;
-		copy_to_user(buffer, buf->buffer + file->f_pos, size);
 		file->f_pos += size;
 		break;
 	case SND_INFO_ENTRY_DATA:
@@ -436,13 +242,8 @@ static int snd_info_entry_read(struct inode *inode, struct file *file,
 	return size;
 }
 
-#ifdef LINUX_2_1
 static ssize_t snd_info_entry_write(struct file *file, const char *buffer,
 				    size_t count, loff_t * offset)
-#else
-static int snd_info_entry_write(struct inode *inode, struct file *file,
-				const char *buffer, int count)
-#endif
 {
 	struct snd_info_private_data *data;
 	struct snd_info_entry *entry;
@@ -466,9 +267,8 @@ static int snd_info_entry_write(struct inode *inode, struct file *file,
 		size1 = buf->len - file->f_pos;
 		if (size1 < size)
 			size = size1;
-		if (verify_area(VERIFY_READ, buffer, size))
+		if (copy_from_user(buf->buffer + file->f_pos, buffer, size))
 			return -EFAULT;
-		copy_from_user(buf->buffer + file->f_pos, buffer, size);
 		if (buf->size < file->f_pos + size)
 			buf->size = file->f_pos + size;
 		file->f_pos += size;
@@ -492,10 +292,10 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 	snd_info_buffer_t *buffer;
 	int mode, err;
 
-	snd_mutex_down_static(info);
+	down(&info_mutex);
 	entry = snd_info_find_entry(inode->i_ino);
 	if (!entry) {
-		snd_mutex_up_static(info);
+		up(&info_mutex);
 		return -ENODEV;
 	}
 	mode = file->f_flags & O_ACCMODE;
@@ -505,7 +305,7 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 		    (entry->type == SND_INFO_ENTRY_DATA &&
 		    			!entry->t.data.read) ||
 		    entry->type == SND_INFO_ENTRY_DEVICE) {
-			snd_mutex_up_static(info);
+			up(&info_mutex);
 			return -ENODEV;
 		}
 	}
@@ -515,14 +315,14 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 		    (entry->type == SND_INFO_ENTRY_DATA &&
 		    			!entry->t.data.write) ||
 		    entry->type == SND_INFO_ENTRY_DEVICE) {
-			snd_mutex_up_static(info);
+			up(&info_mutex);
 			return -ENODEV;
 		}
 	}
 	data = (struct snd_info_private_data *)
-			snd_calloc(sizeof(struct snd_info_private_data));
+			snd_kcalloc(sizeof(struct snd_info_private_data), GFP_KERNEL);
 	if (!data) {
-		snd_mutex_up_static(info);
+		up(&info_mutex);
 		return -ENOMEM;
 	}
 	data->entry = entry;
@@ -530,19 +330,19 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 	case SND_INFO_ENTRY_TEXT:
 		if (mode == O_RDONLY || mode == O_RDWR) {
 			buffer = (snd_info_buffer_t *)
-				 	snd_calloc(sizeof(snd_info_buffer_t));
+				 	snd_kcalloc(sizeof(snd_info_buffer_t), GFP_KERNEL);
 			if (!buffer) {
-				snd_free(data, sizeof(struct snd_info_private_data));
-				snd_mutex_up_static(info);
+				snd_kfree(data);
+				up(&info_mutex);
 				return -ENOMEM;
 			}
 			buffer->len = (entry->t.text.read_size +
 				      (PAGE_SIZE - 1)) & ~(PAGE_SIZE - 1);
-			buffer->buffer = vmalloc(buffer->len);
+			buffer->buffer = snd_vmalloc(buffer->len);
 			if (!buffer->buffer) {
-				snd_free(buffer, sizeof(snd_info_buffer_t));
-				snd_free(data, sizeof(struct snd_info_private_data));
-				snd_mutex_up_static(info);
+				snd_kfree(buffer);
+				snd_kfree(data);
+				up(&info_mutex);
 				return -ENOMEM;
 			}
 			buffer->curr = buffer->buffer;
@@ -550,27 +350,27 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 		}
 		if (mode == O_WRONLY || mode == O_RDWR) {
 			buffer = (snd_info_buffer_t *)
-					snd_calloc(sizeof(snd_info_buffer_t));
+					snd_kcalloc(sizeof(snd_info_buffer_t), GFP_KERNEL);
 			if (!buffer) {
 				if (mode == O_RDWR) {
-					vfree(data->rbuffer->buffer);
-					snd_free(data->rbuffer, sizeof(snd_info_buffer_t));
+					snd_vfree(data->rbuffer->buffer);
+					snd_kfree(data->rbuffer);
 				}
-				snd_free(data, sizeof(struct snd_info_private_data));
-				snd_mutex_up_static(info);
+				snd_kfree(data);
+				up(&info_mutex);
 				return -ENOMEM;
 			}
 			buffer->len = (entry->t.text.write_size +
 				      (PAGE_SIZE - 1)) & ~(PAGE_SIZE - 1);
-			buffer->buffer = vmalloc(buffer->len);
+			buffer->buffer = snd_vmalloc(buffer->len);
 			if (!buffer->buffer) {
 				if (mode == O_RDWR) {
-					vfree(data->rbuffer->buffer);
-					snd_free(data->rbuffer, sizeof(snd_info_buffer_t));
+					snd_vfree(data->rbuffer->buffer);
+					snd_kfree(data->rbuffer);
 				}
-				snd_free(buffer, sizeof(snd_info_buffer_t));
-				snd_free(data, sizeof(struct snd_info_private_data));
-				snd_mutex_up_static(info);
+				snd_kfree(buffer);
+				snd_kfree(data);
+				up(&info_mutex);
 				return -ENOMEM;
 			}
 			buffer->curr = buffer->buffer;
@@ -582,8 +382,8 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 			if ((err = entry->t.data.open(entry->private_data,
 						      entry, mode,
 						      &data->file_private_data)) < 0) {
-				snd_free(data, sizeof(struct snd_info_private_data));
-				snd_mutex_up_static(info);
+				snd_kfree(data);
+				up(&info_mutex);
 				return err;
 			}
 		}
@@ -593,13 +393,13 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 	MOD_INC_USE_COUNT;
 	if (entry->use_inc)
 		entry->use_inc(entry->card);
-	snd_mutex_up_static(info);
+	up(&info_mutex);
 	if (entry->type == SND_INFO_ENTRY_TEXT &&
 	    (mode == O_RDONLY || mode == O_RDWR)) {
 		if (entry->t.text.read) {
-			snd_mutex_down(entry, access);
+			down(&entry->access);
 			entry->t.text.read(data->rbuffer, entry->private_data);
-			snd_mutex_up(entry, access);
+			up(&entry->access);
 		}
 	}
 	return 0;
@@ -620,8 +420,8 @@ static int snd_info_entry_release(struct inode *inode, struct file *file)
 	switch (entry->type) {
 	case SND_INFO_ENTRY_TEXT:
 		if (mode == O_RDONLY || mode == O_RDWR) {
-			vfree(data->rbuffer->buffer);
-			snd_free(data->rbuffer, sizeof(snd_info_buffer_t));
+			snd_vfree(data->rbuffer->buffer);
+			snd_kfree(data->rbuffer);
 		}
 		if (mode == O_WRONLY || mode == O_RDWR) {
 			if (entry->t.text.write) {
@@ -634,8 +434,8 @@ static int snd_info_entry_release(struct inode *inode, struct file *file)
 						data->wbuffer->error);
 				}
 			}
-			vfree(data->wbuffer->buffer);
-			snd_free(data->wbuffer, sizeof(snd_info_buffer_t));
+			snd_vfree(data->wbuffer->buffer);
+			snd_kfree(data->wbuffer);
 		}
 		break;
 	case SND_INFO_ENTRY_DATA:
@@ -648,11 +448,10 @@ static int snd_info_entry_release(struct inode *inode, struct file *file)
 	if (entry->use_dec)
 		entry->use_dec(entry->card);
 	MOD_DEC_USE_COUNT;
-	snd_free(data, sizeof(struct snd_info_private_data));
+	snd_kfree(data);
 	return 0;
 }
 
-#ifdef SND_POLL
 static unsigned int snd_info_entry_poll(struct file *file, poll_table * wait)
 {
 	struct snd_info_private_data *data;
@@ -678,32 +477,6 @@ static unsigned int snd_info_entry_poll(struct file *file, poll_table * wait)
 	}
 	return mask;
 }
-#else
-static int snd_info_entry_select(struct inode *inode, struct file *file,
-			         int sel_type, select_table * wait)
-{
-	struct snd_info_private_data *data;
-	struct snd_info_entry *entry;
-
-	data = (struct snd_info_private_data *) file->private_data;
-	if (!data)
-		return 0;
-	entry = data->entry;
-	switch (entry->type) {
-	case SND_INFO_ENTRY_DATA:
-		if (entry->t.data.select)
-			return entry->t.data.select(entry->private_data,
-						    data->file_private_data,
-						    file, sel_type, wait);
-		if (sel_type == SEL_IN && entry->t.data.read)
-			return 1;
-		if (sel_type == SEL_OUT && entry->t.data.write)
-			return 1;
-		break;
-	}
-	return 0;
-}
-#endif
 
 static int snd_info_entry_ioctl(struct inode *inode, struct file *file,
 				unsigned int cmd, unsigned long arg)
@@ -726,16 +499,9 @@ static int snd_info_entry_ioctl(struct inode *inode, struct file *file,
 	return -ENOTTY;
 }
 
-#ifdef LINUX_2_1
 static int snd_info_entry_mmap(struct file *file, struct vm_area_struct *vma)
-#else
-static int snd_info_entry_mmap(struct inode *inode, struct file *file,
-			       struct vm_area_struct *vma)
-#endif
 {
-#ifdef LINUX_2_1
 	struct inode *inode = file->f_dentry->d_inode;
-#endif
 	struct snd_info_private_data *data;
 	struct snd_info_entry *entry;
 
@@ -760,30 +526,17 @@ static struct file_operations snd_info_entry_operations =
 	snd_info_entry_read,	/* read */
 	snd_info_entry_write,	/* write */
 	NULL,			/* readdir */
-#ifdef SND_POLL
 	snd_info_entry_poll,	/* poll */
-#else
-	snd_info_entry_select,	/* select */
-#endif
 	snd_info_entry_ioctl,	/* ioctl - default */
 	snd_info_entry_mmap,	/* mmap */
 	snd_info_entry_open,	/* open */
-#ifdef SND_FOPS_FLUSH
 	NULL,			/* flush */
-#endif
-#ifdef LINUX_2_1
 	snd_info_entry_release,	/* release */
-#else
-	(void (*)(struct inode * inode, struct file * file))
-			snd_info_entry_release,	/* release */
-#endif
 	NULL,			/* can't fsync */
-#ifdef LINUX_2_1
 	NULL,			/* fasync */
 	NULL,			/* check_media_change */
 	NULL,			/* revalidate */
 	NULL,			/* lock */
-#endif
 };
 
 static struct inode_operations snd_info_entry_inode_operations =
@@ -807,6 +560,7 @@ static struct inode_operations snd_info_entry_inode_operations =
 	NULL			/* permission */
 };
 
+#ifndef CONFIG_DEVFS_FS
 static struct inode_operations snd_info_device_inode_operations =
 {
 	&snd_fops,		/* default sound info directory file-ops */
@@ -827,25 +581,26 @@ static struct inode_operations snd_info_device_inode_operations =
 	NULL,			/* truncate */
 	NULL			/* permission */
 };
+#endif
 
-#ifdef LINUX_2_1
 static int snd_info_card_readlink(struct dentry *dentry,
 				  char *buffer, int buflen)
 {
 	int len;
 	snd_card_t *card;
 
-	snd_mutex_down_static(info);
+	down(&info_mutex);
 	card = snd_info_find_card(dentry->d_inode->i_ino);
 	if (!card) {
-		snd_mutex_up_static(info);
+		up(&info_mutex);
 		return -ENOENT;
 	}
 	len = strlen(card->id) + 1;
 	if (buflen < len)
 		len = buflen;
-	copy_to_user(buffer, card->id, len);
-	snd_mutex_up_static(info);
+	if (copy_to_user(buffer, card->id, len))
+		return -EFAULT;
+	up(&info_mutex);
 	return len;
 }
 
@@ -856,63 +611,16 @@ static struct dentry *snd_info_card_followlink(struct dentry *dentry,
 	snd_card_t *card;
 	char tmp[16];
 
-	snd_mutex_down_static(info);
+	down(&info_mutex);
 	card = snd_info_find_card(dentry->d_inode->i_ino);
 	if (!card || !card->proc_dir) {
-		snd_mutex_up_static(info);
+		up(&info_mutex);
 		return NULL;
 	}
 	strcpy(tmp, card->id);
-	snd_mutex_up_static(info);
+	up(&info_mutex);
 	return lookup_dentry(tmp, base, follow);
 }
-#else
-static int snd_info_card_readlink(struct inode *inode,
-				  char *buffer, int buflen)
-{
-	int len;
-	snd_card_t *card;
-
-	iput(inode);
-	snd_mutex_down_static(info);
-	card = snd_info_find_card(inode->i_ino);
-	if (!card) {
-		snd_mutex_up_static(info);
-		return -ENOENT;
-	}
-	len = strlen(card->id) + 1;
-	if (buflen < len)
-		len = buflen;
-	copy_to_user(buffer, card->id, len);
-	snd_mutex_up_static(info);
-	return len;
-}
-
-static int snd_info_card_followlink(struct inode *dir, struct inode *inode,
-				    int flag, int mode,
-				    struct inode **res_inode)
-{
-	struct inode *in;
-	snd_card_t *card;
-	struct proc_dir_entry *entry;
-
-	*res_inode = NULL;
-	iput(dir);
-	snd_mutex_down_static(info);
-	card = snd_info_find_card(inode->i_ino);
-	if (!card || !(entry = card->proc_dir)) {
-		snd_mutex_up_static(info);
-		return -ENOENT;
-	}
-	in = snd_info_get_inode(inode->i_sb, entry->low_ino, entry);
-	iput(inode);
-	snd_mutex_up_static(info);
-	if (!in)
-		return -ENOENT;
-	*res_inode = in;
-	return 0;
-}
-#endif
 
 static struct file_operations snd_info_card_link_operations =
 {
@@ -950,76 +658,34 @@ struct inode_operations snd_info_card_link_inode_operations =
 };
 
 
-#ifndef LINUX_2_1
-static struct proc_dir_entry snd_info_proc_root =
-{
-	0, 6, "asound",		/* inode, namelen, name */
-	S_IFDIR | S_IRUGO | S_IXUGO, 2, 0, 0,	/* mode, nlink, uid, gid */
-	0, &snd_info_dir_inode_operations,	/* size, ops */
-	NULL, NULL,		/* get_info, fill_inode */
-	NULL,			/* next */
-	NULL, NULL		/* parent, subdir */
-};
-static struct proc_dir_entry snd_info_proc_dev =
-{
-	0, 3, "dev",		/* inode, namelen, name */
-	S_IFDIR | S_IRUGO | S_IXUGO, 2, 0, 0,	/* mode, nlink, uid, gid */
-	0, &snd_info_dir_inode_operations,	/* size, ops */
-	NULL, NULL,		/* get_info, fill_inode */
-	NULL,			/* next */
-	NULL, NULL		/* parent, subdir */
-};
-static struct proc_dir_entry snd_info_proc_seq =
-{
-	0, 3, "seq",		/* inode, namelen, name */
-	S_IFDIR | S_IRUGO | S_IXUGO, 2, 0, 0,	/* mode, nlink, uid, gid */
-	0, &snd_info_dir_inode_operations,	/* size, ops */
-	NULL, NULL,		/* get_info, fill_inode */
-	NULL,			/* next */
-	NULL, NULL		/* parent, subdir */
-};
-#endif
 
 int snd_info_init(void)
 {
-#ifdef LINUX_2_1
 	struct proc_dir_entry *p;
-#endif
 
 	snd_info_entries = NULL;
 	snd_info_devices = NULL;
-#ifdef LINUX_2_1
 	p = create_proc_entry("asound", S_IFDIR | S_IRUGO | S_IXUGO, &proc_root);
 	if (!p)
 		return -ENOMEM;
 	p->fill_inode = snd_info_fill_inode;
 	snd_proc_root = p;
+#ifndef CONFIG_DEVFS_FS
 	p = create_proc_entry("dev", S_IFDIR | S_IRUGO | S_IXUGO, snd_proc_root);
 	if (!p)
 		return -ENOMEM;
 	p->fill_inode = snd_info_fill_inode;
 	snd_proc_dev = p;
+#endif
 	p = create_proc_entry("seq", S_IFDIR | S_IRUGO | S_IXUGO, snd_proc_root);
 	if (!p)
 		return -ENOMEM;
 	p->fill_inode = snd_info_fill_inode;
 	snd_proc_seq = p;
-#else
-	if (proc_register_dynamic(&proc_root, &snd_info_proc_root) < 0) {
-		return -EAGAIN;
-	}
-	snd_proc_root = &snd_info_proc_root;
-	if (proc_register_dynamic(snd_proc_root, &snd_info_proc_dev) < 0) {
-		return -EAGAIN;
-	}
-	snd_proc_dev = &snd_info_proc_dev;
-	if (proc_register_dynamic(snd_proc_root, &snd_info_proc_seq) < 0) {
-		return -EAGAIN;
-	}
-	snd_proc_seq = &snd_info_proc_seq;
-#endif
 	snd_info_version_init();
+#ifdef CONFIG_SND_MEMORY_DEBUG
 	snd_memory_info_init();
+#endif
 	snd_minor_info_init();
 #ifdef CONFIG_SND_OSSEMUL
 	snd_minor_info_oss_init();
@@ -1035,7 +701,9 @@ int snd_info_done(void)
 	snd_minor_info_oss_done();
 #endif
 	snd_minor_info_done();
+#ifdef CONFIG_SND_MEMORY_DEBUG
 	snd_memory_info_done();
+#endif
 	snd_info_version_done();
 #ifdef CONFIG_SND_DEBUG
 	if (snd_info_devices) {
@@ -1048,8 +716,10 @@ int snd_info_done(void)
 	if (snd_proc_root) {
 		if (snd_proc_seq)
 			proc_unregister(snd_proc_seq, snd_proc_seq->low_ino);
+#ifndef CONFIG_DEVFS_FS
 		if (snd_proc_dev)
 			proc_unregister(snd_proc_dev, snd_proc_dev->low_ino);
+#endif
 		proc_unregister(&proc_root, snd_proc_root->low_ino);
 	}
 	return 0;
@@ -1059,37 +729,14 @@ int snd_info_done(void)
 
  */
 
-#ifndef LINUX_2_1
-static struct proc_dir_entry snd_info_proc_card_template =
-{
-	0, 0, 0,		/* inode, namelen, name */
-	S_IFDIR | S_IRUGO | S_IXUGO, 2, 0, 0,	/* mode, nlink, uid, gid */
-	0, &snd_info_dir_inode_operations,	/* size, ops */
-	NULL, NULL,		/* get_info, fill_inode */
-	NULL,			/* next */
-	NULL, NULL		/* parent, subdir */
-};
-static struct proc_dir_entry snd_info_proc_card_link_template =
-{
-	0, 0, 0,		/* inode, namelen, name */
-	S_IFLNK | S_IRUGO | S_IWUGO | S_IXUGO, 1, 0, 0,	/* mode, nlink, uid, gid */
-	0, &snd_info_card_link_inode_operations,	/* size, ops */
-	NULL, NULL,		/* get_info, fill_inode */
-	NULL,			/* next */
-	NULL, NULL		/* parent, subdir */
-};
-#endif
 
 int snd_info_card_register(snd_card_t * card)
 {
 	char str[4];
-#ifdef LINUX_2_1
 	struct proc_dir_entry *p;
-#endif
 
 	if (!card)
 		return -EINVAL;
-#ifdef LINUX_2_1
 	p = create_proc_entry(card->id, S_IFDIR | S_IRUGO | S_IXUGO, snd_proc_root);
 	if (!p)
 		return -ENOMEM;
@@ -1105,49 +752,6 @@ int snd_info_card_register(snd_card_t * card)
 	p->fill_inode = snd_info_fill_inode;
 	p->ops = &snd_info_card_link_inode_operations;
 	card->proc_dir_link = p;
-#else
-	card->proc_dir = (struct proc_dir_entry *)
-				snd_malloc(sizeof(struct proc_dir_entry));
-	if (!card->proc_dir) {
-		return -ENOMEM;
-	}
-	*(card->proc_dir) = snd_info_proc_card_template;
-	card->proc_dir->namelen = strlen(card->proc_dir->name = card->id);
-	if (proc_register_dynamic(snd_proc_root, card->proc_dir) < 0) {
-		snd_free(card->proc_dir, sizeof(struct proc_dir_entry));
-		card->proc_dir = NULL;
-		return -EAGAIN;
-	}
-	card->proc_dir_link = (struct proc_dir_entry *)
-				snd_malloc(sizeof(struct proc_dir_entry));
-	if (!card->proc_dir_link) {
-		proc_unregister(snd_proc_root, card->proc_dir->low_ino);
-		snd_free(card->proc_dir, sizeof(struct proc_dir_entry));
-		card->proc_dir = NULL;
-		return -ENOMEM;
-	}
-	*(card->proc_dir_link) = snd_info_proc_card_link_template;
-	sprintf(str, "%i", card->number);
-	card->proc_dir_link->name = snd_malloc(4);
-	if (!card->proc_dir_link->name) {
-		snd_free(card->proc_dir_link, sizeof(struct proc_dir_entry));
-		proc_unregister(snd_proc_root, card->proc_dir->low_ino);
-		snd_free(card->proc_dir, sizeof(struct proc_dir_entry));
-		card->proc_dir_link = NULL;
-		card->proc_dir = NULL;
-		return -ENOMEM;
-	}
-	strcpy((char *) card->proc_dir_link->name, str);
-	card->proc_dir_link->namelen = strlen(str);
-	if (proc_register_dynamic(snd_proc_root, card->proc_dir_link) < 0) {
-		snd_free(card->proc_dir_link, sizeof(struct proc_dir_entry));
-		proc_unregister(snd_proc_root, card->proc_dir->low_ino);
-		snd_free(card->proc_dir, sizeof(struct proc_dir_entry));
-		card->proc_dir_link = NULL;
-		card->proc_dir = NULL;
-		return -EAGAIN;
-	}
-#endif
 	return 0;
 }
 
@@ -1155,19 +759,10 @@ int snd_info_card_unregister(snd_card_t * card)
 {
 	if (!card)
 		return -EINVAL;
-	if (card->proc_dir_link) {
+	if (card->proc_dir_link)
 		proc_unregister(snd_proc_root, card->proc_dir_link->low_ino);
-#ifndef LINUX_2_1
-		snd_free((char *) card->proc_dir_link->name, 4);
-		snd_free(card->proc_dir_link, sizeof(struct proc_dir_entry));
-#endif
-	}
-	if (card->proc_dir) {
+	if (card->proc_dir)
 		proc_unregister(snd_proc_root, card->proc_dir->low_ino);
-#ifndef LINUX_2_1
-		snd_free(card->proc_dir, sizeof(struct proc_dir_entry));
-#endif
-	}
 	card->proc_dir = NULL;
 	return 0;
 }
@@ -1234,12 +829,12 @@ snd_info_entry_t *snd_info_create_entry(snd_card_t * card, const char *name)
 {
 	snd_info_entry_t *entry;
 
-	entry = (snd_info_entry_t *) snd_calloc(sizeof(snd_info_entry_t));
+	entry = (snd_info_entry_t *) snd_kcalloc(sizeof(snd_info_entry_t), GFP_KERNEL);
 	if (!entry)
 		return NULL;
-	entry->name = snd_malloc_strdup((char *) name);
+	entry->name = snd_kmalloc_strdup(name, GFP_KERNEL);
 	if (!entry->name) {
-		snd_free(entry, sizeof(snd_info_entry_t));
+		snd_kfree(entry);
 		return NULL;
 	}
 	entry->mode = S_IFREG | S_IRUGO;
@@ -1249,7 +844,7 @@ snd_info_entry_t *snd_info_create_entry(snd_card_t * card, const char *name)
 		entry->use_inc = card->use_inc;
 		entry->use_dec = card->use_dec;
 	}
-	snd_mutex_prepare(entry, access);
+	init_MUTEX(&entry->access);
 	return entry;
 }
 
@@ -1258,38 +853,13 @@ void snd_info_free_entry(snd_info_entry_t * entry)
 	if (!entry)
 		return;
 	if (entry->name)
-		snd_free_str((char *) entry->name);
+		snd_kfree((char *)entry->name);
 	if (entry->private_free)
 		entry->private_free(entry->private_data);
-	snd_free(entry, sizeof(snd_info_entry_t));
+	snd_kfree(entry);
 }
 
-#ifndef LINUX_2_1
-static void snd_info_device_fill_inode(struct inode *inode)
-{
-	struct proc_dir_entry *de;
-	snd_info_entry_t *entry;
-
-	de = (struct proc_dir_entry *) inode->u.generic_ip;
-	if (!de)
-		return;
-	entry = (snd_info_entry_t *) de->data;
-	if (!entry)
-		return;
-	inode->i_rdev = MKDEV(entry->t.device.major, entry->t.device.minor);
-}
-
-static struct proc_dir_entry snd_info_device_template =
-{
-	0, 0, NULL,		/* inode, namelen, name */
-	S_IFCHR | S_IRUGO | S_IWUGO, 1, 0, 0,	/* mode, nlink, uid, gid */
-	0, &snd_info_device_inode_operations,	/* size, ops */
-	NULL,			/* get_info */
-	snd_info_device_fill_inode,	/* fill_inode */
-	NULL,			/* next */
-	NULL, NULL		/* parent, subdir */
-};
-#else
+#ifndef CONFIG_DEVFS_FS
 static void snd_info_device_fill_inode(struct inode *inode, int fill)
 {
 	struct proc_dir_entry *de;
@@ -1312,10 +882,15 @@ static void snd_info_device_fill_inode(struct inode *inode, int fill)
 
 snd_info_entry_t *snd_info_create_device(const char *name, unsigned int number, unsigned int mode)
 {
+#ifdef CONFIG_DEVFS_FS
+	char dname[32];
+#endif
 	unsigned short major = number >> 16;
 	unsigned short minor = (unsigned short) number;
 	snd_info_entry_t *entry, *e;
+#ifndef CONFIG_DEVFS_FS
 	struct proc_dir_entry *p = NULL;
+#endif
 
 	if (!major)
 		major = snd_major;
@@ -1329,38 +904,24 @@ snd_info_entry_t *snd_info_create_device(const char *name, unsigned int number,
 	entry->mode = mode;
 	entry->t.device.major = major;
 	entry->t.device.minor = minor;
-	snd_mutex_down_static(info);
-#ifdef LINUX_2_1
+	down(&info_mutex);
+#ifndef CONFIG_DEVFS_FS
 	p = create_proc_entry(entry->name, entry->mode, snd_proc_dev);
 	p->fill_inode = snd_info_device_fill_inode;
 	if (p) {
 		p->ops = &snd_info_device_inode_operations;
 	} else {
-		snd_mutex_up_static(info);
+		up(&info_mutex);
 		snd_info_free_entry(entry);
 		return NULL;
 	}
-#else
-	p = (struct proc_dir_entry *) snd_malloc(sizeof(struct proc_dir_entry));
-	if (!p) {
-		snd_mutex_up_static(info);
-		snd_info_free_entry(entry);
-		return NULL;
-	}
-	*p = snd_info_device_template;
-	p->namelen = strlen(p->name = entry->name);
-	p->mode = entry->mode;
-	if (proc_register_dynamic(snd_proc_dev, p) < 0) {
-		snd_free(p, sizeof(struct proc_dir_entry));
-		snd_mutex_up_static(info);
-		snd_info_free_entry(entry);
-		return NULL;
-	}
-#endif
 	p->gid = snd_device_gid;
 	p->uid = snd_device_uid;
 	p->data = (void *) entry;
 	entry->p = p;
+#else
+	entry->p = NULL;
+#endif
 	if (!(e = snd_info_devices)) {
 		snd_info_devices = entry;
 	} else {
@@ -1368,18 +929,34 @@ snd_info_entry_t *snd_info_create_device(const char *name, unsigned int number,
 			e = e->next;
 		e->next = entry;
 	}
-	snd_mutex_up_static(info);
+	up(&info_mutex);
+#ifdef CONFIG_DEVFS_FS
+	sprintf(dname, "snd/%s", name);
+	devfs_register(dname, 0, DEVFS_FL_TOLERANT,
+			major, minor, mode, 0, 0,
+			&snd_fops, NULL);
+#endif
 	return entry;
 }
 
 void snd_info_free_device(snd_info_entry_t * entry)
 {
+#ifdef CONFIG_DEVFS_FS
+	char dname[32];
+	devfs_handle_t master;
+#endif
 	snd_info_entry_t *e;
 
+#ifndef CONFIG_DEVFS_FS
 	if (!entry || !entry->p)
+#else
+	if (!entry)
+#endif
 		return;
-	snd_mutex_down_static(info);
+	down(&info_mutex);
+#ifndef CONFIG_DEVFS_FS
 	proc_unregister(snd_proc_dev, entry->p->low_ino);
+#endif
 	if (entry == (e = snd_info_devices)) {
 		snd_info_devices = snd_info_devices->next;
 	} else {
@@ -1388,25 +965,15 @@ void snd_info_free_device(snd_info_entry_t * entry)
 		if (e->next)
 			e->next = e->next->next;
 	}
-	snd_mutex_up_static(info);
-#ifndef LINUX_2_1
-	snd_free(entry->p, sizeof(struct proc_dir_entry));
+	up(&info_mutex);
+#ifdef CONFIG_DEVFS_FS
+	sprintf(dname, "snd/%s", entry->name);
+	master = devfs_find_handle(dname, 0, 0, 0, DEVFS_SPECIAL_CHR, 0);
+	devfs_unregister(master);
 #endif
 	snd_info_free_entry(entry);
 }
 
-#ifndef LINUX_2_1
-static struct proc_dir_entry snd_info_entry_template =
-{
-	0, 0, NULL,		/* inode, namelen, name */
-	S_IFREG | S_IRUGO, 1, 0, 0,		/* mode, nlink, uid, gid */
-	0, &snd_info_entry_inode_operations,	/* size, ops */
-	NULL, NULL,		/* get_info, fill_inode */
-	NULL,			/* next */
-	NULL, NULL		/* parent, subdir */
-};
-#endif
-
 static struct proc_dir_entry *snd_info_get_root(snd_info_entry_t * entry)
 {
 	struct proc_dir_entry *root;
@@ -1414,9 +981,11 @@ static struct proc_dir_entry *snd_info_get_root(snd_info_entry_t * entry)
 	root = snd_proc_root;
 	if (entry->card)
 		root = entry->card->proc_dir;
+#ifndef CONFIG_DEVFS_FS
 	if (entry->type == SND_INFO_ENTRY_DATA &&
 	    entry->subtype == SND_INFO_ENTRY_SDEVICE)
 		root = snd_proc_dev;
+#endif
 	if (entry->subtype == SND_INFO_ENTRY_SEQUENCER)
 		root = snd_proc_seq;
 	return root;
@@ -1439,33 +1008,16 @@ int snd_info_register(snd_info_entry_t * entry)
 	len = strlen(entry->name);
 	card = entry->card;
 	root = snd_info_get_root(entry);
-	snd_mutex_down_static(info);
-#ifdef LINUX_2_1
+	down(&info_mutex);
 	p = create_proc_entry(entry->name, entry->mode, root);
 	if (p) {
 		p->fill_inode = snd_info_fill_inode;
 		p->ops = &snd_info_entry_inode_operations;
 		p->size = entry->size;
 	} else {
-		snd_mutex_up_static(info);
+		up(&info_mutex);
 		return -ENOMEM;
 	}
-#else
-	p = (struct proc_dir_entry *) snd_malloc(sizeof(struct proc_dir_entry));
-	if (!p) {
-		snd_mutex_up_static(info);
-		return -ENOMEM;
-	}
-	*p = snd_info_entry_template;
-	p->namelen = strlen(p->name = entry->name);
-	p->mode = entry->mode;
-	p->size = entry->size;
-	if (proc_register_dynamic(root, p) < 0) {
-		snd_free(p, sizeof(struct proc_dir_entry));
-		snd_mutex_up_static(info);
-		return -ENOMEM;
-	}
-#endif
 	entry->p = p;
 	if (!card) {
 		if (!(e = snd_info_entries)) {
@@ -1484,7 +1036,7 @@ int snd_info_register(snd_info_entry_t * entry)
 			e->next = entry;
 		}
 	}
-	snd_mutex_up_static(info);
+	up(&info_mutex);
 	return 0;
 }
 
@@ -1496,7 +1048,7 @@ int snd_info_unregister(snd_info_entry_t * entry)
 	if (!entry || !entry->p)
 		return -EINVAL;
 	root = snd_info_get_root(entry);
-	snd_mutex_down_static(info);
+	down(&info_mutex);
 	proc_unregister(root, entry->p->low_ino);
 	if (!entry->card) {
 		if (entry == (e = snd_info_entries)) {
@@ -1517,10 +1069,7 @@ int snd_info_unregister(snd_info_entry_t * entry)
 				e->next = e->next->next;
 		}
 	}
-	snd_mutex_up_static(info);
-#ifndef LINUX_2_1
-	snd_free(entry->p, sizeof(struct proc_dir_entry));
-#endif
+	up(&info_mutex);
 	snd_info_free_entry(entry);
 	return 0;
 }
@@ -1533,15 +1082,11 @@ static snd_info_entry_t *snd_info_version_entry = NULL;
 
 static void snd_info_version_read(snd_info_buffer_t * buffer, void *private_data)
 {
-#ifdef MODULE_PARM
 	static char *kernel_version = UTS_RELEASE;
-#else
-	extern char kernel_version[];
-#endif
 
 	snd_iprintf(buffer,
 		    "Advanced Linux Sound Architecture Driver Version " CONFIG_SND_VERSION ".\n"
-		    "Compiled in " __DATE__ " for kernel %s"
+		    "Compiled on " __DATE__ " for kernel %s"
 #ifdef __SMP__
 		    " (SMP)"
 #endif
@@ -1592,10 +1137,10 @@ int snd_info_store_text(snd_info_entry_t * entry)
 	memset(&ibuffer, 0, sizeof(ibuffer));
 	ibuffer.len = (entry->t.text.read_size + (PAGE_SIZE - 1)) &
 							 ~(PAGE_SIZE - 1);
-	ibuffer.buffer = ibuffer.curr = vmalloc(ibuffer.len);
+	ibuffer.buffer = ibuffer.curr = snd_vmalloc(ibuffer.len);
 	if (!ibuffer.buffer)
 		return -ENOMEM;
-	snd_mutex_down(entry, access);
+	down(&entry->access);
 	entry->t.text.read(&ibuffer, entry->private_data);
 	result = ibuffer.error;
 	if (!result) {
@@ -1612,8 +1157,8 @@ int snd_info_store_text(snd_info_entry_t * entry)
 			result = snd_persist_remove(key);
 		}
 	}
-	snd_mutex_up(entry, access);
-	vfree(ibuffer.buffer);
+	up(&entry->access);
+	snd_vfree(ibuffer.buffer);
 	return result;
 }
 
@@ -1630,10 +1175,10 @@ int snd_info_restore_text(snd_info_entry_t * entry)
 	memset(&ibuffer, 0, sizeof(ibuffer));
 	ibuffer.len = (entry->t.text.write_size + (PAGE_SIZE - 1)) &
 							~(PAGE_SIZE - 1);
-	ibuffer.buffer = ibuffer.curr = vmalloc(ibuffer.len);
+	ibuffer.buffer = ibuffer.curr = snd_vmalloc(ibuffer.len);
 	if (!ibuffer.buffer)
 		return -ENOMEM;
-	snd_mutex_down(entry, access);
+	down(&entry->access);
 	if (entry->card)
 		sprintf(key, "snd:%i/%s", entry->card->number, entry->name);
 	else
@@ -1645,8 +1190,8 @@ int snd_info_restore_text(snd_info_entry_t * entry)
 		entry->t.text.write(&ibuffer, entry->private_data);
 		result = ibuffer.error;
 	}
-	snd_mutex_up(entry, access);
-	vfree(ibuffer.buffer);
+	up(&entry->access);
+	snd_vfree(ibuffer.buffer);
 	return result;
 }
 
