@@ -19,23 +19,22 @@
  *
  */
 
-#include "driver.h"
-#include "minors.h"
-#include "midi.h"
-#include "control.h"
-#include "info.h"
-#include "sndpersist.h"
+#include "../include/driver.h"
+#include "../include/minors.h"
+#include "../include/midi.h"
+#include "../include/control.h"
+#include "../include/info.h"
+#include "../include/sndpersist.h"
 
 snd_rawmidi_t *snd_rawmidi_devices[SND_CARDS * SND_RAWMIDI_DEVICES];
 
-snd_mutex_define_static(register);
+static DECLARE_MUTEX(register_mutex);
 
 int snd_rawmidi_receive(snd_rawmidi_t * rmidi, char *buffer, int count);
 int snd_rawmidi_transmit(snd_rawmidi_t * rmidi, char *buffer, int count);
 
 static inline unsigned short snd_rawmidi_file_flags(struct file *file)
 {
-#ifdef LINUX_2_1
 	switch (file->f_mode & (FMODE_READ | FMODE_WRITE)) {
 	case FMODE_WRITE:
 		return SND_RAWMIDI_LFLG_OUTPUT;
@@ -44,16 +43,6 @@ static inline unsigned short snd_rawmidi_file_flags(struct file *file)
 	default:
 		return SND_RAWMIDI_LFLG_OPEN;
 	}
-#else
-	switch (file->f_flags & O_ACCMODE) {
-	case O_WRONLY:
-		return SND_RAWMIDI_LFLG_OUTPUT;
-	case O_RDONLY:
-		return SND_RAWMIDI_LFLG_INPUT;
-	default:
-		return SND_RAWMIDI_LFLG_OPEN;
-	}
-#endif
 }
 
 static inline int snd_rawmidi_input_ok(snd_rawmidi_t * rmidi)
@@ -61,9 +50,9 @@ static inline int snd_rawmidi_input_ok(snd_rawmidi_t * rmidi)
 	unsigned long flags;
 	int result;
 
-	snd_spin_lock(rmidi, input, &flags);
+	spin_lock_irqsave(&rmidi->input_lock, flags);
 	result = rmidi->input.u.s.used >= rmidi->input.u.s.used_min;
-	snd_spin_unlock(rmidi, input, &flags);
+	spin_unlock_irqrestore(&rmidi->input_lock, flags);
 	return result;
 }
 
@@ -72,11 +61,11 @@ static inline int snd_rawmidi_output_ok(snd_rawmidi_t * rmidi)
 	unsigned long flags;
 	int result, free;
 
-	snd_spin_lock(rmidi, output, &flags);
+	spin_lock_irqsave(&rmidi->output_lock, flags);
 	free = rmidi->output.u.s.size - rmidi->output.u.s.used;
 	result = rmidi->output.u.s.used <= rmidi->output.u.s.used_max &&
 	         free >= rmidi->output.u.s.used_room;
-	snd_spin_unlock(rmidi, output, &flags);
+	spin_unlock_irqrestore(&rmidi->output_lock, flags);
 	return result;
 }
 
@@ -93,7 +82,7 @@ int snd_rawmidi_init_buffer(snd_rawmidi_t * rmidi, int direction)
 	pdir->u.s.used_room = 1;
 	pdir->u.s.used_min = 1;
 	pdir->u.s.buffer = NULL;
-	if ((pdir->u.s.buffer = snd_malloc(pdir->u.s.size)) == NULL) {
+	if ((pdir->u.s.buffer = snd_kmalloc(pdir->u.s.size, GFP_KERNEL)) == NULL) {
 		return -ENOMEM;
 	}
 	pdir->u.s.head = pdir->u.s.tail = pdir->u.s.used = 0;
@@ -107,7 +96,7 @@ int snd_rawmidi_done_buffer(snd_rawmidi_t * rmidi, int direction)
 	pdir = direction == SND_RAWMIDI_LFLG_INPUT ?
 					&rmidi->input : &rmidi->output;
 	if (pdir->u.s.buffer) {
-		snd_free(pdir->u.s.buffer, pdir->u.s.size);
+		snd_kfree(pdir->u.s.buffer);
 		pdir->u.s.buffer = NULL;
 	}
 	return 0;
@@ -120,7 +109,7 @@ static void snd_rawmidi_poll_output(unsigned long data)
 
 	rmidi = (snd_rawmidi_t *) data;
 	rmidi->output.hw.io.write(rmidi);
-	snd_spin_lock(rmidi, output, &flags);
+	spin_lock_irqsave(&rmidi->output_lock, flags);
 	if (rmidi->output.u.s.used > 0) {
 		rmidi->output.flags |= SND_RAWMIDI_FLG_TIMER;
 		rmidi->output.timer.expires = 1 + jiffies;
@@ -129,7 +118,7 @@ static void snd_rawmidi_poll_output(unsigned long data)
 		del_timer(&rmidi->output.timer);
 		rmidi->output.flags &= ~SND_RAWMIDI_FLG_TIMER;
 	}
-	snd_spin_unlock(rmidi, output, &flags);
+	spin_unlock_irqrestore(&rmidi->output_lock, flags);
 }
 
 void snd_rawmidi_trigger_output(snd_rawmidi_t * rmidi, int up)
@@ -142,25 +131,25 @@ void snd_rawmidi_trigger_output(snd_rawmidi_t * rmidi, int up)
 #endif
 	if (!(rmidi->output.hw.flags & SND_RAWMIDI_HW_POLL)) {
 		flg = up ? SND_RAWMIDI_FLG_TRIGGER : 0;
-		snd_spin_lock(rmidi, output, &flags);
+		spin_lock_irqsave(&rmidi->output_lock, flags);
 		if ((rmidi->output.flags & SND_RAWMIDI_FLG_TRIGGER) != flg) {
 			if (up) {
 				rmidi->output.flags |= SND_RAWMIDI_FLG_TRIGGER;
 			} else {
 				rmidi->output.flags &= ~SND_RAWMIDI_FLG_TRIGGER;
 			}
-			snd_spin_unlock(rmidi, output, &flags);
+			spin_unlock_irqrestore(&rmidi->output_lock, flags);
 			rmidi->output.hw.trigger(rmidi, up);
 			return;
 		}
-		snd_spin_unlock(rmidi, output, &flags);
+		spin_unlock_irqrestore(&rmidi->output_lock, flags);
 	} else {
-		snd_spin_lock(rmidi, output, &flags);
+		spin_lock_irqsave(&rmidi->output_lock, flags);
 		if (up) {
 			if (!(rmidi->output.flags & SND_RAWMIDI_FLG_TIMER)) {
-				snd_spin_unlock(rmidi, output, &flags);
+				spin_unlock_irqrestore(&rmidi->output_lock, flags);
 				rmidi->output.hw.io.write(rmidi);
-				snd_spin_lock(rmidi, output, &flags);
+				spin_lock_irqsave(&rmidi->output_lock, flags);
 				if (rmidi->output.u.s.used > 0) {
 					rmidi->output.flags |= SND_RAWMIDI_FLG_TIMER;
 					rmidi->output.timer.expires = 1 + jiffies;
@@ -173,7 +162,7 @@ void snd_rawmidi_trigger_output(snd_rawmidi_t * rmidi, int up)
 				del_timer(&rmidi->output.timer);
 			}
 		}
-		snd_spin_unlock(rmidi, output, &flags);
+		spin_unlock_irqrestore(&rmidi->output_lock, flags);
 	}
 }
 
@@ -184,12 +173,12 @@ static void snd_rawmidi_poll_input(unsigned long data)
 
 	rmidi = (snd_rawmidi_t *) data;
 	rmidi->output.hw.io.read(rmidi);
-	snd_spin_lock(rmidi, input, &flags);
+	spin_lock_irqsave(&rmidi->input_lock, flags);
 	rmidi->input.flags |= SND_RAWMIDI_FLG_TIMER;
 	rmidi->input.timer.data = (unsigned long) rmidi;
 	rmidi->input.timer.expires = 1 + jiffies;
 	add_timer(&rmidi->input.timer);
-	snd_spin_unlock(rmidi, input, &flags);
+	spin_unlock_irqrestore(&rmidi->input_lock, flags);
 }
 
 void snd_rawmidi_trigger_input(snd_rawmidi_t * rmidi, int up)
@@ -202,25 +191,25 @@ void snd_rawmidi_trigger_input(snd_rawmidi_t * rmidi, int up)
 #endif
 	if (!(rmidi->input.hw.flags & SND_RAWMIDI_HW_POLL)) {
 		flg = up ? SND_RAWMIDI_FLG_TRIGGER : 0;
-		snd_spin_lock(rmidi, input, &flags);
+		spin_lock_irqsave(&rmidi->input_lock, flags);
 		if ((rmidi->input.flags & SND_RAWMIDI_FLG_TRIGGER) != flg) {
 			if (up) {
 				rmidi->input.flags |= SND_RAWMIDI_FLG_TRIGGER;
 			} else {
 				rmidi->input.flags &= ~SND_RAWMIDI_FLG_TRIGGER;
 			}
-			snd_spin_unlock(rmidi, input, &flags);
+			spin_unlock_irqrestore(&rmidi->input_lock, flags);
 			rmidi->input.hw.trigger(rmidi, up);
 			return;
 		}
-		snd_spin_unlock(rmidi, input, &flags);
+		spin_unlock_irqrestore(&rmidi->input_lock, flags);
 	} else {
-		snd_spin_lock(rmidi, input, &flags);
+		spin_lock_irqsave(&rmidi->input_lock, flags);
 		if (up) {
 			if (!(rmidi->input.flags & SND_RAWMIDI_FLG_TIMER)) {
-				snd_spin_unlock(rmidi, input, &flags);
+				spin_unlock_irqrestore(&rmidi->input_lock, flags);
 				rmidi->input.hw.io.read(rmidi);
-				snd_spin_lock(rmidi, input, &flags);
+				spin_lock_irqsave(&rmidi->input_lock, flags);
 				rmidi->input.flags |= SND_RAWMIDI_FLG_TIMER;
 				rmidi->input.timer.expires = 1 + jiffies;
 				add_timer(&rmidi->input.timer);
@@ -231,7 +220,7 @@ void snd_rawmidi_trigger_input(snd_rawmidi_t * rmidi, int up)
 				del_timer(&rmidi->input.timer);
 			}
 		}
-		snd_spin_unlock(rmidi, input, &flags);
+		spin_unlock_irqrestore(&rmidi->input_lock, flags);
 	}
 }
 
@@ -247,25 +236,20 @@ int snd_rawmidi_drain_output(snd_rawmidi_t * rmidi)
 
 int snd_rawmidi_flush_output(snd_rawmidi_t * rmidi)
 {
-	unsigned long flags;
 	int err;
+	long timeout;
 
 	err = 0;
 	while (rmidi->output.u.s.used > 0) {
 #if 0
 		snd_printk("flush - used = %i\n", rmidi->output.u.s.used);
 #endif
-		snd_spin_lock(rmidi, output_sleep, &flags);
-		rmidi->output.flags |= SND_RAWMIDI_FLG_FLUSH;
-		snd_sleep(rmidi, output, 10 * HZ);
-		rmidi->output.flags &= ~SND_RAWMIDI_FLG_FLUSH;
-		snd_spin_unlock(rmidi, output_sleep, &flags);
-		if (snd_sleep_abort(rmidi, output)) {
+		timeout = interruptible_sleep_on_timeout(&rmidi->output_sleep, 10 * HZ);
+		if (signal_pending(current)) {
 			err = -EINTR;
 			break;
 		}
-		if (rmidi->output.u.s.used >= rmidi->output.u.s.size &&
-		    snd_timeout(rmidi, output)) {
+		if (rmidi->output.u.s.used > 0 && !timeout) {
 			err = -EIO;
 			break;
 		}
@@ -295,21 +279,21 @@ static int snd_rawmidi_open(unsigned short minor, int cardnum, int device,
 	rmidi = snd_rawmidi_devices[(cardnum * SND_RAWMIDI_DEVICES) + device];
 	if (!rmidi)
 		return -ENODEV;
-	snd_mutex_down(rmidi, open);
+	down(&rmidi->open_mutex);
 	if (fflags & SND_RAWMIDI_LFLG_INPUT) {
 		if (!(rmidi->info_flags & SND_RAWMIDI_INFO_INPUT)) {
-			snd_mutex_up(rmidi, open);
+			up(&rmidi->open_mutex);
 			return -ENXIO;
 		}
 	}
 	if (fflags & SND_RAWMIDI_LFLG_OUTPUT) {
 		if (!(rmidi->info_flags & SND_RAWMIDI_INFO_OUTPUT)) {
-			snd_mutex_up(rmidi, open);
+			up(&rmidi->open_mutex);
 			return -ENXIO;
 		}
 	}
 	if (rmidi->flags & fflags) {
-		snd_mutex_up(rmidi, open);
+		up(&rmidi->open_mutex);
 		return -EBUSY;
 	}
 #ifdef CONFIG_SND_OSSEMUL
@@ -318,12 +302,12 @@ static int snd_rawmidi_open(unsigned short minor, int cardnum, int device,
 #endif
 	if (fflags & SND_RAWMIDI_LFLG_INPUT) {
 		if (snd_rawmidi_init_buffer(rmidi, SND_RAWMIDI_LFLG_INPUT) < 0) {
-			snd_mutex_up(rmidi, open);
+			up(&rmidi->open_mutex);
 			return -ENOMEM;
 		}
 		if ((err = rmidi->input.hw.open(rmidi)) < 0) {
 			snd_rawmidi_done_buffer(rmidi, SND_RAWMIDI_LFLG_INPUT);
-			snd_mutex_up(rmidi, open);
+			up(&rmidi->open_mutex);
 			return err;
 		}
 		rmidi->input.data = snd_rawmidi_receive;
@@ -341,7 +325,7 @@ static int snd_rawmidi_open(unsigned short minor, int cardnum, int device,
 				snd_rawmidi_done_buffer(rmidi, SND_RAWMIDI_LFLG_INPUT);
 				rmidi->input.data = NULL;
 			}
-			snd_mutex_up(rmidi, open);
+			up(&rmidi->open_mutex);
 			return -ENOMEM;
 		}
 		if ((err = rmidi->output.hw.open(rmidi)) < 0) {
@@ -350,7 +334,7 @@ static int snd_rawmidi_open(unsigned short minor, int cardnum, int device,
 				snd_rawmidi_done_buffer(rmidi, SND_RAWMIDI_LFLG_INPUT);
 				rmidi->input.data = NULL;
 			}
-			snd_mutex_up(rmidi, open);
+			up(&rmidi->open_mutex);
 			return err;
 		}
 		rmidi->output.data = snd_rawmidi_transmit;
@@ -365,7 +349,7 @@ static int snd_rawmidi_open(unsigned short minor, int cardnum, int device,
 	file->private_data = rmidi;
 	MOD_INC_USE_COUNT;
 	rmidi->card->use_inc(rmidi->card);
-	snd_mutex_up(rmidi, open);
+	up(&rmidi->open_mutex);
 	return 0;
 }
 
@@ -379,7 +363,7 @@ static int snd_rawmidi_release(unsigned short minor, int cardnum, int device,
 	rmidi = (snd_rawmidi_t *) file->private_data;
 	if (!rmidi)
 		return -ENODEV;
-	snd_mutex_down(rmidi, open);
+	down(&rmidi->open_mutex);
 	if (fflags & SND_RAWMIDI_LFLG_INPUT) {
 		snd_rawmidi_trigger_input(rmidi, 0);
 		rmidi->input.hw.close(rmidi);
@@ -397,7 +381,7 @@ static int snd_rawmidi_release(unsigned short minor, int cardnum, int device,
 	rmidi->flags &= ~fflags;
 	rmidi->card->use_dec(rmidi->card);
 	MOD_DEC_USE_COUNT;
-	snd_mutex_up(rmidi, open);
+	up(&rmidi->open_mutex);
 	return 0;
 }
 
@@ -405,14 +389,13 @@ static int snd_rawmidi_info(snd_rawmidi_t * rmidi, snd_rawmidi_info_t * _info)
 {
 	snd_rawmidi_info_t info;
 
-	if (verify_area(VERIFY_WRITE, _info, sizeof(snd_rawmidi_info_t)))
-		return -EFAULT;
 	memset(&info, 0, sizeof(info));
 	info.type = rmidi->card->type;
 	info.flags = rmidi->info_flags;
 	strcpy(info.id, rmidi->id);
 	strcpy(info.name, rmidi->name);
-	copy_to_user(_info, &info, sizeof(snd_rawmidi_info_t));
+	if (copy_to_user(_info, &info, sizeof(snd_rawmidi_info_t)))
+		return -EFAULT;
 	return 0;
 }
 
@@ -421,11 +404,10 @@ static int snd_rawmidi_output_info(snd_rawmidi_t * rmidi,
 {
 	snd_rawmidi_output_info_t info;
 
-	if (verify_area(VERIFY_WRITE, _info, sizeof(snd_rawmidi_output_info_t)))
-		return -EFAULT;
 	memset(&info, 0, sizeof(info));
 	info.switches = snd_switch_count(&rmidi->output.switches);
-	copy_to_user(_info, &info, sizeof(snd_rawmidi_output_info_t));
+	if (copy_to_user(_info, &info, sizeof(snd_rawmidi_output_info_t)))
+		return -EFAULT;
 	return 0;
 }
 
@@ -434,11 +416,10 @@ static int snd_rawmidi_input_info(snd_rawmidi_t * rmidi,
 {
 	snd_rawmidi_input_info_t info;
 
-	if (verify_area(VERIFY_WRITE, _info, sizeof(snd_rawmidi_input_info_t)))
-		return -EFAULT;
 	memset(&info, 0, sizeof(info));
 	info.switches = snd_switch_count(&rmidi->input.switches);
-	copy_to_user(_info, &info, sizeof(snd_rawmidi_input_info_t));
+	if (copy_to_user(_info, &info, sizeof(snd_rawmidi_input_info_t)))
+		return -EFAULT;
 	return 0;
 }
 
@@ -448,9 +429,8 @@ static int snd_rawmidi_output_params(snd_rawmidi_t * rmidi,
 	snd_rawmidi_output_params_t params;
 	char *newbuf;
 
-	if (verify_area(VERIFY_READ, _params, sizeof(snd_rawmidi_output_params_t)))
+	if (copy_from_user(&params, _params, sizeof(snd_rawmidi_output_params_t)))
 		return -EFAULT;
-	copy_from_user(&params, _params, sizeof(snd_rawmidi_output_params_t));
 	snd_rawmidi_flush_output(rmidi);
 	if (params.size < 32 || params.size > 1024L * 1024L)
 		return -EINVAL;
@@ -458,9 +438,9 @@ static int snd_rawmidi_output_params(snd_rawmidi_t * rmidi,
 		return -EINVAL;
 	if (params.max < 1 || params.max >= params.size || params.max < params.room)
 		return -EINVAL;
-	if ((newbuf = (char *) snd_malloc(params.size)) == NULL)
+	if ((newbuf = (char *) snd_kmalloc(params.size, GFP_KERNEL)) == NULL)
 		return -ENOMEM;
-	snd_free(rmidi->output.u.s.buffer, rmidi->output.u.s.size);
+	snd_kfree(rmidi->output.u.s.buffer);
 	rmidi->output.u.s.buffer = newbuf;
 	rmidi->output.u.s.size = params.size;
 	rmidi->output.u.s.used_max = params.max;
@@ -474,17 +454,16 @@ static int snd_rawmidi_input_params(snd_rawmidi_t * rmidi,
 	snd_rawmidi_input_params_t params;
 	char *newbuf;
 
-	if (verify_area(VERIFY_READ, _params, sizeof(snd_rawmidi_input_params_t)))
+	if (copy_from_user(&params, _params, sizeof(snd_rawmidi_input_params_t)))
 		return -EFAULT;
-	copy_from_user(&params, _params, sizeof(snd_rawmidi_input_params_t));
 	snd_rawmidi_flush_input(rmidi);
 	if (params.size < 32 || params.size > 1024L * 1024L)
 		return -EINVAL;
 	if (params.min < 1 || params.min >= params.size)
 		return -EINVAL;
-	if ((newbuf = (char *) snd_malloc(params.size)) == NULL)
+	if ((newbuf = (char *) snd_kmalloc(params.size, GFP_KERNEL)) == NULL)
 		return -ENOMEM;
-	snd_free(rmidi->input.u.s.buffer, rmidi->input.u.s.size);
+	snd_kfree(rmidi->input.u.s.buffer);
 	rmidi->input.u.s.buffer = newbuf;
 	rmidi->input.u.s.size = params.size;
 	rmidi->input.u.s.used_min = params.min;
@@ -497,15 +476,14 @@ static int snd_rawmidi_output_status(snd_rawmidi_t * rmidi,
 	unsigned long flags;
 	snd_rawmidi_output_status_t status;
 
-	if (verify_area(VERIFY_WRITE, _status, sizeof(snd_rawmidi_output_status_t)))
-		return -EFAULT;
 	memset(&status, 0, sizeof(status));
-	snd_spin_lock(rmidi, output, &flags);
+	spin_lock_irqsave(&rmidi->output_lock, flags);
 	status.size = rmidi->output.u.s.size;
 	status.count = rmidi->output.u.s.size - rmidi->output.u.s.used;
 	status.queue = rmidi->output.u.s.used;
-	snd_spin_unlock(rmidi, output, &flags);
-	copy_to_user(_status, &status, sizeof(snd_rawmidi_output_status_t));
+	spin_unlock_irqrestore(&rmidi->output_lock, flags);
+	if (copy_to_user(_status, &status, sizeof(snd_rawmidi_output_status_t)))
+		return -EFAULT;
 	return 0;
 }
 
@@ -515,17 +493,16 @@ static int snd_rawmidi_input_status(snd_rawmidi_t * rmidi,
 	unsigned long flags;
 	snd_rawmidi_input_status_t status;
 
-	if (verify_area(VERIFY_WRITE, _status, sizeof(snd_rawmidi_input_status_t)))
-		return -EFAULT;
 	memset(&status, 0, sizeof(status));
-	snd_spin_lock(rmidi, input, &flags);
+	spin_lock_irqsave(&rmidi->input_lock, flags);
 	status.size = rmidi->input.u.s.size;
 	status.count = rmidi->input.u.s.used;
 	status.free = rmidi->input.u.s.size - rmidi->input.u.s.used;
 	status.overrun = rmidi->input.u.s.xruns;
 	rmidi->input.u.s.xruns = 0;
-	snd_spin_unlock(rmidi, input, &flags);
-	copy_to_user(_status, &status, sizeof(snd_rawmidi_input_status_t));
+	spin_unlock_irqrestore(&rmidi->input_lock, flags);
+	if (copy_to_user(_status, &status, sizeof(snd_rawmidi_input_status_t)))
+		return -EFAULT;
 	return 0;
 }
 
@@ -604,7 +581,8 @@ static int snd_rawmidi_switch_write(snd_control_t * control,
 	if (result > 0) {
 		snd_switch_t sw;
 
-		copy_from_user(&sw, _sw, sizeof(*_sw));
+		if (copy_from_user(&sw, _sw, sizeof(*_sw)))
+			return -EFAULT;
 		snd_control_notify_switch_value_change(control,
 						       iface,
 						       sw.name,
@@ -694,7 +672,7 @@ int snd_rawmidi_receive(snd_rawmidi_t * rmidi, char *buffer, int count)
 		snd_printd("snd_rawmidi_receive: input isn't active!!!\n");
 		return -EINVAL;
 	}
-	snd_spin_lock(rmidi, input, &flags);
+	spin_lock_irqsave(&rmidi->input_lock, flags);
 	rmidi->input.bytes += count;
 	count1 = rmidi->input.u.s.size - rmidi->input.u.s.head;
 	if (count1 > count)
@@ -719,10 +697,9 @@ int snd_rawmidi_receive(snd_rawmidi_t * rmidi, char *buffer, int count)
 			rmidi->input.u.s.used += count1;
 		}
 	}
-	snd_spin_unlock(rmidi, input, &flags);
-	if ((rmidi->input.flags & SND_RAWMIDI_FLG_SLEEP) && snd_rawmidi_input_ok(rmidi)) {
-		snd_wakeup(rmidi, input);
-	}
+	spin_unlock_irqrestore(&rmidi->input_lock, flags);
+	if (snd_rawmidi_input_ok(rmidi))
+		wake_up(&rmidi->input_sleep);
 #if 0
 	snd_printk("rx: head = %i, tail = %i, used = %i, size = %i\n", rmidi->input.head, rmidi->input.tail, rmidi->input.used, rmidi->input.size);
 #endif
@@ -732,45 +709,40 @@ int snd_rawmidi_receive(snd_rawmidi_t * rmidi, char *buffer, int count)
 static long snd_rawmidi_read(struct file *file, char *buf, long count)
 {
 	unsigned long flags;
-	long result;
+	long result, timeout;
 	int count1;
 	snd_rawmidi_t *rmidi;
 
 	rmidi = (snd_rawmidi_t *) file->private_data;
 	if (!rmidi)
 		return -EIO;
-	if (verify_area(VERIFY_WRITE, buf, count))
-		return -EFAULT;
 	result = 0;
 	snd_rawmidi_trigger_input(rmidi, 1);
 	while (count > 0) {
 		while (!snd_rawmidi_input_ok(rmidi)) {
 			if (file->f_flags & O_NONBLOCK)
 				return result;
-			snd_spin_lock(rmidi, input_sleep, &flags);
-			rmidi->input.flags |= SND_RAWMIDI_FLG_SLEEP;
-			snd_sleep(rmidi, input, 10 * HZ);
-			rmidi->input.flags &= ~SND_RAWMIDI_FLG_SLEEP;
-			snd_spin_unlock(rmidi, input_sleep, &flags);
-			if (snd_sleep_abort(rmidi, input))
+			timeout = interruptible_sleep_on_timeout(&rmidi->input_sleep, 10 * HZ);
+			if (signal_pending(current))
 				return result;
-			if (!rmidi->input.u.s.used && snd_timeout(rmidi, input))
+			if (!rmidi->input.u.s.used && !timeout)
 				return -EIO;
 		}
 		while (count > 0 && rmidi->input.u.s.used) {
 			count1 = rmidi->input.u.s.size - rmidi->input.u.s.tail;
 			if (count1 > count)
 				count1 = count;
-			snd_spin_lock(rmidi, input, &flags);
+			spin_lock_irqsave(&rmidi->input_lock, flags);
 			if (count1 > rmidi->input.u.s.used)
 				count1 = rmidi->input.u.s.used;
-			snd_spin_unlock(rmidi, input, &flags);
-			copy_to_user(buf, rmidi->input.u.s.buffer + rmidi->input.u.s.tail, count1);
-			snd_spin_lock(rmidi, input, &flags);
+			spin_unlock_irqrestore(&rmidi->input_lock, flags);
+			if (copy_to_user(buf, rmidi->input.u.s.buffer + rmidi->input.u.s.tail, count1))
+				return -EFAULT;
+			spin_lock_irqsave(&rmidi->input_lock, flags);
 			rmidi->input.u.s.tail += count1;
 			rmidi->input.u.s.tail %= rmidi->input.u.s.size;
 			rmidi->input.u.s.used -= count1;
-			snd_spin_unlock(rmidi, input, &flags);
+			spin_unlock_irqrestore(&rmidi->input_lock, flags);
 			result += count1;
 			buf += count1;
 			count -= count1;
@@ -792,17 +764,16 @@ int snd_rawmidi_transmit(snd_rawmidi_t * rmidi, char *buffer, int count)
 		snd_printd("snd_rawmidi_transmit: output isn't active!!!\n");
 		return 0;
 	}
-	snd_spin_lock(rmidi, output, &flags);
+	spin_lock_irqsave(&rmidi->output_lock, flags);
 	count1 = rmidi->output.u.s.size - rmidi->output.u.s.tail;
 	if (count1 > count)
 		count1 = count;
 	if (count1 > rmidi->output.u.s.used)
 		count1 = rmidi->output.u.s.used;
 	if (!count1) {
-		snd_spin_unlock(rmidi, output, &flags);
+		spin_unlock_irqrestore(&rmidi->output_lock, flags);
 		snd_rawmidi_trigger_output(rmidi, 0);
-		if (rmidi->output.flags & (SND_RAWMIDI_FLG_FLUSH | SND_RAWMIDI_FLG_SLEEP))
-			snd_wakeup(rmidi, output);
+		wake_up(&rmidi->output_sleep);
 		return 0;
 	}
 	memcpy(buffer, rmidi->output.u.s.buffer + rmidi->output.u.s.tail, count1);
@@ -818,17 +789,14 @@ int snd_rawmidi_transmit(snd_rawmidi_t * rmidi, char *buffer, int count)
 		result += count;
 	}
 	if (!rmidi->output.u.s.used) {
-		snd_spin_unlock(rmidi, output, &flags);
+		spin_unlock_irqrestore(&rmidi->output_lock, flags);
 		if (!result)
 			snd_rawmidi_trigger_output(rmidi, 0);
-		if (rmidi->output.flags & (SND_RAWMIDI_FLG_FLUSH | SND_RAWMIDI_FLG_SLEEP))
-			snd_wakeup(rmidi, output);
 	} else {
-		snd_spin_unlock(rmidi, output, &flags);
-		if ((rmidi->output.flags & SND_RAWMIDI_FLG_SLEEP) &&
-		    snd_rawmidi_output_ok(rmidi))
-			snd_wakeup(rmidi, output);
+		spin_unlock_irqrestore(&rmidi->output_lock, flags);
 	}
+	if (snd_rawmidi_output_ok(rmidi))
+		wake_up(&rmidi->output_sleep);
 #if 0
 	snd_printk("write1: head = %i, tail = %i, used = %i, size = %i\n", rmidi->output.u.s.head, rmidi->output.u.s.tail, rmidi->output.u.s.used, rmidi->output.u.s.size);
 	snd_printk("transmit result = %i\n", result);
@@ -840,7 +808,7 @@ int snd_rawmidi_transmit(snd_rawmidi_t * rmidi, char *buffer, int count)
 static long snd_rawmidi_write(struct file *file, const char *buf, long count)
 {
 	unsigned long flags;
-	long result;
+	long result, timeout;
 	int count1;
 	snd_rawmidi_t *rmidi;
 
@@ -852,31 +820,28 @@ static long snd_rawmidi_write(struct file *file, const char *buf, long count)
 		while (!snd_rawmidi_output_ok(rmidi)) {
 			if (file->f_flags & O_NONBLOCK)
 				return result;
-			snd_spin_lock(rmidi, output_sleep, &flags);
-			rmidi->output.flags |= SND_RAWMIDI_FLG_SLEEP;
-			snd_sleep(rmidi, output, 10 * HZ);
-			rmidi->output.flags &= ~SND_RAWMIDI_FLG_SLEEP;
-			snd_spin_unlock(rmidi, output_sleep, &flags);
-			if (snd_sleep_abort(rmidi, output))
+			timeout = interruptible_sleep_on_timeout(&rmidi->output_sleep, 10 * HZ);
+			if (signal_pending(current))
 				return -EINTR;
 			if (rmidi->output.u.s.used >= rmidi->output.u.s.size &&
-			    snd_timeout(rmidi, output))
+			    !timeout)
 				return -EIO;
 		}
 		while (count > 0 && rmidi->output.u.s.used < rmidi->output.u.s.size) {
 			count1 = rmidi->output.u.s.size - rmidi->output.u.s.head;
 			if (count1 > count)
 				count1 = count;
-			snd_spin_lock(rmidi, output, &flags);
+			spin_lock_irqsave(&rmidi->output_lock, flags);
 			if (count1 > rmidi->output.u.s.size - rmidi->output.u.s.used)
 				count1 = rmidi->output.u.s.size - rmidi->output.u.s.used;
-			snd_spin_unlock(rmidi, output, &flags);
-			copy_from_user(rmidi->output.u.s.buffer + rmidi->output.u.s.head, buf, count1);
-			snd_spin_lock(rmidi, output, &flags);
+			spin_unlock_irqrestore(&rmidi->output_lock, flags);
+			if (copy_from_user(rmidi->output.u.s.buffer + rmidi->output.u.s.head, buf, count1))
+				return -EFAULT;
+			spin_lock_irqsave(&rmidi->output_lock, flags);
 			rmidi->output.u.s.head += count1;
 			rmidi->output.u.s.head %= rmidi->output.u.s.size;
 			rmidi->output.u.s.used += count1;
-			snd_spin_unlock(rmidi, output, &flags);
+			spin_unlock_irqrestore(&rmidi->output_lock, flags);
 			result += count1;
 			buf += count1;
 			count -= count1;
@@ -886,11 +851,9 @@ static long snd_rawmidi_write(struct file *file, const char *buf, long count)
 	return result;
 }
 
-#ifdef SND_POLL
 static unsigned int snd_rawmidi_poll(struct file *file, poll_table * wait)
 {
 	snd_rawmidi_t *rmidi;
-	unsigned long flags;
 	unsigned int mask;
 	unsigned short fflags;
 
@@ -901,17 +864,10 @@ static unsigned int snd_rawmidi_poll(struct file *file, poll_table * wait)
 
 	if (fflags & SND_RAWMIDI_LFLG_INPUT) {
 		snd_rawmidi_trigger_input(rmidi, 1);
-		snd_spin_lock(rmidi, input_sleep, &flags);
-		rmidi->input.flags |= SND_RAWMIDI_FLG_SLEEP;
-		snd_poll_wait(file, rmidi, input, wait);
-		snd_spin_unlock(rmidi, input_sleep, &flags);
-	}
-	if (fflags & SND_RAWMIDI_LFLG_OUTPUT) {
-		snd_spin_lock(rmidi, output_sleep, &flags);
-		rmidi->output.flags |= SND_RAWMIDI_FLG_SLEEP;
-		snd_poll_wait(file, rmidi, output, wait);
-		snd_spin_unlock(rmidi, output_sleep, &flags);
+		poll_wait(file, &rmidi->input_sleep, wait);
 	}
+	if (fflags & SND_RAWMIDI_LFLG_OUTPUT)
+		poll_wait(file, &rmidi->output_sleep, wait);
 	mask = 0;
 	if (fflags & SND_RAWMIDI_LFLG_INPUT) {
 		if (snd_rawmidi_input_ok(rmidi))
@@ -923,53 +879,6 @@ static unsigned int snd_rawmidi_poll(struct file *file, poll_table * wait)
 	}
 	return mask;
 }
-#else
-static int snd_rawmidi_select(struct file *file,
-			      int sel_type,
-			      select_table * wait)
-{
-	snd_rawmidi_t *rmidi;
-	unsigned long flags;
-	unsigned short fflags;
-
-	fflags = snd_rawmidi_file_flags(file);
-	rmidi = (snd_rawmidi_t *) file->private_data;
-	if (!rmidi)
-		return -EIO;
-	switch (sel_type) {
-	case SEL_IN:
-		if (!(fflags & SND_RAWMIDI_LFLG_INPUT))
-			return 0;
-		snd_rawmidi_trigger_input(rmidi, 1);
-		snd_spin_lock(rmidi, input_sleep, &flags);
-		if (!snd_rawmidi_input_ok(rmidi)) {
-			rmidi->input.flags |= SND_RAWMIDI_FLG_SLEEP;
-			snd_select_wait(rmidi, input, wait);
-			snd_spin_unlock(rmidi, input_sleep, &flags);
-			return 0;
-		}
-		rmidi->input.flags &= ~SND_RAWMIDI_FLG_SLEEP;
-		snd_spin_unlock(rmidi, input_sleep, &flags);
-		return 1;
-	case SEL_OUT:
-		if (!(fflags & SND_RAWMIDI_LFLG_OUTPUT))
-			return 0;
-		snd_spin_lock(rmidi, output_sleep, &flags);
-		if (!snd_rawmidi_output_ok(rmidi)) {
-			rmidi->output.flags |= SND_RAWMIDI_FLG_SLEEP;
-			snd_select_wait(rmidi, output, wait);
-			snd_spin_unlock(rmidi, output_sleep, &flags);
-			return 0;
-		}
-		rmidi->output.flags &= ~SND_RAWMIDI_FLG_SLEEP;
-		snd_spin_unlock(rmidi, output_sleep, &flags);
-		return 1;
-	case SEL_EX:
-		break;
-	}
-	return 0;
-}
-#endif
 
 /*
 
@@ -1043,11 +952,7 @@ static snd_minor_t snd_rawmidi_reg =
 	snd_rawmidi_write,	/* write */
 	snd_rawmidi_open,	/* open */
 	snd_rawmidi_release,	/* release */
-#ifdef SND_POLL
 	snd_rawmidi_poll,	/* poll */
-#else
-	snd_rawmidi_select,	/* select */
-#endif
 	snd_rawmidi_ioctl,	/* ioctl */
 	NULL,			/* mmap */
 };
@@ -1058,18 +963,16 @@ snd_rawmidi_t *snd_rawmidi_new_device(snd_card_t * card, char *id)
 
 	if (!card)
 		return NULL;
-	rmidi = (snd_rawmidi_t *) snd_malloc(sizeof(snd_rawmidi_t));
+	rmidi = (snd_rawmidi_t *) snd_kmalloc(sizeof(snd_rawmidi_t), GFP_KERNEL);
 	if (!rmidi)
 		return NULL;
 	memset(rmidi, 0, sizeof(snd_rawmidi_t));
 	rmidi->card = card;
-	snd_spin_prepare(rmidi, input);
-	snd_spin_prepare(rmidi, input_sleep);
-	snd_spin_prepare(rmidi, output);
-	snd_spin_prepare(rmidi, output_sleep);
-	snd_sleep_prepare(rmidi, input);
-	snd_sleep_prepare(rmidi, output);
-	snd_mutex_prepare(rmidi, open);
+	rmidi->input_lock = SPIN_LOCK_UNLOCKED;
+	rmidi->output_lock = SPIN_LOCK_UNLOCKED;
+	init_waitqueue_head(&rmidi->input_sleep);
+	init_waitqueue_head(&rmidi->output_sleep);
+	init_MUTEX(&rmidi->open_mutex);
 	snd_switch_prepare(&rmidi->input.switches);
 	snd_switch_prepare(&rmidi->output.switches);
 	rmidi->input.timer.data = (unsigned long) rmidi;
@@ -1094,7 +997,7 @@ int snd_rawmidi_free(snd_rawmidi_t * rmidi)
 		rmidi->input.hw.private_free(rmidi->input.hw.private_data);
 	if (rmidi->private_free)
 		rmidi->private_free(rmidi->private_data);
-	snd_free(rmidi, sizeof(snd_rawmidi_t));
+	snd_kfree(rmidi);
 	return 0;
 }
 
@@ -1113,7 +1016,7 @@ static void snd_rawmidi_store(snd_rawmidi_t * rmidi)
 	snd_switch_lock(liswitch, 0);
 	size = sizeof(rmidi->id) + sizeof(rmidi->name) +
 	       snd_switch_size(loswitch) + snd_switch_size(liswitch);
-	buffer = (char *) vmalloc(size);
+	buffer = (char *) snd_vmalloc(size);
 	if (!buffer) {
 		snd_printk("cannot store rawmidi settings - no enough memory\n" );
 		snd_switch_lock(loswitch, 1);
@@ -1130,7 +1033,7 @@ static void snd_rawmidi_store(snd_rawmidi_t * rmidi)
 	if (err < 0) {
 		snd_printd("cannot store rawmidi output switch settings\n");
 		snd_switch_lock(liswitch, 1);
-		vfree(buffer);
+		snd_vfree(buffer);
 		return;
 	}
 	pos += err;
@@ -1138,12 +1041,12 @@ static void snd_rawmidi_store(snd_rawmidi_t * rmidi)
 	snd_switch_lock(liswitch, 1);
 	if (err < 0) {
 		snd_printd("cannot store rawmidi input switch settings\n");
-		vfree(buffer);
+		snd_vfree(buffer);
 		return;
 	}
 	sprintf(key, "snd:%i/rawmidi:%i", rmidi->card->number, rmidi->device);
 	snd_persist_store(key, buffer, size);
-	vfree(buffer);
+	snd_vfree(buffer);
 }
 
 static void snd_rawmidi_restore(snd_rawmidi_t * rmidi)
@@ -1158,7 +1061,7 @@ static void snd_rawmidi_restore(snd_rawmidi_t * rmidi)
 	size = snd_persist_length(key);
 	if (size <= 0)
 		return;
-	buffer = (char *) vmalloc(size);
+	buffer = (char *) snd_vmalloc(size);
 	if (!buffer) {
 		snd_printk("cannot restore rawmidi settings - no enough memory\n" );
 		return;
@@ -1166,31 +1069,31 @@ static void snd_rawmidi_restore(snd_rawmidi_t * rmidi)
 	size = snd_persist_restore(key, buffer, size);
 	/* ok. I'm very paranoid in bellow code */
 	if (size < (int)(sizeof(rmidi->id) + sizeof(rmidi->name))) {
-		vfree(buffer);
+		snd_vfree(buffer);
 		return;
 	}
 	pos = 0;
 	if (strcmp(buffer + pos, rmidi->id)) {
-		vfree(buffer);
+		snd_vfree(buffer);
 		return;
 	}
 	pos += sizeof(rmidi->id);
 	if (strcmp(buffer + pos, rmidi->name)) {
-		vfree(buffer);
+		snd_vfree(buffer);
 		return;
 	}
 	pos += sizeof(rmidi->name);
 	err = snd_switch_restore(&rmidi->output.switches, rmidi, buffer + pos, size - pos);
 	if (err < 0) {
 		snd_printk("cannot restore rawmidi output switch settings\n");
-		vfree(buffer);
+		snd_vfree(buffer);
 		return;
 	}
 	pos += err;
 	err = snd_switch_restore(&rmidi->input.switches, rmidi, buffer + pos, size - pos);
 	if (err < 0)
 		snd_printk("cannot restore rawmidi input switch settings\n");
-	vfree(buffer);
+	snd_vfree(buffer);
 }
 
 int __snd_rawmidi_register(snd_rawmidi_t * rmidi, int rawmidi_device)
@@ -1199,14 +1102,14 @@ int __snd_rawmidi_register(snd_rawmidi_t * rmidi, int rawmidi_device)
 	snd_info_entry_t *entry;
 	char name[16];
 
-	snd_mutex_down_static(register);
+	down(&register_mutex);
 	if (rawmidi_device < 0 || rawmidi_device >= SND_RAWMIDI_DEVICES) {
-		snd_mutex_up_static(register);
+		up(&register_mutex);
 		return -EINVAL;
 	}
 	idx = (rmidi->card->number * SND_RAWMIDI_DEVICES) + rawmidi_device;
 	if (snd_rawmidi_devices[idx]) {
-		snd_mutex_up_static(register);
+		up(&register_mutex);
 		return -EBUSY;
 	}
 	rmidi->device = rawmidi_device;
@@ -1215,7 +1118,7 @@ int __snd_rawmidi_register(snd_rawmidi_t * rmidi, int rawmidi_device)
 	if ((err = snd_register_device(SND_DEVICE_TYPE_RAWMIDI,
 				rmidi->card, rawmidi_device, &snd_rawmidi_reg, name)) < 0) {
 		snd_rawmidi_devices[idx] = NULL;
-		snd_mutex_up_static(register);
+		up(&register_mutex);
 		return err;
 	}
 #ifdef CONFIG_SND_OSSEMUL
@@ -1229,8 +1132,8 @@ int __snd_rawmidi_register(snd_rawmidi_t * rmidi, int rawmidi_device)
 		}
 	}
 #endif
-	snd_mutex_up_static(register);
-	sprintf(name, "midi%i", rawmidi_device);
+	up(&register_mutex);
+	sprintf(name, "midiD%d", rawmidi_device);
 	entry = snd_info_create_entry(rmidi->card, name);
 	if (entry) {
 		entry->private_data = rmidi;
@@ -1255,10 +1158,10 @@ int __snd_rawmidi_unregister(snd_rawmidi_t * rmidi)
 
 	if (!rmidi)
 		return -EINVAL;
-	snd_mutex_down_static(register);
+	down(&register_mutex);
 	idx = (rmidi->card->number * SND_RAWMIDI_DEVICES) + rmidi->device;
 	if (snd_rawmidi_devices[idx] != rmidi) {
-		snd_mutex_up_static(register);
+		up(&register_mutex);
 		return -EINVAL;
 	}
 	snd_rawmidi_store(rmidi);
@@ -1275,7 +1178,7 @@ int __snd_rawmidi_unregister(snd_rawmidi_t * rmidi)
 #endif
 	snd_unregister_device(SND_DEVICE_TYPE_RAWMIDI, rmidi->card, rmidi->device);
 	snd_rawmidi_devices[idx] = NULL;
-	snd_mutex_up_static(register);
+	up(&register_mutex);
 	return snd_rawmidi_free(rmidi);
 }
 
