@@ -20,46 +20,40 @@
  */
 
 #define SND_MAIN_OBJECT_FILE
-#include "driver.h"
+#include "../../include/driver.h"
 
-#include "seq_kernel.h"
+#include "../../include/seq_kernel.h"
 #include "seq_clientmgr.h"
 #include "seq_memory.h"
 #include "seq_queue.h"
+#include "seq_lock.h"
 #include "seq_timer.h"
 #include "seq_system.h"
 #include "seq_info.h"
+#include "../../include/seq_device.h"
 
+int snd_seq_client_load[64] = {63, [1 ... 63] = -1};
 int snd_seq_default_timer = SND_TIMER_TYPE_GLOBAL | SND_TIMER_GLOBAL_SYSTEM;
 int snd_seq_default_timer_resolution = 250;	/* Hz */
 
-#ifdef MODULE_PARM		/* hey - we have new 2.1.18+ kernel... */
 MODULE_AUTHOR("Frank van de Pol <frank@vande-pol.demon.nl>, Jaroslav Kysela <perex@suse.cz>");
 MODULE_DESCRIPTION("Advanced Linux Sound Architecture sequencer.");
 MODULE_SUPPORTED_DEVICE("sound");
 
+MODULE_PARM(snd_seq_client_load, "i");
+MODULE_PARM_DESC(snd_seq_client_load, "The numbers of global (system) clients to load through kmod.");
 MODULE_PARM(snd_seq_default_timer, "i");
 MODULE_PARM_DESC(snd_seq_default_timer, "The default timer number.");
 MODULE_PARM(snd_seq_default_timer_resolution, "i");
 MODULE_PARM_DESC(snd_seq_default_timer_resolution, "The default timer resolution in Hz.");
-#endif
-
 
 /*
  *  INIT PART
  */
 
 
-#ifndef LINUX_2_1
-extern struct symbol_table snd_symbol_table_seq_export;
-#endif
-
 int init_module(void)
 {
-#ifndef LINUX_2_1
-	if (register_symtab(&snd_symbol_table_seq_export) < 0)
-		return -ENOMEM;
-#endif
 	client_init_data();
 
 	/* init memory, room for selected events */
@@ -77,6 +71,9 @@ int init_module(void)
 	/* register our internal client */
 	snd_seq_system_client_init();
 
+	/* load the registered drivers */
+	snd_seq_device_load_drivers();
+
 	return 0;
 }
 
@@ -99,3 +96,22 @@ void cleanup_module(void)
 	snd_sequencer_memory_done();
 }
 
+  /* seq_lock.c */
+EXPORT_SYMBOL(snd_seq_sleep_in_lock);
+EXPORT_SYMBOL(snd_seq_sleep_timeout_in_lock);
+  /* seq_clientmgr.c */
+EXPORT_SYMBOL(snd_seq_create_kernel_client);
+EXPORT_SYMBOL(snd_seq_delete_kernel_client);
+EXPORT_SYMBOL(snd_seq_kernel_client_enqueue);
+EXPORT_SYMBOL(snd_seq_kernel_client_enqueue_blocking);
+EXPORT_SYMBOL(snd_seq_kernel_client_dispatch);
+EXPORT_SYMBOL(snd_seq_kernel_client_ctl);
+EXPORT_SYMBOL(snd_seq_kernel_client_write_poll);
+  /* seq_memory.c */
+EXPORT_SYMBOL(snd_seq_ext_malloc);
+EXPORT_SYMBOL(snd_seq_ext_free);
+  /* seq_ports.c */
+EXPORT_SYMBOL(snd_port_init_callback);
+EXPORT_SYMBOL(snd_port_alloc_callback);
+EXPORT_SYMBOL(snd_seq_event_port_attach);
+EXPORT_SYMBOL(snd_seq_event_port_detach);
