@@ -21,7 +21,7 @@
 #ifndef __SND_SEQ_CLIENTMGR_H
 #define __SND_SEQ_CLIENTMGR_H
 
-#include "seq_kernel.h"
+#include "../../include/seq_kernel.h"
 #include "seq_fifo.h"
 #include "seq_ports.h"
 
@@ -32,8 +32,9 @@ struct snd_seq_stru_user_client {
 	struct file *file;	/* file struct of client */
 	/* ... */
 	
-	/* locks */
-	snd_spin_define(input_sleep);
+	/* fifo */
+	fifo_t *fifo;	/* queue for incoming events */
+	int fifo_pool_size;
 };
 
 struct snd_seq_stru_kernel_client {
@@ -53,20 +54,21 @@ struct snd_seq_stru_client {
 	unsigned int filter;	/* filter flags */
 	unsigned char multicast_filter[8];
 	unsigned char event_filter[32];
-	fifo_t *outqueue;	/* queue for outgoing (to client) data */
 	unsigned int use;	/* port use counter */
-	snd_spin_define(use);
-	snd_sleep_define(use);
-	/* watermarks */
-	int low[SND_SEQ_MAX_QUEUES];
-	int high[SND_SEQ_MAX_QUEUES];
+	spinlock_t use_lock;
+	wait_queue_head_t use_sleep;
 	/* ports */
 	client_port_t *ports;	/* exported ports */
 	unsigned int ports_use;	/* counter for exported ports use */
-	snd_spin_define(ports);	/* lock for exported ports */
-	snd_sleep_define(ports); /* sleeper for exported ports */
-	/* masks */
-	/* filters */
+	spinlock_t ports_lock;	/* lock for exported ports */
+	wait_queue_head_t ports_sleep; /* sleeper for exported ports */
+	/* output pool */
+	pool_t *pool;		/* memory pool for this client */
+	int pool_size;		/* pool size to be allocated */
+	int pool_room;
+	wait_queue_head_t output_sleep;
+	spinlock_t output_sleep_lock;
+
 	union {
 		user_client_t user;
 		kernel_client_t kernel;
@@ -91,11 +93,13 @@ extern client_t *snd_seq_client_use_ptr(int clientid);
 extern void snd_seq_client_unlock(client_t *client);
 
 /* dispatch event to client(s) */
-extern void snd_seq_dispatch_event(snd_seq_event_cell_t *cell, int atomic);
+extern int snd_seq_dispatch_event(snd_seq_event_cell_t *cell, int atomic);
 
 /* exported to other modules */
 extern int snd_seq_register_kernel_client(snd_seq_client_callback_t *callback, void *private_data);
 extern int snd_seq_unregister_kernel_client(int client);
 extern int snd_seq_kernel_client_enqueue(int client, snd_seq_event_t *ev, int atomic);
+int snd_seq_kernel_client_enqueue_blocking(int client, snd_seq_event_t * ev, struct file *file, int atomic);
+int snd_seq_kernel_client_write_poll(int clientid, struct file *file, poll_table *wait);
 
 #endif
