@@ -27,10 +27,11 @@ Possible options for midisynth module:
 
 
 #define SND_MAIN_OBJECT_FILE
-#include "driver.h"
-#include "midi.h"
-#include "seq_kernel.h"
-#include "seq_midi.h"
+#include "../../include/driver.h"
+#include "../../include/midi.h"
+#include "../../include/seq_kernel.h"
+#include "../../include/seq_midi.h"
+#include "../../include/seq_device.h"
 
 
 /* data for this midi synth driver */
@@ -52,7 +53,7 @@ typedef struct {
 } seq_midisynth_client_t;
 
 static seq_midisynth_client_t *synths[SND_CARDS];
-snd_mutex_define_static(register);
+static DECLARE_MUTEX(register_mutex);
 
 /* fill standard header data, source port & channel are filled in */
 static void snd_seq_midi_setheader(snd_seq_event_t * ev, int port, int channel)
@@ -357,7 +358,7 @@ static int event_process_midi(snd_seq_event_t * ev, int direct, void *private_da
 	int channel = ev->dest.channel, client;
 	unsigned char msg[10];	/* buffer for constructing midi messages */
 	snd_rawmidi_t *rmidi;
-	snd_seq_event_t event;
+	unsigned char *sysex;
 
 	if (msynth == NULL) {
 		snd_printd("msynth == NULL\n");
@@ -389,34 +390,9 @@ static int event_process_midi(snd_seq_event_t * ev, int direct, void *private_da
 	/* decode actual event data... */
 	switch (ev->type) {
 		case SND_SEQ_EVENT_NOTE:
-			/* note event with specified length, first trigger note on */
-			msg[0] = (channel & 0x0f) | 0x90;	/* note on */
-			msg[1] = ev->data.note.note & 0x7f;
-			msg[2] = ev->data.note.velocity & 0x7f;
-			if (msynth->running_state == msg[0]) {
-				dump_midi(rmidi, msg + 1, 2);
-			} else {
-				msynth->running_state = msg[0];
-				dump_midi(rmidi, msg, 3);
-			}
-
-			/* enqueue note off event */
-			memcpy(&event, ev, sizeof(event));
-			switch (event.flags & SND_SEQ_TIME_STAMP_MASK) {
-				case SND_SEQ_TIME_STAMP_TICK:
-					event.time.tick = event.data.note.duration;
-					break;
-				case SND_SEQ_TIME_STAMP_REAL:
-					event.time.real.tv_sec = event.data.note.duration / 1000;	/* unit for duration is ms */
-					event.time.real.tv_nsec = 1000000 * (event.data.note.duration % 1000);
-					break;
-			}
-			event.flags = (event.flags & ~SND_SEQ_TIME_MODE_MASK) | SND_SEQ_TIME_MODE_REL;
-			event.type = SND_SEQ_EVENT_NOTEOFF;
-			snd_seq_kernel_client_enqueue(client, &event, 1);
+			/* should be processed before queued */
 			break;
 
-
 		case SND_SEQ_EVENT_NOTEOFF:
 			msg[0] = (channel & 0x0f) | 0x80;	/* note off */
 			msg[1] = ev->data.note.note & 0x7f;
@@ -501,13 +477,17 @@ static int event_process_midi(snd_seq_event_t * ev, int direct, void *private_da
 
 
 
-		case SND_SEQ_EVENT_SYSEX:{
-				unsigned char *sysex = (unsigned char *) ev->data.ext.ptr;
-
-				//printf("Event  = System Exclusive len=%d\n", ev->data.ext.len);
-				dump_midi(rmidi, sysex, ev->data.ext.len);
+		case SND_SEQ_EVENT_SYSEX:
+			if ((ev->flags & SND_SEQ_EVENT_LENGTH_MASK) != SND_SEQ_EVENT_LENGTH_VARIABLE) {
+				/* invalid event */
+				snd_printd("seq_midi: invalid sysex event flags = 0x%x\n", ev->flags);
 				msynth->running_state = 0;
+				break;
 			}
+			sysex = (unsigned char *) ev->data.ext.ptr;
+			if (sysex)
+				dump_midi(rmidi, sysex, ev->data.ext.len);
+			msynth->running_state = 0;
 			break;
 
 
@@ -567,7 +547,7 @@ static seq_midisynth_t *snd_seq_midisynth_new(snd_card_t *card, int device)
 {
 	seq_midisynth_t *msynth;
 	
-	msynth = snd_calloc(sizeof(seq_midisynth_t));
+	msynth = snd_kcalloc(sizeof(seq_midisynth_t), GFP_KERNEL);
 	if (msynth == NULL) {
 		snd_printd("malloc failed for snd_seq_midisynth_new()\n");
 		return NULL;
@@ -580,13 +560,19 @@ static seq_midisynth_t *snd_seq_midisynth_new(snd_card_t *card, int device)
 }
 
 /* open associated midi device for input */
-static int midisynth_subscribe(void *private_data)
+static int midisynth_subscribe(void *private_data, snd_seq_port_subscribe_t *info)
 {
 	int err;
 	seq_midisynth_t *msynth = (seq_midisynth_t *)private_data;
 
 	MOD_INC_USE_COUNT;
 
+	if ((info->midi_channels && info->midi_channels != 16) ||
+	    info->midi_voices || info->synth_voices) {
+	    	MOD_DEC_USE_COUNT;
+	    	return -EINVAL;
+	}
+
 	/* open midi port */
 	if ((err = snd_midi_open(msynth->card->number, msynth->device, SND_RAWMIDI_LFLG_INPUT, &msynth->input_rmidi)) < 0) {
 		snd_printd("midi input open failed!!!\n");
@@ -600,7 +586,7 @@ static int midisynth_subscribe(void *private_data)
 }
 
 /* close associated midi device for input */
-static int midisynth_unsubscribe(void *private_data)
+static int midisynth_unsubscribe(void *private_data, snd_seq_port_subscribe_t *info)
 {
 	int err;
 	seq_midisynth_t *msynth = (seq_midisynth_t *)private_data;
@@ -614,13 +600,19 @@ static int midisynth_unsubscribe(void *private_data)
 }
 
 /* open associated midi device for output */
-static int midisynth_use(void *private_data)
+static int midisynth_use(void *private_data, snd_seq_port_subscribe_t *info)
 {
 	int err;
 	seq_midisynth_t *msynth = (seq_midisynth_t *)private_data;
 
 	MOD_INC_USE_COUNT;
 
+	if ((info->midi_channels && info->midi_channels != 16) ||
+	    info->midi_voices || info->synth_voices) {
+	    	MOD_DEC_USE_COUNT;
+	    	return -EINVAL;
+	}
+
 	/* open midi port */
 	if ((err = snd_midi_open(msynth->card->number, msynth->device, SND_RAWMIDI_LFLG_OUTPUT, &msynth->output_rmidi)) < 0) {
 		snd_printd("midi output open failed!!!\n");
@@ -631,7 +623,7 @@ static int midisynth_use(void *private_data)
 }
 
 /* close associated midi device for output */
-static int midisynth_unuse(void *private_data)
+static int midisynth_unuse(void *private_data, snd_seq_port_subscribe_t *info)
 {
 	int err;
 	seq_midisynth_t *msynth = (seq_midisynth_t *)private_data;
@@ -670,13 +662,16 @@ static void snd_seq_midisynth_delete(seq_midisynth_t **ms)
 	port.port = msynth->seq_port;
 	snd_seq_kernel_client_ctl(port.client, SND_SEQ_IOCTL_DELETE_PORT, &port);
 
-	snd_free(msynth,sizeof(seq_midisynth_t));		
+	snd_kfree(msynth);
 }
 
 
 /* register new midi synth port */
 /* if portname = NULL, the name will be fetched from the MIDI device name */
-int snd_seq_midisynth_register_port(snd_card_t *card, int device, char *portname)
+int
+snd_seq_midisynth_register_port(snd_card_t *card, int device, char *portname,
+				char *id, void *arg, int size,
+				snd_seq_dev_entry_t *result)
 {
 	seq_midisynth_client_t *client;
 	seq_midisynth_t *msynth;
@@ -689,13 +684,13 @@ int snd_seq_midisynth_register_port(snd_card_t *card, int device, char *portname
 
 	if (!card || device < 0 || device >= SND_RAWMIDI_DEVICES)
 		return -EINVAL;
-	snd_mutex_down_static(register);
+	down(&register_mutex);
 	client = synths[card->number];
 	if (!client) {
 		newclient = 1;
-		client = snd_calloc(sizeof(seq_midisynth_client_t));
+		client = snd_kcalloc(sizeof(seq_midisynth_client_t), GFP_KERNEL);
 		if (!client) {
-			snd_mutex_up_static(register);
+			up(&register_mutex);
 			return -ENOMEM;
 		}
 		memset(&callbacks, 0, sizeof(callbacks));
@@ -703,8 +698,8 @@ int snd_seq_midisynth_register_port(snd_card_t *card, int device, char *portname
 		callbacks.allow_input = callbacks.allow_output = 1;
 		client->seq_client = snd_seq_create_kernel_client(card, 0, &callbacks);
 		if (client->seq_client < 0) {
-			snd_free(client, sizeof(seq_midisynth_client_t));
-			snd_mutex_up_static(register);
+			snd_kfree(client);
+			up(&register_mutex);
 			return -ENOMEM;
 		}
 		/* set our client name */
@@ -718,9 +713,9 @@ int snd_seq_midisynth_register_port(snd_card_t *card, int device, char *portname
 	if (!client->ports[device] || snd_midi_info(card->number, device, &info)<0) {
 		if (newclient) {
 			snd_seq_delete_kernel_client(client->seq_client);
-			snd_free(client, sizeof(seq_midisynth_client_t));
+			snd_kfree(client);
 		}
-		snd_mutex_up_static(register);
+		up(&register_mutex);
 		return -EBUSY;
 	}
 	msynth->seq_client = client->seq_client;
@@ -754,29 +749,32 @@ int snd_seq_midisynth_register_port(snd_card_t *card, int device, char *portname
 		snd_seq_midisynth_delete(&client->ports[device]);
 		if (newclient) {
 			snd_seq_delete_kernel_client(client->seq_client);
-			snd_free(client, sizeof(seq_midisynth_client_t));
+			snd_kfree(client);
 		}
-		snd_mutex_up_static(register);
+		up(&register_mutex);
 		return -ENOMEM;
 	}
 
 	msynth->seq_port = port.port;
 	if (newclient)
 		synths[card->number] = client;
-	snd_mutex_up_static(register);
+	up(&register_mutex);
+	*result = (void*)1; /* set dummy value */
 	return 0;	/* success */
 }
 
 /* release midi synth port */
-int snd_seq_midisynth_unregister_port(snd_card_t *card, int device)
+int
+snd_seq_midisynth_unregister_port(snd_card_t *card, int device, void *arg,
+				  int size, snd_seq_dev_entry_t rec)
 {
 	seq_midisynth_client_t *client;
 	int c;
 	
-	snd_mutex_down_static(register);
+	down(&register_mutex);
 	client = synths[card->number];
 	if (!client) {
-		snd_mutex_up_static(register);
+		up(&register_mutex);
 		return -ENODEV;
 	}
 	snd_seq_event_port_detach(client->seq_client, client->ports[device]->seq_port);
@@ -787,35 +785,31 @@ int snd_seq_midisynth_unregister_port(snd_card_t *card, int device)
 	if (c >= SND_RAWMIDI_DEVICES) {
 		snd_seq_delete_kernel_client(client->seq_client);
 		synths[card->number] = NULL;
-		snd_free(client, sizeof(seq_midisynth_client_t));
+		snd_kfree(client);
 	}
-	snd_mutex_up_static(register);
+	up(&register_mutex);
 	return 0;
 }
 
-#ifndef LINUX_2_1
-extern struct symbol_table snd_symbol_table_seq_midi_export;
-#endif
-
 
 int init_module(void)
 {
-#ifndef LINUX_2_1
-	if (register_symtab(&snd_symbol_table_seq_midi_export) < 0)
-		return -ENOMEM;
-#endif
+	static snd_seq_dev_ops_t ops = {
+		snd_seq_midisynth_register_port,
+		snd_seq_midisynth_unregister_port,
+	};
 	memset(&synths, 0, sizeof(synths));
+	snd_seq_device_register_driver(SND_SEQ_DEV_MIDISYNTH, &ops);
 	return 0;
 }
 
 
 void cleanup_module(void)
 {
+	snd_seq_device_unregister_driver(SND_SEQ_DEV_MIDISYNTH);
 }
 
 
-#ifdef MODULE_PARM		/* hey - we have new 2.1.18+ kernel... */
 MODULE_AUTHOR("Frank van de Pol <frank@vande-pol.demon.nl>");
 MODULE_DESCRIPTION("Advanced Linux Sound Architecture sequencer MIDI synth.");
 MODULE_SUPPORTED_DEVICE("sound");
-#endif
