@@ -30,15 +30,13 @@
  */
 
 #define SND_MAIN_OBJECT_FILE
-#include "driver.h"
-#include "seq_kernel.h"
-#include "seq_midi_emul.h"
+#include "../../include/driver.h"
+#include "../../include/seq_kernel.h"
+#include "../../include/seq_midi_emul.h"
 
-#ifdef MODULE_PARM		/* hey - we have new 2.1.18+ kernel... */
 MODULE_AUTHOR("Takashi Iwai / Steve Ratcliffe");
 MODULE_DESCRIPTION("Advanced Linux Sound Architecture sequencer MIDI emulation.");
 MODULE_SUPPORTED_DEVICE("sound");
-#endif
 
 /* Prototypes for static functions */
 static void note_off(snd_midi_op_t *ops, void *drv, snd_midi_channel_t *chan, int note, int vel);
@@ -48,8 +46,6 @@ static void do_control(snd_midi_op_t *ops, void *private,
 static void rpn(snd_midi_op_t *ops, void *drv, snd_midi_channel_t *chan, snd_midi_channel_set_t *chset);
 static void nrpn(snd_midi_op_t *ops, void *drv, snd_midi_channel_t *chan, snd_midi_channel_set_t *chset);
 static void sysex(snd_midi_op_t *ops, void *private, unsigned char *sysex, int len, snd_midi_channel_set_t *chset);
-static void pre_note_proc(int client, snd_midi_channel_t *chan,
-	 snd_seq_event_t *ev);
 static void all_sounds_off(snd_midi_op_t *ops, void *private, snd_midi_channel_t *chan);
 static void all_notes_off(snd_midi_op_t *ops, void *private, snd_midi_channel_t *chan);
 
@@ -90,12 +86,9 @@ snd_midi_process_event(snd_midi_op_t *ops,
 	chan = chanset->channels + ev->dest.channel;
 	drv  = chanset->private_data;
 
-	/*
-	 * First deal with EVENT_NOTE, this is converted into a NOTE_ON
-	 * and the corresponding NOTE_OFF, which is queued.
-	 */
+	/* EVENT_NOTE should be processed before queued */
 	if (ev->type == SND_SEQ_EVENT_NOTE)
-		pre_note_proc(chanset->client, chan, ev);
+		return;
 
 	/* Make sure that we don't have a note on that should really be
 	 * a note off */
@@ -113,13 +106,16 @@ snd_midi_process_event(snd_midi_op_t *ops,
 	switch (ev->type) {
 	case SND_SEQ_EVENT_NOTEON:
 		chan->note[ev->data.note.note] = SND_MIDI_NOTE_ON;
-		ops->note_on(drv, ev->data.note.note, ev->data.note.velocity, chan);
+		if (ops->note_on)
+			ops->note_on(drv, ev->data.note.note, ev->data.note.velocity, chan);
 		break;
 	case SND_SEQ_EVENT_NOTEOFF:
-		note_off(ops, drv, chan, ev->data.note.note, ev->data.note.velocity);
+		if (ops->note_off)
+			note_off(ops, drv, chan, ev->data.note.note, ev->data.note.velocity);
 		break;
 	case SND_SEQ_EVENT_KEYPRESS:
-		ops->key_press(drv, ev->data.note.note, ev->data.note.velocity, chan);
+		if (ops->key_press)
+			ops->key_press(drv, ev->data.note.note, ev->data.note.velocity, chan);
 		break;
 	case SND_SEQ_EVENT_CONTROLLER:
 		do_control(ops, drv, chanset, chan,
@@ -130,11 +126,13 @@ snd_midi_process_event(snd_midi_op_t *ops,
 		break;
 	case SND_SEQ_EVENT_PITCHBEND:
 		chan->midi_pitchbend = ev->data.control.value;
-		ops->control(drv, SND_MCTL_PITCHBEND, chan);
+		if (ops->control)
+			ops->control(drv, SND_MCTL_PITCHBEND, chan);
 		break;
 	case SND_SEQ_EVENT_CHANPRESS:
 		chan->midi_pressure = ev->data.control.value;
-		ops->control(drv, SND_MCTL_CHAN_PRESSURE, chan);
+		if (ops->control)
+			ops->control(drv, SND_MCTL_CHAN_PRESSURE, chan);
 		break;
 	case SND_SEQ_EVENT_CONTROL14:
 		/* Best guess is that this is any of the 14 bit controller values */
@@ -177,6 +175,8 @@ snd_midi_process_event(snd_midi_op_t *ops,
 		rpn(ops, drv, chan, chanset);
 		break;
 	case SND_SEQ_EVENT_SYSEX:
+		if ((ev->flags & SND_SEQ_EVENT_LENGTH_MASK) != SND_SEQ_EVENT_LENGTH_VARIABLE)
+			break;
 		sysex(ops, drv, (unsigned char*)ev->data.ext.ptr,
 		      ev->data.ext.len, chanset);
 		break;
@@ -230,7 +230,8 @@ note_off(snd_midi_op_t *ops, void *drv, snd_midi_channel_t *chan, int note, int
 		chan->note[note] |= SND_MIDI_NOTE_RELEASED;
 	} else {
 		chan->note[note] = 0;
-		ops->note_off(drv, note, vel, chan);
+		if (ops->note_off)
+			ops->note_off(drv, note, vel, chan);
 	}
 }
 
@@ -258,7 +259,8 @@ do_control(snd_midi_op_t *ops, void *drv, snd_midi_channel_set_t *chset,
 			for (i = 0; i < 128; i++) {
 				if (chan->note[i] & SND_MIDI_NOTE_RELEASED) {
 					chan->note[i] = SND_MIDI_NOTE_OFF;
-					ops->note_off(drv, i, 0, chan);
+					if (ops->note_off)
+						ops->note_off(drv, i, 0, chan);
 				}
 			}
 		}
@@ -279,7 +281,8 @@ do_control(snd_midi_op_t *ops, void *drv, snd_midi_channel_set_t *chset,
 					chan->note[i] &= ~SND_MIDI_NOTE_SUSTENUTO;
 					if (chan->note[i] & SND_MIDI_NOTE_RELEASED) {
 						chan->note[i] = SND_MIDI_NOTE_OFF;
-						ops->note_off(drv, i, 0, chan);
+						if (ops->note_off)
+							ops->note_off(drv, i, 0, chan);
 					}
 				}
 			}
@@ -311,6 +314,17 @@ do_control(snd_midi_op_t *ops, void *drv, snd_midi_channel_set_t *chset,
 		all_notes_off(ops, drv, chan);
 		break;
 
+	case SND_MCTL_MSB_BANK:
+		if (chset->midi_mode == SND_MIDI_MODE_XG) {
+			if (value == 127)
+				chan->drum_channel = 1;
+			else
+				chan->drum_channel = 0;
+		}
+		break;
+	case SND_MCTL_LSB_BANK:
+		break;
+
 	case SND_MCTL_SOFT_PEDAL:
 	case SND_MCTL_LEGATO_FOOTSWITCH:
 	case SND_MCTL_HOLD2:
@@ -415,7 +429,8 @@ nrpn(snd_midi_op_t *ops, void *drv, snd_midi_channel_t *chan,
      snd_midi_channel_set_t *chset)
 {
 	/* parse XG NRPNs here if possible */
-	ops->nrpn(drv, chan, chset);
+	if (ops->nrpn)
+		ops->nrpn(drv, chan, chset);
 }
 
 
@@ -543,38 +558,8 @@ sysex(snd_midi_op_t *ops, void *private, unsigned char *buf, int len, snd_midi_c
 		}
 	}
 
-	ops->sysex(private, buf - 1, len + 1, parsed, chset);
-}
-
-/*
- * Deal with the NOTE event.  This is converted into a note on event
- * and a corresponding note off event.
- */
-static void
-pre_note_proc(int client, snd_midi_channel_t *chan, snd_seq_event_t *ev)
-{
-	snd_seq_event_t offev;
-
-	/* Convert this one */
-	ev->type = SND_SEQ_EVENT_NOTEON;
-
-	/* Enqueue the note off first, to make sure */
-	memcpy(&offev, ev, sizeof(offev));
-	switch (offev.flags & SND_SEQ_TIME_STAMP_MASK) {
-	case SND_SEQ_TIME_STAMP_TICK:
-		offev.time.tick = offev.data.note.duration;
-		break;
-	case SND_SEQ_TIME_STAMP_REAL:
-		/* unit for duration is ms */
-		offev.time.real.tv_sec = ev->data.note.duration / 1000;
-		offev.time.real.tv_nsec =
-			1000000 * (ev->data.note.duration % 1000);
-		break;
-	}
-
-	offev.flags = (offev.flags & ~SND_SEQ_TIME_MODE_MASK) | SND_SEQ_TIME_MODE_REL;
-	offev.type = SND_SEQ_EVENT_NOTEOFF;
-	snd_seq_kernel_client_enqueue(client, &offev, 1);
+	if (ops->sysex)
+		ops->sysex(private, buf - 1, len + 1, parsed, chset);
 }
 
 /*
@@ -640,7 +625,7 @@ snd_midi_channel_t *snd_midi_channel_init_set(int n)
 	snd_midi_channel_t *chan;
 	int  i;
 
-	chan = snd_malloc(n * sizeof(snd_midi_channel_t));
+	chan = snd_kmalloc(n * sizeof(snd_midi_channel_t), GFP_KERNEL);
 	if (chan) {
 		for (i = 0; i < n; i++)
 			snd_midi_channel_init(chan+i, i);
@@ -656,7 +641,7 @@ snd_midi_channel_set_t *snd_midi_channel_alloc_set(int n)
 {
 	snd_midi_channel_set_t *chset;
 
-	chset = snd_malloc(sizeof(*chset));
+	chset = snd_kmalloc(sizeof(*chset), GFP_KERNEL);
 	chset->channels = snd_midi_channel_init_set(n);
 
 	chset->private_data = NULL;
@@ -672,21 +657,12 @@ void snd_midi_channel_free_set(snd_midi_channel_set_t *chset)
 	if (chset == NULL)
 		return;
 	if (chset->channels != NULL)
-		snd_free(chset->channels, sizeof(snd_midi_channel_t) *
-					  chset->max_channels);
-	snd_free(chset, sizeof(*chset));
+		snd_kfree(chset->channels);
+	snd_kfree(chset);
 }
 
-#ifndef LINUX_2_1
-extern struct symbol_table snd_symbol_table_seq_midi_emul_export;
-#endif
-
 int init_module(void)
 {
-#ifndef LINUX_2_1
-	if (register_symtab(&snd_symbol_table_seq_midi_emul_export) < 0)
-		return -ENOMEM;
-#endif
 	return 0;
 }
 
@@ -695,3 +671,10 @@ void cleanup_module(void)
 {
 }
 
+
+EXPORT_SYMBOL(snd_midi_process_event);
+EXPORT_SYMBOL(snd_midi_channel_set_clear);
+EXPORT_SYMBOL(snd_midi_channel_init);
+EXPORT_SYMBOL(snd_midi_channel_init_set);
+EXPORT_SYMBOL(snd_midi_channel_alloc_set);
+EXPORT_SYMBOL(snd_midi_channel_free_set);
