@@ -20,10 +20,11 @@
  *
  */
 
-#include "driver.h"
+#include "../../include/driver.h"
 #include "seq_system.h"
 #include "seq_ports.h"
 #include "seq_clientmgr.h"
+#include "seq_lock.h"
 
 /*
 
@@ -64,9 +65,9 @@ void snd_seq_subscribers_lock(subscribers_group_t *group)
 
 	if (!group)
 		return;
-	snd_spin_lock(group, use, &flags);
+	spin_lock_irqsave(&group->use_lock, flags);
 	group->use++;
-	snd_spin_unlock(group, use, &flags);
+	spin_unlock_irqrestore(&group->use_lock, flags);
 }
 
 /* unlock subscribers (and wakeup) - to allow create/delete operations */
@@ -76,26 +77,21 @@ void snd_seq_subscribers_unlock(subscribers_group_t *group)
 
 	if (!group)
 		return;
-	snd_spin_lock(group, use, &flags);
-	if (!(--group->use) && (snd_getlock(group, use) & SND_WK_SLEEP)) {
-		snd_getlock(group, use) &= ~SND_WK_SLEEP;
-		snd_wakeup(group, use);
-	}
-	snd_spin_unlock(group, use, &flags);
+	spin_lock_irqsave(&group->use_lock, flags);
+	if (!(--group->use) && waitqueue_active(&group->use_sleep))
+		wake_up(&group->use_sleep);
+	spin_unlock_irqrestore(&group->use_lock, flags);
 }
 
 /* lock subscribers and or wait for wakeup */
-static void snd_seq_subscribers_wait(subscribers_group_t *group, unsigned long *flags)
+static void snd_seq_subscribers_wait(subscribers_group_t *group, unsigned long *rflags)
 {
-	if (!group || !flags)
+
+	if (!group || !rflags)
 		return;
-	snd_spin_lock(group, use, flags);
+	spin_lock_irqsave(&group->use_lock, *rflags);
 	while (group->use) {
-		snd_getlock(group, use) |= SND_WK_SLEEP;
-		snd_spin_unlock(group, use, flags);
-		snd_sleep(group, use, HZ);
-		snd_spin_lock(group, use, flags);
-		snd_getlock(group, use) &= ~SND_WK_SLEEP;
+		snd_seq_sleep_in_lock(&group->use_sleep, &group->use_lock);
 	}
 }
 
@@ -105,15 +101,15 @@ static int snd_seq_subscribers_delete(subscribers_group_t *group)
 	unsigned long flags;
 	subscribers_t *s, *o;
 
-	snd_spin_lock(group, use, &flags);
+	spin_lock_irqsave(&group->use_lock, flags);
 	s = group->list;
 	group->list = NULL;
-	snd_spin_unlock(group, use, &flags);
+	spin_unlock_irqrestore(&group->use_lock, flags);
 	/* release resources...*/
 	while (s) {
 		o = s;
 		s = s->next;
-		snd_free(o, sizeof(subscribers_t));
+		snd_kfree(o);
 	}
 	return 0;	/* success */
 }
@@ -128,9 +124,9 @@ void snd_seq_ports_lock(client_t *client)
 
 	if (!client)
 		return;
-	snd_spin_lock(client, ports, &flags);
+	spin_lock_irqsave(&client->ports_lock, flags);
 	client->ports_use++;
-	snd_spin_unlock(client, ports, &flags);
+	spin_unlock_irqrestore(&client->ports_lock, flags);
 }
 
 /* unlock ports (and wakeup) - to allow create/delete operations */
@@ -140,27 +136,21 @@ void snd_seq_ports_unlock(client_t *client)
 
 	if (!client)
 		return;
-	snd_spin_lock(client, ports, &flags);
-	if (!(--client->ports_use) &&
-	    (snd_getlock(client, ports) & SND_WK_SLEEP)) {
-		snd_getlock(client, ports) &= ~SND_WK_SLEEP;
-		snd_wakeup(client, ports);
-	}
-	snd_spin_unlock(client, ports, &flags);
+	spin_lock_irqsave(&client->ports_lock, flags);
+	if (!(--client->ports_use) && waitqueue_active(&client->ports_sleep))
+		wake_up(&client->ports_sleep);
+	spin_unlock_irqrestore(&client->ports_lock, flags);
 }
 
 /* lock ports and or wait for wakeup */
-static void snd_seq_ports_wait(client_t *client, unsigned long *flags)
+static void snd_seq_ports_wait(client_t *client, unsigned long *rflags)
 {
-	if (!client)
+
+	if (!client || !rflags)
 		return;
-	snd_spin_lock(client, ports, flags);
+	spin_lock_irqsave(&client->ports_lock, *rflags);
 	while (client->ports_use) {
-		snd_getlock(client, ports) |= SND_WK_SLEEP;
-		snd_spin_unlock(client, ports, flags);
-		snd_sleep(client, ports, HZ);
-		snd_spin_lock(client, ports, flags);
-		snd_getlock(client, ports) &= ~SND_WK_SLEEP;
+		snd_seq_sleep_in_lock(&client->ports_sleep, &client->ports_lock);
 	}
 }
 
@@ -201,7 +191,7 @@ client_port_t *snd_seq_create_port(client_t *client)
 	}
 
 	/* create a new port */
-	new_port = snd_calloc(sizeof(client_port_t));
+	new_port = snd_kcalloc(sizeof(client_port_t), GFP_KERNEL);
 	if (new_port) {
 		new_port->port = -1;
 		strcpy(new_port->name, "");
@@ -213,16 +203,16 @@ client_port_t *snd_seq_create_port(client_t *client)
 
 	/* init port data */
 	sprintf(new_port->name, "port-%d", num);
-	snd_spin_prepare(&new_port->input, use);
-	snd_sleep_prepare(&new_port->input, use);
-	snd_spin_prepare(&new_port->itrack, use);
-	snd_sleep_prepare(&new_port->itrack, use);
-	snd_spin_prepare(&new_port->output, use);
-	snd_sleep_prepare(&new_port->output, use);
-	snd_spin_prepare(&new_port->otrack, use);
-	snd_sleep_prepare(&new_port->otrack, use);
-	snd_mutex_prepare(new_port, subscribe);
-	snd_mutex_prepare(new_port, use);
+	new_port->input.use_lock = SPIN_LOCK_UNLOCKED;
+	init_waitqueue_head(&new_port->input.use_sleep);
+	new_port->itrack.use_lock = SPIN_LOCK_UNLOCKED;
+	init_waitqueue_head(&new_port->itrack.use_sleep);
+	new_port->output.use_lock = SPIN_LOCK_UNLOCKED;
+	init_waitqueue_head(&new_port->output.use_sleep);
+	new_port->otrack.use_lock = SPIN_LOCK_UNLOCKED;
+	init_waitqueue_head(&new_port->otrack.use_sleep);
+	init_MUTEX(&new_port->subscribe_mutex);
+	init_MUTEX(&new_port->use_mutex);
 
 	/* wait if we can do some operation - spinlock is enabled!! */
 	snd_seq_ports_wait(client, &flags);
@@ -245,7 +235,7 @@ client_port_t *snd_seq_create_port(client_t *client)
 		num++;
 	}
 	new_port->port = num;	/* store the port number in the port */
-	snd_spin_unlock(client, ports, &flags);
+	spin_unlock_irqrestore(&client->ports_lock, flags);
 
 	return new_port;
 }
@@ -255,6 +245,7 @@ client_port_t *snd_seq_create_port(client_t *client)
 static int snd_seq_port_delete(client_t *client, client_port_t *port)
 {
 	snd_seq_addr_t addr;
+	snd_seq_port_subscribe_t subs;
 	subscribers_t *s;
 	client_t *c;
 	client_port_t *p;
@@ -274,10 +265,16 @@ static int snd_seq_port_delete(client_t *client, client_port_t *port)
 		p = snd_seq_port_use_ptr(c, s->addr.port);
 		if (!p)
 			continue;
+		memset(&subs, 0, sizeof(subs));
+		subs.sender.queue = subs.dest.queue = s->addr.queue;
+		subs.sender.client = s->addr.client;
+		subs.sender.port = s->addr.port;
+		subs.dest.client = client->number;
+		subs.dest.port = port->port;
+		snd_seq_port_unsubscribe(p, &subs);
 		addr.queue = s->addr.queue;
 		addr.client = client->number;
 		addr.port = port->port;
-		snd_seq_port_unsubscribe(p);
 		snd_seq_port_remove_subscriber(&p->input, &addr);
 		snd_seq_ports_unlock(c);
 		snd_seq_client_unlock(c);
@@ -291,10 +288,16 @@ static int snd_seq_port_delete(client_t *client, client_port_t *port)
 		p = snd_seq_port_use_ptr(c, s->addr.port);
 		if (!p)
 			continue;
+		memset(&subs, 0, sizeof(subs));
+		subs.dest.queue = subs.sender.queue = s->addr.queue;
+		subs.dest.client = s->addr.client;
+		subs.dest.port = s->addr.port;
+		subs.sender.client = client->number;
+		subs.sender.port = port->port;
+		snd_seq_port_unuse(p, &subs);
 		addr.queue = s->addr.queue;
 		addr.client = client->number;
 		addr.port = port->port;
-		snd_seq_port_unuse(p);
 		snd_seq_port_remove_subscriber(&p->output, &addr);
 		snd_seq_ports_unlock(c);
 		snd_seq_client_unlock(c);
@@ -304,7 +307,7 @@ static int snd_seq_port_delete(client_t *client, client_port_t *port)
 	if (port->private_free)
 		port->private_free(port->private_data);
 
-	snd_free(port, sizeof(client_port_t));
+	snd_kfree(port);
 	return 0;
 }
 
@@ -319,7 +322,7 @@ int snd_seq_delete_port(client_t *client, int port)
 	if ((p = client->ports) != NULL) {
 		if (p->port == port) {
 			client->ports = p->next;
-			snd_spin_unlock(client, ports, &flags);
+			spin_unlock_irqrestore(&client->ports_lock, flags);
 			return snd_seq_port_delete(client, p);
 		}
 		while (p->next) {
@@ -327,14 +330,14 @@ int snd_seq_delete_port(client_t *client, int port)
 				if (p->next->port == port) {
 					client_port_t *del = p->next;
 					p->next = del->next;
-					snd_spin_unlock(client, ports, &flags);
+					spin_unlock_irqrestore(&client->ports_lock, flags);
 					return snd_seq_port_delete(client, del);
 				}
 			}
 			p = p->next;
 		}
 	}
-	snd_spin_unlock(client, ports, &flags);
+	spin_unlock_irqrestore(&client->ports_lock, flags);
 	return -ENOENT; 	/* error, port not found */
 }
 
@@ -349,7 +352,7 @@ int snd_seq_delete_ports(client_t *client)
 	snd_seq_ports_wait(client, &flags);
 	p = client->ports;
 	client->ports = NULL;
-	snd_spin_unlock(client, ports, &flags);
+	spin_unlock_irqrestore(&client->ports_lock, flags);
 	while (p != NULL) {
 		next = p->next;
 		snd_seq_port_delete(client, p);
@@ -417,7 +420,7 @@ int snd_seq_port_add_subscriber(subscribers_group_t *group, snd_seq_addr_t *addr
 
 	if (!group || !addr)
 		return -EINVAL;
-	n = snd_calloc(sizeof(subscribers_t));
+	n = snd_kcalloc(sizeof(subscribers_t), GFP_KERNEL);
 	if (!n)
 		return -ENOMEM;
 	n->addr = *addr;
@@ -430,8 +433,8 @@ int snd_seq_port_add_subscriber(subscribers_group_t *group, snd_seq_addr_t *addr
 		group->exclusive = exclusive ? 1 : 0;
 	} else {
 		if (exclusive) {
-			snd_free(n, sizeof(subscribers_t));
-			snd_spin_unlock(group, use, &flags);
+			snd_kfree(n);
+			spin_unlock_irqrestore(&group->use_lock, flags);
 			return -EBUSY;
 		}
 		/* add to the end of the list */
@@ -440,7 +443,7 @@ int snd_seq_port_add_subscriber(subscribers_group_t *group, snd_seq_addr_t *addr
 		}
 		s->next = n;
 	}
-	snd_spin_unlock(group, use, &flags);
+	spin_unlock_irqrestore(&group->use_lock, flags);
 	return 0;
 }
 
@@ -465,8 +468,8 @@ int snd_seq_port_remove_subscriber(subscribers_group_t *group, snd_seq_addr_t *a
 			group->list = s->next;
 			if (group->exclusive)
 				group->exclusive = 0;
-			snd_free(s, sizeof(subscribers_t));
-			snd_spin_unlock(group, use, &flags);
+			snd_kfree(s);
+			spin_unlock_irqrestore(&group->use_lock, flags);
 			return 0;
 		}
 		p = s;
@@ -475,96 +478,100 @@ int snd_seq_port_remove_subscriber(subscribers_group_t *group, snd_seq_addr_t *a
 				p->next = s->next;
 				if (group->exclusive)
 					group->exclusive = 0;
-				snd_free(s, sizeof(subscribers_t));
-				snd_spin_unlock(group, use, &flags);
+				snd_kfree(s);
+				spin_unlock_irqrestore(&group->use_lock, flags);
 				return 0;
 			}
 			p = s;
 			s = s->next;
 		}
 	}
-	snd_spin_unlock(group, use, &flags);
+	spin_unlock_irqrestore(&group->use_lock, flags);
 	return -ENOENT;
 }
 
 
 /* subscribe port */
-int snd_seq_port_subscribe(client_port_t *port)
+int snd_seq_port_subscribe(client_port_t *port, snd_seq_port_subscribe_t *info)
 {
 	int result;
 
 	if (!port->subscribe)
 		return 0;
-	snd_mutex_down(port, subscribe);
+	down(&port->subscribe_mutex);
 	port->subscribe_count++;
 	if (port->subscribe_count > 1) {
-		snd_mutex_up(port, subscribe);
+		up(&port->subscribe_mutex);
 		return 0;
 	}
-	result = port->subscribe(port->private_data);
+	result = port->subscribe(port->private_data, info);
 	if (result < 0)
 		port->subscribe_count--;
-	snd_mutex_up(port, subscribe);
+	up(&port->subscribe_mutex);
 	return result;
 }
 
 /* unsubscribe port */
-int snd_seq_port_unsubscribe(client_port_t *port)
+int snd_seq_port_unsubscribe(client_port_t *port, snd_seq_port_subscribe_t *info)
 {
 	int result;
 
 	if (!port->subscribe)
 		return 0;
-	snd_mutex_down(port, subscribe);
+	down(&port->subscribe_mutex);
 	port->subscribe_count--;
 	if (port->subscribe_count > 0) {
-		snd_mutex_up(port, subscribe);
+		up(&port->subscribe_mutex);
 		return 0;
 	}
-	result = port->unsubscribe(port->private_data);
+	result = port->unsubscribe(port->private_data, info);
 	if (result < 0)
 		port->subscribe_count++;	/* wrong? */
-	snd_mutex_up(port, subscribe);
+	up(&port->subscribe_mutex);
 	return result;
 }
 
 /* use port */
-int snd_seq_port_use(client_port_t *port)
+int snd_seq_port_use(client_port_t *port, snd_seq_port_subscribe_t *info)
 {
 	int result;
 
 	if (!port->use)
 		return 0;
-	snd_mutex_down(port, use);
+	down(&port->use_mutex);
 	port->use_count++;
 	if (port->use_count > 1) {
-		snd_mutex_up(port, use);
+		up(&port->use_mutex);
 		return 0;
 	}
-	result = port->use(port->private_data);
+	result = port->use(port->private_data, info);
 	if (result < 0)
 		port->use_count--;
-	snd_mutex_up(port, use);
+	up(&port->use_mutex);
 	return result;
 }
 
 /* unuse port */
-int snd_seq_port_unuse(client_port_t *port)
+int snd_seq_port_unuse(client_port_t *port, snd_seq_port_subscribe_t *info)
 {
 	int result;
 
 	if (!port->use)
 		return 0;
-	snd_mutex_down(port, use);
-	port->use_count--;
-	if (port->use_count) {
-		snd_mutex_up(port, use);
+	down(&port->use_mutex);
+	if (port->use_count == 0) {
+		up(&port->use_mutex);
 		return 0;
 	}
-	result = port->unuse(port->private_data);
+	port->use_count--;
+	if (port->use_count) {
+		up(&port->use_mutex);
+		return 0;
+	}
+	result = port->unuse(port->private_data, info);
 	if (result < 0)
 		port->use_count++;	/* wrong? */
-	snd_mutex_up(port, use);
+	up(&port->use_mutex);
 	return result;
 }
 
@@ -591,7 +598,7 @@ snd_seq_port_callback_t * snd_port_alloc_callback(void)
 {
 	snd_seq_port_callback_t *p;
 
-	p = snd_malloc(sizeof(snd_seq_port_callback_t));
+	p = snd_kmalloc(sizeof(snd_seq_port_callback_t), GFP_KERNEL);
 	snd_port_init_callback(p);
 
 	return p;
