@@ -21,7 +21,7 @@
 #ifndef __SND_SEQ_PORTS_H
 #define __SND_SEQ_PORTS_H
 
-#include "seq_kernel.h"
+#include "../../include/seq_kernel.h"
 
 /* list of 'exported' ports */
 
@@ -50,8 +50,8 @@ typedef struct subscribers_group_t {
 	unsigned int count;		/* count of subscribers */
 	int exclusive: 1;		/* exclusive mode */
 	unsigned int use;		/* subscribers use counter */
-	snd_spin_define(use);		/* subscribers lock */
-	snd_sleep_define(use);		/* subscribers sleeper */
+	spinlock_t use_lock;		/* subscribers lock */
+	wait_queue_head_t use_sleep;	/* subscribers sleeper */
 } subscribers_group_t;
 
 typedef struct client_port_t {
@@ -68,11 +68,11 @@ typedef struct client_port_t {
 	snd_seq_kernel_port_subscribe_t *subscribe;
 	snd_seq_kernel_port_unsubscribe_t *unsubscribe;
 	unsigned int subscribe_count;
-	snd_mutex_define(subscribe);
+	struct semaphore subscribe_mutex;
 	snd_seq_kernel_port_use_t *use;
 	snd_seq_kernel_port_unuse_t *unuse;
 	unsigned int use_count;
-	snd_mutex_define(use);
+	struct semaphore use_mutex;
 	snd_seq_kernel_port_input_t *event_input;
 	snd_seq_kernel_port_private_free_t *private_free;
 	void *private_data;
@@ -124,15 +124,15 @@ extern int snd_seq_port_add_subscriber(subscribers_group_t *group, snd_seq_addr_
 extern int snd_seq_port_remove_subscriber(subscribers_group_t *group, snd_seq_addr_t *dest);
 
 /* subscribe port */
-int snd_seq_port_subscribe(client_port_t *port);
+int snd_seq_port_subscribe(client_port_t *port, snd_seq_port_subscribe_t *info);
 
 /* unsubscribe port */
-int snd_seq_port_unsubscribe(client_port_t *port);
+int snd_seq_port_unsubscribe(client_port_t *port, snd_seq_port_subscribe_t *info);
 
 /* use port */
-int snd_seq_port_use(client_port_t *port);
+int snd_seq_port_use(client_port_t *port, snd_seq_port_subscribe_t *info);
 
 /* unuse port */
-int snd_seq_port_unuse(client_port_t *port);
+int snd_seq_port_unuse(client_port_t *port, snd_seq_port_subscribe_t *info);
 
 #endif
