@@ -18,7 +18,7 @@
  *
  */
 
-#include "driver.h"
+#include "../../include/driver.h"
 
 #include "seq_memory.h"
 #include "seq_queue.h"
@@ -74,18 +74,17 @@ queue_t *snd_seq_queue_new(void)
 {
 	queue_t *q;
 
-	q = snd_calloc(sizeof(queue_t));
+	q = snd_kcalloc(sizeof(queue_t), GFP_KERNEL);
 	if (q == NULL) {
 		snd_printd("malloc failed for snd_seq_queue_new()\n");
 		return NULL;
 	}
 
-	snd_spin_prepare(q, owner);
-	snd_spin_prepare(q, clients);
-	snd_spin_prepare(q, check);
-	snd_mutex_prepare(q, use);
+	q->owner_lock = SPIN_LOCK_UNLOCKED;
+	q->clients_lock = SPIN_LOCK_UNLOCKED;
+	q->check_lock = SPIN_LOCK_UNLOCKED;
+	init_MUTEX(&q->use_mutex);
 
-	q->pool = snd_seq_pool_new();
 	q->tickq = snd_seq_prioq_new();
 	q->timeq = snd_seq_prioq_new();
 	q->timer = snd_seq_timer_new();
@@ -116,9 +115,8 @@ void snd_seq_queue_delete(queue_t ** queue)
 	snd_seq_prioq_delete(&q->tickq);
 	snd_seq_prioq_delete(&q->timeq);
 	snd_seq_timer_delete(&q->timer);
-	snd_seq_pool_delete(&q->pool);
 
-	snd_free(q, sizeof(queue_t));
+	snd_kfree(q);
 }
 
 
@@ -148,14 +146,14 @@ void snd_seq_check_queue(queue_t *q, int atomic)
 		return;
 
 	/* make this function non-reentrant */
-	snd_spin_lock(q, check, &flags);
+	spin_lock_irqsave(&q->check_lock, flags);
 	if (q->check_blocked) {
 		q->check_again = 1;
-		snd_spin_unlock(q, check, &flags);
+		spin_unlock_irqrestore(&q->check_lock, flags);
 		return;		/* other thread is already checking queues */
 	}
 	q->check_blocked = 1;
-	snd_spin_unlock(q, check, &flags);
+	spin_unlock_irqrestore(&q->check_lock, flags);
 
       __again:
 
@@ -198,14 +196,14 @@ void snd_seq_check_queue(queue_t *q, int atomic)
 	}
 
 	/* free lock */
-	snd_spin_lock(q, check, &flags);
+	spin_lock_irqsave(&q->check_lock, flags);
 	if (q->check_again) {
 		q->check_again = 0;
-		snd_spin_unlock(q, check, &flags);
+		spin_unlock_irqrestore(&q->check_lock, flags);
 		goto __again;
 	}
 	q->check_blocked = 0;
-	snd_spin_unlock(q, check, &flags);
+	spin_unlock_irqrestore(&q->check_lock, flags);
 }
 
 /* check queues and dispatch events */
@@ -224,37 +222,20 @@ void snd_seq_check_queues(int atomic)
 }
 
 
-/* change time stamp if possible */
-static void snd_seq_enqueue_process_time(snd_seq_event_cell_t * cell, int realtime)
-{
-	switch (cell->event.flags & SND_SEQ_TIME_STAMP_MASK) {
-	case SND_SEQ_TIME_STAMP_TICK:
-		if (cell->event.time.tick)	/* don't change */
-			return;
-		break;
-	case SND_SEQ_TIME_STAMP_REAL:
-		if (cell->event.time.real.tv_sec || cell->event.time.real.tv_nsec)
-			return;
-		break;
-	}
-	cell->event.flags &= ~SND_SEQ_TIME_STAMP_MASK;
-	cell->event.flags |=
-		realtime ? SND_SEQ_TIME_STAMP_REAL : SND_SEQ_TIME_STAMP_TICK;
-}
-
-
 /* enqueue a event to singe queue */
-static void snd_seq_enqueue_single_event(snd_seq_event_cell_t * cell, int atomic)
+int snd_seq_enqueue_event(snd_seq_event_cell_t *cell, int atomic)
 {
 	int dest;
 	queue_t *q;
 
-	if (cell == NULL)
-		return;
-	dest = cell->event.dest.queue;	/* dest queue */
+	if (cell == NULL) {
+		snd_printd("oops: snd_seq_enqueue_event() called with NULL cell\n");
+		return -EINVAL;
+	}
+	dest = cell->event.dest.queue;		/* dest queue */
 	q = queueptr(dest);
 	if (q == NULL)
-		return;
+		return -EINVAL;
 	/* handle relative time stamps, convert them into absolute */
 	if ((cell->event.flags & SND_SEQ_TIME_MODE_MASK) == SND_SEQ_TIME_MODE_REL) {
 		switch (cell->event.flags & SND_SEQ_TIME_STAMP_MASK) {
@@ -285,102 +266,11 @@ static void snd_seq_enqueue_single_event(snd_seq_event_cell_t * cell, int atomic
 	/* trigger dispatching */
 	snd_seq_check_queues(atomic);
 
-	return;
+	return 0;
 }
 
 
 
-/* enqueue a event received from one client */
-void snd_seq_enqueue_event(snd_seq_event_cell_t * cell, int atomic)
-{
-	int dest;
-
-	if (cell == NULL) {
-		snd_printd("oops: snd_seq_enqueue_event() called with NULL cell\n");
-		return;
-	}
-	dest = cell->event.dest.queue;		/* dest queue */
-
-	/* handle special addressing types */
-	switch (dest) {
-		case SND_SEQ_ADDRESS_BROADCAST:
-			snd_printk("Seq: broadcasts to all queues are not supported yet (dest = %d)\n", dest);
-
-			/* release the original cell */
-			snd_seq_cell_free(cell);
-			break;
-
-		case SND_SEQ_ADDRESS_SUBSCRIBERS:
-			{
-				queue_t *q;
-				client_t *c;
-				client_port_t *p;
-				subscribers_t *subs;
-
-				/* get pointer to client and lock it */
-				c = snd_seq_client_use_ptr(cell->event.source.client);
-				if (!c) {
-					/* release the original cell */
-					snd_seq_cell_free(cell);
-					break;
-				}
-
-				/* get pointer to subscription list */
-				p = snd_seq_port_use_ptr(c, cell->event.source.port);
-				if (!p) {
-					snd_seq_client_unlock(c);
-					/* release the original cell */
-					snd_seq_cell_free(cell);
-					break;
-				}
-				if (!(p->capability & SND_SEQ_PORT_CAP_SUBSCRIPTION)) {
-					snd_seq_ports_unlock(c);
-					snd_seq_client_unlock(c);
-					/* release the original cell */
-					snd_seq_cell_free(cell);
-					break;
-				}
-				/* go through list of subscribers */
-				snd_seq_subscribers_lock(&p->input);
-				for (subs = p->input.list; subs != NULL; subs = subs->next) {
-
-					snd_seq_event_cell_t *new_cell;
-
-					/* enqueue copy of the cell for the specified subscriber */
-					q = queueptr(subs->addr.queue);
-					if (q == NULL)
-						continue;
-					new_cell = snd_seq_cell_dup(q->pool, cell, atomic);
-					if (new_cell) {
-						snd_seq_enqueue_process_time(new_cell, subs->realtime);
-						new_cell->event.dest.queue = subs->addr.queue;
-						new_cell->event.dest.client = subs->addr.client;
-						new_cell->event.dest.port = subs->addr.port;
-						snd_seq_enqueue_single_event(new_cell, atomic);
-					} else {
-						snd_printk("seq: failed enqueing for subscriber %d:%d:%d (no mem)\n",
-						     subs->addr.queue,
-						     subs->addr.client,
-						     subs->addr.port);
-					}
-				}
-				snd_seq_subscribers_unlock(&p->input);
-				snd_seq_ports_unlock(c);
-				snd_seq_client_unlock(c);
-
-				/* release the original cell */
-				snd_seq_cell_free(cell);
-				break;
-
-			}
-			break;
-
-		default:
-			snd_seq_enqueue_single_event(cell, atomic);
-	}
-}
-
-
 /* check if client has access to queue's parameters, return 1 if client is
    allowed to make changes to the queue */
 
@@ -410,14 +300,14 @@ int snd_seq_queue_timer_stop(int queueid, int client)
 	if (!q)
 		return -EINVAL;
 
-	snd_spin_lock(q, owner, &flags);
+	spin_lock_irqsave(&q->owner_lock, flags);
 	if (check_access(q, client)) {
 		q->owner = client;
 		q->klocked = 0;
-		snd_spin_unlock(q, owner, &flags);
+		spin_unlock_irqrestore(&q->owner_lock, flags);
 		snd_seq_timer_stop(q->timer);
 	} else {
-		snd_spin_unlock(q, owner, &flags);
+		spin_unlock_irqrestore(&q->owner_lock, flags);
 		return -EPERM;
 	}
 	return 0;
@@ -432,16 +322,16 @@ int snd_seq_queue_timer_start(int queueid, int client)
 	if (!q)
 		return -EINVAL;
 
-	snd_spin_lock(q, owner, &flags);
+	spin_lock_irqsave(&q->owner_lock, flags);
 	if (check_access(q, client)) {
 		q->owner = client;
 		q->klocked = 1;
-		snd_spin_unlock(q, owner, &flags);
+		spin_unlock_irqrestore(&q->owner_lock, flags);
 		snd_seq_prioq_leave(q->tickq, client, 1);
 		snd_seq_prioq_leave(q->timeq, client, 1);
 		snd_seq_timer_start(q->timer);
 	} else {
-		snd_spin_unlock(q, owner, &flags);
+		spin_unlock_irqrestore(&q->owner_lock, flags);
 		return -EPERM;
 	}
 	return 0;
@@ -456,14 +346,14 @@ int snd_seq_queue_timer_continue(int queueid, int client)
 	if (!q)
 		return -EINVAL;
 
-	snd_spin_lock(q, owner, &flags);
+	spin_lock_irqsave(&q->owner_lock, flags);
 	if (check_access(q, client)) {
 		q->owner = client;
 		q->klocked = 1;
-		snd_spin_unlock(q, owner, &flags);
+		spin_unlock_irqrestore(&q->owner_lock, flags);
 		snd_seq_timer_continue(q->timer);
 	} else {
-		snd_spin_unlock(q, owner, &flags);
+		spin_unlock_irqrestore(&q->owner_lock, flags);
 		return -EPERM;
 	}
 	return 0;
@@ -479,7 +369,7 @@ int snd_seq_queue_timer_set_tempo(int queueid, int client, int tempo)
 	if (!q)
 		return -EINVAL;
 
-	snd_spin_lock(q, owner, &flags);
+	spin_lock_irqsave(&q->owner_lock, flags);
 	if (check_access(q, client)) {
 		result = snd_seq_timer_set_tempo(q->timer, tempo);
 		if (result >= 0)
@@ -487,7 +377,7 @@ int snd_seq_queue_timer_set_tempo(int queueid, int client, int tempo)
 	} else {
 		result = -EPERM;
 	}
-	snd_spin_unlock(q, owner, &flags);
+	spin_unlock_irqrestore(&q->owner_lock, flags);
 	return result;
 }
 
@@ -500,7 +390,7 @@ int snd_seq_queue_timer_set_ppq(int queueid, int client, int ppq)
 	if (!q)
 		return -EINVAL;
 
-	snd_spin_lock(q, owner, &flags);
+	spin_lock_irqsave(&q->owner_lock, flags);
 	if (check_access(q, client)) {
 		result = snd_seq_timer_set_ppq(q->timer, ppq);
 		if (result >= 0)
@@ -508,7 +398,7 @@ int snd_seq_queue_timer_set_ppq(int queueid, int client, int ppq)
 	} else {
 		result = -EPERM;
 	}
-	snd_spin_unlock(q, owner, &flags);
+	spin_unlock_irqrestore(&q->owner_lock, flags);
 	return result;
 }
 
@@ -522,13 +412,13 @@ int snd_seq_queue_set_owner(int queueid, int client)
 	if (!q)
 		return -EINVAL;
 
-	snd_spin_lock(q, owner, &flags);
+	spin_lock_irqsave(&q->owner_lock, flags);
 	if (check_access(q, client)) {
 		q->owner = client;
 	} else {
 		result = -EPERM;
 	}
-	snd_spin_unlock(q, owner, &flags);
+	spin_unlock_irqrestore(&q->owner_lock, flags);
 	return result;
 }
 
@@ -543,14 +433,14 @@ int snd_seq_queue_set_locked(int queueid, int client, int locked)
 	if (!q)
 		return -EINVAL;
 
-	snd_spin_lock(q, owner, &flags);
+	spin_lock_irqsave(&q->owner_lock, flags);
 	if (check_access(q, client)) {
 		q->locked = locked ? 1 : 0;
 		q->owner = client;
 	} else {
 		result = -EPERM;
 	}
-	snd_spin_unlock(q, owner, &flags);
+	spin_unlock_irqrestore(&q->owner_lock, flags);
 	return result;
 }
 
@@ -617,8 +507,8 @@ int snd_seq_queue_use(int queueid, int client, int use)
 	queue = queueptr(queueid);
 	if (queue == NULL)
 		return -EINVAL;
-	snd_mutex_down(queue, use);
-	snd_spin_lock(queue, clients, &flags);
+	down(&queue->use_mutex);
+	spin_lock_irqsave(&queue->clients_lock, flags);
 	if (use) {
 		if (!test_and_set_bit(client, &queue->clients_bitmap))
 			queue->clients++;
@@ -626,17 +516,15 @@ int snd_seq_queue_use(int queueid, int client, int use)
 		if (test_and_clear_bit(client, &queue->clients_bitmap))
 			queue->clients--;
 	}
-	snd_spin_unlock(queue, clients, &flags);
+	spin_unlock_irqrestore(&queue->clients_lock, flags);
 	if (queue->clients) {
-		snd_seq_pool_init(queue->pool, SND_SEQ_MAX_EVENTS);
 		if (use && queue->clients == 1)
 			snd_seq_timer_defaults(queue->timer);
 		snd_seq_timer_open(queue);
 	} else {
 		snd_seq_timer_close(queue);
-		snd_seq_pool_done(queue->pool);
 	}
-	snd_mutex_up(queue, use);
+	up(&queue->use_mutex);
 	return 0;
 }
 
@@ -652,11 +540,11 @@ void snd_seq_queue_client_termination(int client)
 		q = queueptr(c);
 		if (!q)
 			continue;
-		snd_spin_lock(q, owner, &flags);
+		spin_lock_irqsave(&q->owner_lock, flags);
 		if (q->owner == client) {
 			q->klocked = 1;
 		}
-		snd_spin_unlock(q, owner, &flags);
+		spin_unlock_irqrestore(&q->owner_lock, flags);
 		if (q->owner == client) {
 			if (q->timer->running)
 				snd_seq_timer_stop(q->timer);
@@ -678,17 +566,32 @@ void snd_seq_queue_client_leave(int client)
 			continue;
 		snd_seq_prioq_leave(q->tickq, client, q->owner == client);
 		snd_seq_prioq_leave(q->timeq, client, q->owner == client);
-		snd_spin_lock(q, owner, &flags);
+		spin_lock_irqsave(&q->owner_lock, flags);
 		if (q->owner == client) {
 			q->owner = SEQ_QUEUE_NO_OWNER;
 			q->locked = 0;
 			q->klocked = 0;
 		}
-		snd_spin_unlock(q, owner, &flags);
+		spin_unlock_irqrestore(&q->owner_lock, flags);
 		snd_seq_queue_use(c, client, 0);
 	}
 }
 
+/* remove cells only */
+void snd_seq_queue_client_leave_cells(int client)
+{
+	int c;
+	queue_t *q;
+
+	for (c = 0; c < SND_SEQ_MAX_QUEUES; c++) {
+		q = queueptr(c);
+		if (!q)
+			continue;
+		snd_seq_prioq_leave(q->tickq, client, 0);
+		snd_seq_prioq_leave(q->timeq, client, 0);
+	}
+}
+
 /* exported to seq_info.c */
 void snd_seq_info_queues_read(snd_info_buffer_t * buffer, void *private_data)
 {
@@ -696,6 +599,7 @@ void snd_seq_info_queues_read(snd_info_buffer_t * buffer, void *private_data)
 	int bpm;
 	queue_t *q;
 	timer_t *tmr;
+	snd_seq_event_cell_t *cell;
 
 	for (c = 0; c < SND_SEQ_MAX_QUEUES; c++) {
 		q = queueptr(c);
@@ -719,6 +623,13 @@ void snd_seq_info_queues_read(snd_info_buffer_t * buffer, void *private_data)
 		snd_iprintf(buffer, "current BPM        : %d\n", bpm);
 		snd_iprintf(buffer, "current time       : %d.%09d s\n", tmr->cur_time.tv_sec, tmr->cur_time.tv_nsec);
 		snd_iprintf(buffer, "current tick       : %d\n", tmr->cur_tick);
+		cell = snd_seq_prioq_cell_peek(q->timeq);
+		snd_iprintf(buffer, "queue top (time)   : %d.%09d s\n",
+			    (cell ? cell->event.time.real.tv_sec : 0),
+			    (cell ? cell->event.time.real.tv_nsec : 0));
+		cell = snd_seq_prioq_cell_peek(q->tickq);
+		snd_iprintf(buffer, "queue top (tick)   : %d\n",
+			    (cell ? cell->event.time.tick : 0));
 		snd_iprintf(buffer, "\n");
 	}
 }
