@@ -29,7 +29,6 @@
 
 struct snd_seq_stru_queue {
 	int queue;		/* queue number */
-	pool_t *pool;		/* memory pool for this queue */
 
 	prioq_t	*tickq;		/* midi tick event queue */
 	prioq_t	*timeq;		/* real-time event queue */	
@@ -40,14 +39,14 @@ struct snd_seq_stru_queue {
 		klocked:1,	/* kernel lock (after START) */	
 		check_again:1,
 		check_blocked:1;
-	snd_spin_define(owner);
-	snd_spin_define(check);
+	spinlock_t owner_lock;
+	spinlock_t check_lock;
 
 	/* clients which uses this queue (bitmap) */
 	unsigned int clients_bitmap[SND_SEQ_MAX_CLIENTS/sizeof(unsigned int)];
 	unsigned int clients;	/* users of this queue */
-	snd_spin_define(clients);
-	snd_mutex_define(use);
+	spinlock_t clients_lock;
+	struct semaphore use_mutex;
 };
 
 
@@ -69,9 +68,10 @@ extern void snd_seq_queue_client_termination(int client);
 
 /* final stage */
 extern void snd_seq_queue_client_leave(int client);
+extern void snd_seq_queue_client_leave_cells(int client);
 
 /* enqueue a event received from one the clients */
-extern void snd_seq_enqueue_event(snd_seq_event_cell_t *cell, int atomic);
+extern int snd_seq_enqueue_event(snd_seq_event_cell_t *cell, int atomic);
 
 
 /* return pointer to queue structure for specified id */
