@@ -19,7 +19,7 @@
  *
  */
 
-#include "driver.h"
+#include "../../include/driver.h"
 #include "seq_system.h"
 #include "seq_timer.h"
 #include "seq_queue.h"
