@@ -20,8 +20,7 @@
  *
  */
 
-#include "driver.h"
-#include "timer.h"
+#include "../../include/driver.h"
 #include "seq_timer.h"
 #include "seq_queue.h"
 #include "seq_info.h"
@@ -39,12 +38,12 @@ timer_t *snd_seq_timer_new(void)
 {
 	timer_t *tmr;
 	
-	tmr = snd_calloc(sizeof(timer_t));
+	tmr = snd_kcalloc(sizeof(timer_t), GFP_KERNEL);
 	if (tmr == NULL) {
 		snd_printd("malloc failed for snd_seq_timer_new() \n");
 		return NULL;
 	}
-	snd_spin_prepare(tmr, lock);
+	tmr->lock = SPIN_LOCK_UNLOCKED;
 
 	/* reset setup to defaults */
 	snd_seq_timer_defaults(tmr);
@@ -70,7 +69,7 @@ void snd_seq_timer_delete(timer_t **tmr)
 	snd_seq_timer_stop(t);
 	snd_seq_timer_reset(t);
 
-	snd_free(t,sizeof(timer_t));
+	snd_kfree(t);
 }
 
 void snd_seq_timer_defaults(timer_t * tmr)
@@ -91,7 +90,7 @@ void snd_seq_timer_reset(timer_t * tmr)
 {
 	unsigned long flags;
 
-	snd_spin_lock(tmr, lock, &flags);
+	spin_lock_irqsave(&tmr->lock, flags);
 
 	/* reset time & songposition */
 	tmr->cur_tick = 0;
@@ -105,7 +104,7 @@ void snd_seq_timer_reset(timer_t * tmr)
 	tmr->cur_tick_time = 0;
 
 	tmr->sync_tmp = 0;
-	snd_spin_unlock(tmr, lock, &flags);
+	spin_unlock_irqrestore(&tmr->lock, flags);
 }
 
 
@@ -132,20 +131,11 @@ static void snd_seq_timer_interrupt(snd_timer_instance_t *timeri,
 	period.tv_sec = 0;
 	period.tv_nsec = resolution % 1000000000;
 
-	snd_spin_lock(tmr, lock, &flags);
+	spin_lock_irqsave(&tmr->lock, flags);
 
 	/* update timer */
 	snd_seq_inc_real_time(&tmr->cur_time, &period);
 
-#if 0
-	/* calculate current tick */
-	/* FIXME: use of floating point stuff, illegal in kernel (??) */
-	if (tmr->tempo > 0) {
-		double delta_time;
-		delta_time = tmr->cur_time.tv_sec - tmr->tempo_time.tv_sec + (1.0E-9 * (double) (tmr->cur_time.tv_nsec - tmr->tempo_time.tv_nsec));
-		tmr->cur_tick = tmr->tempo_tick + (tmr->ppq * delta_time * 1.0E6 / tmr->tempo);
-	}
-#else
 	/* calculate current tick */
 	if (tmr->tempo > 0) {
 		unsigned long result;
@@ -154,9 +144,11 @@ static void snd_seq_timer_interrupt(snd_timer_instance_t *timeri,
 		tmr->cur_tick += result / tmr->tick_time;
 		tmr->cur_tick_time = result % tmr->tick_time;
 	}
-#endif
 
-	snd_spin_unlock(tmr, lock, &flags);
+	/* register actual time of this timer update  */
+	do_gettimeofday(&tmr->last_update);
+
+	spin_unlock_irqrestore(&tmr->lock, flags);
 
 #if 0
 	/* sync timer has expired *//* FIXME: hack to send midi sync info */
@@ -172,7 +164,7 @@ static void snd_seq_timer_interrupt(snd_timer_instance_t *timeri,
 		ev.dest.queue = 1;
 		ev.dest.client = 255;
 		ev.dest.port = 255;
-		if (snd_seq_kernel_client_dispatch(0, &ev)) {
+		if (!snd_seq_kernel_client_dispatch(0, &ev)) {
 			tmr->sync_tmp += tmr->ppq / 24;
 	}
 #endif
@@ -185,8 +177,8 @@ int snd_seq_timer_set_tempo(timer_t * tmr, int tempo)
 {
 	unsigned long flags;
 
-	if (tmr) {
-		snd_spin_lock(tmr, lock, &flags);
+	if (tmr && tempo > 0) {
+		spin_lock_irqsave(&tmr->lock, flags);
 		/* store location of tempo change */
 		tmr->tempo_tick = tmr->cur_tick;
 		tmr->tempo_time.tv_sec = tmr->cur_time.tv_sec;
@@ -196,7 +188,7 @@ int snd_seq_timer_set_tempo(timer_t * tmr, int tempo)
 		snd_seq_timer_set_tick_time(tmr);
 		tmr->cur_tick += tmr->cur_tick_time / tmr->tick_time;
 		tmr->cur_tick_time %= tmr->tick_time;
-		snd_spin_unlock(tmr, lock, &flags);
+		spin_unlock_irqrestore(&tmr->lock, flags);
 		return 0;
 	}
 	return -EINVAL;
@@ -207,19 +199,19 @@ int snd_seq_timer_set_ppq(timer_t * tmr, int ppq)
 {
 	unsigned long flags;
 
-	if (tmr) {
-		if (tmr->running & (ppq != tmr->ppq)) {
+	if (tmr && ppq > 0) {
+		if (tmr->running && (ppq != tmr->ppq)) {
 			/* refuse to change ppq on running timers */
 			/* because it will upset the song position (ticks) */
 			snd_printd("seq: cannot change ppq of a running timer\n");
 			return -EBUSY;
 		} else {
-			snd_spin_lock(tmr, lock, &flags);
+			spin_lock_irqsave(&tmr->lock, flags);
 			tmr->ppq = ppq;
 			snd_seq_timer_set_tick_time(tmr);
 			tmr->cur_tick += tmr->cur_tick_time / tmr->tick_time;
 			tmr->cur_tick_time %= tmr->tick_time;
-			snd_spin_unlock(tmr, lock, &flags);		
+			spin_unlock_irqrestore(&tmr->lock, flags);		
 		}
 		return 0;
 	}
@@ -308,6 +300,7 @@ void snd_seq_timer_start(timer_t * tmr)
 	snd_seq_timer_reset(tmr);
 	snd_timer_start(tmr->timeri, tmr->ticks);
 	tmr->running = 1;
+	do_gettimeofday(&tmr->last_update);
 }
 
 void snd_seq_timer_continue(timer_t * tmr)
@@ -316,6 +309,7 @@ void snd_seq_timer_continue(timer_t * tmr)
 		return;
 	snd_timer_start(tmr->timeri, tmr->ticks);
 	tmr->running = 1;
+	do_gettimeofday(&tmr->last_update);
 }
 
 int snd_seq_timer_midi_open(queue_t *q)
@@ -328,6 +322,34 @@ int snd_seq_timer_midi_close(queue_t *q)
 	return -ENXIO;	/* not implemented */
 }
 
+
+/* return current 'real' time. use timeofday() to get better granularity. */
+snd_seq_real_time_t snd_seq_timer_get_cur_time(timer_t *tmr)
+{
+	snd_seq_real_time_t cur_time = tmr->cur_time;
+	struct timeval tm;
+	
+	if (tmr->running) { 
+		do_gettimeofday(&tm);
+		cur_time.tv_nsec += 1000 * (tm.tv_usec - tmr->last_update.tv_usec);
+		while (cur_time.tv_nsec >= 1000000000) {
+			/* roll-over */
+			cur_time.tv_nsec -= 1000000000;
+        	        cur_time.tv_sec ++;
+        	}
+        }
+                
+	return cur_time;	
+}
+
+/* TODO: use interpolation on tick queue (will only be usefull for very
+ high PPQ values) */
+snd_seq_tick_time_t snd_seq_timer_get_cur_tick(timer_t *tmr)
+{
+	return tmr->cur_tick;
+}
+
+
 /* exported to seq_info.c */
 void snd_seq_info_timer_read(snd_info_buffer_t * buffer, void *private_data)
 {
