@@ -1,6 +1,6 @@
 /*
  *   Sequencer routine for uart16550 (mainly port management)
- *   Copyright (c) by Isaku Yamahata (yamahata@kusm.kyoto-u.ac.jp)
+ *   Copyright (c) by Isaku Yamahata (yamahata@private.email.ne.jp)
  *   16 Jan 1999 first version.
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -19,14 +19,14 @@
  *
  */
 
-#include "driver.h"
+#include "../../include/driver.h"
 
 #ifdef CONFIG_SND_SEQUENCER
 
-#include "seq_kernel.h"
-#include "midi.h"
+#include "../../include/seq_kernel.h"
+#include "../../include/midi.h"
 
-#include "uart16550.h"
+#include "../../include/uart16550.h"
 #include "uart16550_type.h"
 
 #define UART16550_PORT_A 0
@@ -49,21 +49,21 @@ struct snd_seq_uart16550_client {
 
 	int device;		//device number
 
-	int client_index;	//client$BHV9f(B
+	int client_index;	//client number
 
-	int current_port;	//$B8=:_$N%]!<%H(B UART16550_PORT_A,UART16550_PORT_B
+	int current_port;	//current port number UART16550_PORT_A,UART16550_PORT_B
 
-	int port_status;	//PORT_A, PORT_B$B$N(Bopen,close$B$N>uBV$r5-21$7$F$*$/!#(B
+	int port_status;	//remenber status of open/close of PORT_A, PORT_B
 
-	snd_seq_uart16550_port_t *port_a;	//port a$B$KBP1~$9$k(Balsa$B$N(Bport
+	snd_seq_uart16550_port_t *port_a;	//ALSA port associate to port a
 
-	snd_seq_uart16550_port_t *port_b;	//port b$B$KBP1~$9$k(Balsa$B$N(Bport
+	snd_seq_uart16550_port_t *port_b;	//ALSA port associate to port b
 
 };
 
 struct snd_seq_uart16550_port {
 	struct snd_seq_uart16550_client *client;
-	int seq_port;		//alsa$B$N(Bport$BHV9f(B
+        int seq_port;		//ALSA port number
 
 	int port;		//UART16550_PORT_A or UART16550_PORT_B
 
@@ -80,14 +80,14 @@ static snd_seq_uart16550_client_t *snd_seq_uart16550_client_new
 	if (card == NULL || rmidi == NULL)
 		return NULL;
 
-	uart16550_client = snd_malloc(sizeof(snd_seq_uart16550_client_t));
+	uart16550_client = snd_kmalloc(sizeof(snd_seq_uart16550_client_t), GFP_KERNEL);
 	if (uart16550_client == NULL)
 		return NULL;
 
 	uart16550_client->card = card;
 	uart16550_client->rmidi = rmidi;
 	uart16550_client->device = device;
-	uart16550_client->client_index = -1;	//$B<+F03d$jEv$F(B
+	uart16550_client->client_index = -1;	//automatic allocation
 
 	uart16550_client->current_port = UART16550_PORT_UNKNOWN;
 	uart16550_client->port_status = UART16550_PORT_NOT_OPENED;
@@ -99,7 +99,7 @@ static snd_seq_uart16550_client_t *snd_seq_uart16550_client_new
 
 static void snd_seq_uart16550_client_delete
  (snd_seq_uart16550_client_t * uart16550_client) {
-	snd_free(uart16550_client, sizeof(snd_seq_uart16550_client_t));
+	snd_kfree(uart16550_client);
 }
 
 static snd_seq_uart16550_port_t *snd_seq_uart16550_port_new
@@ -110,7 +110,7 @@ static snd_seq_uart16550_port_t *snd_seq_uart16550_port_new
 	!(uart_port == UART16550_PORT_A || uart_port == UART16550_PORT_B))
 		return NULL;
 
-	uart16550_port = snd_malloc(sizeof(snd_seq_uart16550_port_t));
+	uart16550_port = snd_kmalloc(sizeof(snd_seq_uart16550_port_t), GFP_KERNEL);
 	if (uart16550_port == NULL)
 		return NULL;
 
@@ -123,7 +123,7 @@ static snd_seq_uart16550_port_t *snd_seq_uart16550_port_new
 
 static void snd_seq_uart16550_port_delete(snd_seq_uart16550_port_t * uart16550_port)
 {
-	snd_free(uart16550_port, sizeof(snd_seq_uart16550_port_t));
+	snd_kfree(uart16550_port);
 }
 
 ///////////////////////////////////////////////////////////////////port input
@@ -401,7 +401,8 @@ static void snd_midi_command(snd_rawmidi_t * rmidi, void *cmd_private_data,
 
 
 
-static int snd_seq_uart16550_subscribe(void *private_data /* = uart16550_port_t */ )
+static int snd_seq_uart16550_subscribe(void *private_data /* = uart16550_port_t */,
+				       snd_seq_port_subscribe_t *info)
 {
 	snd_seq_uart16550_port_t *uart_port =
 	(snd_seq_uart16550_port_t *) private_data;
@@ -410,6 +411,11 @@ static int snd_seq_uart16550_subscribe(void *private_data /* = uart16550_port_t
 
 	snd_printd("snd_seq_uart16550_subscribe called\n");
 	MOD_INC_USE_COUNT;
+	if ((info->midi_channels && info->midi_channels != 16) ||
+	    info->midi_voices || info->synth_voices) {
+	    	MOD_DEC_USE_COUNT;
+	    	return -EINVAL;
+	}
 	/* open midi port */
 	ret = snd_midi_open(uart_client->card->number, uart_client->device,
 			    SND_RAWMIDI_LFLG_INPUT, &uart_client->rmidi);
@@ -426,7 +432,8 @@ static int snd_seq_uart16550_subscribe(void *private_data /* = uart16550_port_t
 	return 0;
 }
 
-static int snd_seq_uart16550_unsubscribe(void *private_data /* = uart16550_port_t */ )
+static int snd_seq_uart16550_unsubscribe(void *private_data /* = uart16550_port_t */,
+					 snd_seq_port_subscribe_t *info)
 {
 	snd_seq_uart16550_port_t *uart_port =
 	(snd_seq_uart16550_port_t *) private_data;
@@ -819,15 +826,21 @@ static int snd_seq_uart16550_do_unuse(snd_seq_uart16550_client_t * uart_client,
 }
 
 
-static int snd_seq_uart16550_use(void *private_data /* = uart16550_port_t */ )
+static int snd_seq_uart16550_use(void *private_data /* = uart16550_port_t */,
+				 snd_seq_port_subscribe_t *info)
 {
 	snd_seq_uart16550_port_t *uart16550_port =
 	(snd_seq_uart16550_port_t *) private_data;
+
+	if ((info->midi_channels && info->midi_channels != 16) ||
+	    info->midi_voices || info->synth_voices)
+	    	return -EINVAL;
 	return snd_seq_uart16550_do_use(uart16550_port->client,
 					uart16550_port->port);
 }
 
-static int snd_seq_uart16550_unuse(void *private_data /* = uart16550_port_t */ )
+static int snd_seq_uart16550_unuse(void *private_data /* = uart16550_port_t */,
+				   snd_seq_port_subscribe_t *info)
 {
 	snd_seq_uart16550_port_t *uart16550_port =
 	(snd_seq_uart16550_port_t *) private_data;
