@@ -27,10 +27,10 @@
  *
  */
 
-#include "driver.h"
-#include "sb.h"
-#include "info.h"
-#include "ulaw.h"
+#include "../../include/driver.h"
+#include "../../include/sb.h"
+#include "../../include/info.h"
+#include "../../include/ulaw.h"
 
 static inline void snd_sb16_ack_8bit(sbdsp_t * codec)
 {
@@ -42,11 +42,11 @@ static inline void snd_sb16_ack_16bit(sbdsp_t * codec)
 	inb(SBP(codec, DATA_AVAIL_16));
 }
 
-static void snd_sb16_playback_8to16(unsigned char *buffer,
-				    unsigned int offset,
-				    unsigned char *user,
-				    unsigned int count,
-				    int ulaw)
+static int snd_sb16_playback_8to16(unsigned char *buffer,
+				   unsigned int offset,
+				   unsigned char *user,
+				   unsigned int count,
+				   int ulaw)
 {
 	unsigned char *pbuffer;
 	unsigned int size1;
@@ -60,7 +60,8 @@ static void snd_sb16_playback_8to16(unsigned char *buffer,
 	while (count > 0) {
 		size = size1 = sizeof(sbuffer) < count ?
 						sizeof(sbuffer) : count;
-		copy_from_user(sbuffer, user, size);
+		if (copy_from_user(sbuffer, user, size))
+			return -EFAULT;
 		pbuffer = sbuffer;
 		if (ulaw) {
 			while (size1-- > 0) {
@@ -76,9 +77,10 @@ static void snd_sb16_playback_8to16(unsigned char *buffer,
 		count -= size;
 		user += size;
 	}
+	return 0;
 }
 
-static void snd_sb16_record_16to8(unsigned char *buffer,
+static int snd_sb16_capture_16to8(unsigned char *buffer,
 				  unsigned int offset,
 				  unsigned char *user,
 				  unsigned int count,
@@ -108,16 +110,18 @@ static void snd_sb16_record_16to8(unsigned char *buffer,
 				buffer += 2;
 			}
 		}
-		copy_to_user(user, sbuffer, size);
+		if (copy_to_user(user, sbuffer, size))
+			return -EFAULT;
 		count -= size;
 		user += size;
 	}
+	return 0;
 }
 
-static void snd_sb16_playback_16to8(unsigned char *buffer,
-				    unsigned int offset,
-				    unsigned char *user,
-				    unsigned int count)
+static int snd_sb16_playback_16to8(unsigned char *buffer,
+				   unsigned int offset,
+				   unsigned char *user,
+				   unsigned int count)
 {
 	unsigned char *pbuffer;
 	unsigned int size1;
@@ -131,7 +135,8 @@ static void snd_sb16_playback_16to8(unsigned char *buffer,
 	while (count > 0) {
 		size = size1 = sizeof(sbuffer) < count ?
 						sizeof(sbuffer) : count;
-		copy_from_user(sbuffer, user, size);
+		if (copy_from_user(sbuffer, user, size))
+			return -EFAULT;
 		pbuffer = sbuffer + 1;	/* little endian data */
 		size1 >>= 1;
 		while (size1-- > 0) {
@@ -141,9 +146,10 @@ static void snd_sb16_playback_16to8(unsigned char *buffer,
 		count -= size;
 		user += size;
 	}
+	return 0;
 }
 
-static void snd_sb16_record_8to16(unsigned char *buffer,
+static int snd_sb16_capture_8to16(unsigned char *buffer,
 				  unsigned int offset,
 				  unsigned char *user,
 				  unsigned int count)
@@ -165,49 +171,62 @@ static void snd_sb16_record_8to16(unsigned char *buffer,
 			*pbuffer++ = 0x80;
 			*pbuffer++ = *buffer++;
 		}
-		copy_to_user(user, sbuffer, size);
+		if (copy_to_user(user, sbuffer, size))
+			return -EFAULT;
 		count -= size;
 		user += size;
 	}
+	return 0;
 }
 
-static int snd_sb16_playback_ioctl(snd_pcm1_t * pcm1,
-				   unsigned int cmd, unsigned long *arg)
+static int snd_sb16_playback_ioctl(void *private_data,
+				   snd_pcm_subchn_t * subchn,
+				   unsigned int cmd,
+				   unsigned long *arg)
 {
+	snd_pcm1_channel_t *pchn1 = (snd_pcm1_channel_t *)subchn->pcm->playback.private_data;
+	snd_pcm1_subchn_t *subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
+
 	switch (cmd) {
 	case SND_PCM1_IOCTL_RATE:
-		pcm1->playback.real_rate = pcm1->playback.rate;
-		if (pcm1->playback.real_rate < 4000)
-			pcm1->playback.real_rate = 4000;
-		if (pcm1->playback.real_rate > 44100)
-			pcm1->playback.real_rate = 44100;
+		subchn1->real_rate = subchn1->rate;
+		if (subchn1->real_rate < pchn1->hw.min_rate)
+			subchn1->real_rate = pchn1->hw.min_rate;
+		if (subchn1->real_rate > pchn1->hw.max_rate)
+			subchn1->real_rate = pchn1->hw.max_rate;
 		return 0;
 	}
 	return -ENXIO;
 }
 
-static int snd_sb16_record_ioctl(snd_pcm1_t * pcm1,
-				 unsigned int cmd, unsigned long *arg)
+static int snd_sb16_capture_ioctl(void *private_data,
+				 snd_pcm_subchn_t * subchn,
+				 unsigned int cmd,
+				 unsigned long *arg)
 {
+	snd_pcm1_channel_t *pchn1 = (snd_pcm1_channel_t *)subchn->pcm->capture.private_data;
+	snd_pcm1_subchn_t *subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
+
 	switch (cmd) {
 	case SND_PCM1_IOCTL_RATE:
-		pcm1->record.real_rate = pcm1->record.rate;
-		if (pcm1->record.real_rate < 4000)
-			pcm1->record.real_rate = 4000;
-		if (pcm1->record.real_rate > 44100)
-			pcm1->record.real_rate = 44100;
+		subchn1->real_rate = subchn1->rate;
+		if (subchn1->real_rate < pchn1->hw.min_rate)
+			subchn1->real_rate = pchn1->hw.min_rate;
+		if (subchn1->real_rate > pchn1->hw.max_rate)
+			subchn1->real_rate = pchn1->hw.max_rate;
 		return 0;
 	}
 	return -ENXIO;
 }
 
 static void snd_sb16_setup_rate(sbdsp_t * codec,
-				unsigned short rate, int direction)
+				unsigned short rate,
+				int capture)
 {
 	unsigned long flags;
 
-	snd_spin_lock(codec, reg, &flags);
-	if (codec->mode16 & (direction == SND_PCM1_PLAYBACK ? SB_MODE16_PLAYBACK16 : SB_MODE16_RECORD16))
+	spin_lock_irqsave(&codec->reg_lock, flags);
+	if (codec->mode16 & (!capture ? SB_MODE16_PLAYBACK16 : SB_MODE16_CAPTURE16))
 		snd_sb16_ack_16bit(codec);
 	else
 		snd_sb16_ack_8bit(codec);
@@ -219,76 +238,76 @@ static void snd_sb16_setup_rate(sbdsp_t * codec,
 		snd_sb16dsp_command(codec, rate >> 8);
 		snd_sb16dsp_command(codec, rate & 0xff);
 	}
-	snd_spin_unlock(codec, reg, &flags);
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
 }
 
-static void snd_sb16_playback_prepare(snd_pcm1_t * pcm1,
+static void snd_sb16_playback_prepare(void *private_data,
+			              snd_pcm_subchn_t * subchn,
 				      unsigned char *buffer,
 				      unsigned int size,
 				      unsigned int offset,
 				      unsigned int count)
 {
 	unsigned long flags;
-	sbdsp_t *codec;
-	snd_pcm1_channel_t *pchn;
+	sbdsp_t *codec = (sbdsp_t *) private_data;
+	snd_pcm1_subchn_t *subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
 	unsigned char format;
 
-	pchn = &pcm1->playback;
-	codec = (sbdsp_t *) pcm1->private_data;
 	format = SB_DSP4_MODE_UNS_MONO;
-	if (pchn->mode & SND_PCM1_MODE_U) {
-		format = pchn->voices > 1 ? SB_DSP4_MODE_UNS_STEREO : SB_DSP4_MODE_UNS_MONO;
+	if (subchn1->mode & SND_PCM1_MODE_U) {
+		format = subchn1->voices > 1 ? SB_DSP4_MODE_UNS_STEREO : SB_DSP4_MODE_UNS_MONO;
 	} else {
-		format = pchn->voices > 1 ? SB_DSP4_MODE_SIGN_STEREO : SB_DSP4_MODE_SIGN_MONO;
+		format = subchn1->voices > 1 ? SB_DSP4_MODE_SIGN_STEREO : SB_DSP4_MODE_SIGN_MONO;
 	}
-	snd_sb16_setup_rate(codec, pchn->real_rate, SND_PCM1_PLAYBACK);
+	snd_sb16_setup_rate(codec, subchn1->real_rate, 0);
 #if 0
 	snd_printk("playback - buffer = 0x%lx, size = %i (%i), count = %i\n", (long) buffer, size, pchn->size, count);
 #endif
 	if (codec->mode16 & SB_MODE16_PLAYBACK16) {	/* use 16-bit DMA */
-		if (!(pchn->mode & SND_PCM1_MODE_16)) {
+		if (!(subchn1->mode & SND_PCM1_MODE_16)) {
 			size <<= 1;
 			count <<= 1;
 		}
-		snd_dma_program(codec->dma16, buffer, size, DMA_MODE_WRITE | DMA_MODE_AUTOINIT);
+		snd_dma_program(codec->dma16, buffer, size, DMA_MODE_WRITE | DMA_AUTOINIT);
 		count >>= 1;
 		count--;
-		snd_spin_lock(codec, reg, &flags);
+		spin_lock_irqsave(&codec->reg_lock, flags);
 		snd_sb16dsp_command(codec, SB_DSP_SPEAKER_ON);
 		snd_sb16dsp_command(codec, SB_DSP4_OUT16_AI);
 		snd_sb16dsp_command(codec, format);
 		snd_sb16dsp_command(codec, count & 0xff);
 		snd_sb16dsp_command(codec, count >> 8);
 		snd_sb16dsp_command(codec, SB_DSP_DMA16_OFF);
-		snd_spin_unlock(codec, reg, &flags);
+		spin_unlock_irqrestore(&codec->reg_lock, flags);
 	} else {
-		if (pchn->mode & SND_PCM1_MODE_16) {
+		if (subchn1->mode & SND_PCM1_MODE_16) {
 			size >>= 1;
 			count >>= 1;
 		}
-		snd_dma_program(codec->dma8, buffer, size, DMA_MODE_WRITE | DMA_MODE_AUTOINIT);
+		snd_dma_program(codec->dma8, buffer, size, DMA_MODE_WRITE | DMA_AUTOINIT);
 		count--;
-		snd_spin_lock(codec, reg, &flags);
+		spin_lock_irqsave(&codec->reg_lock, flags);
 		snd_sb16dsp_command(codec, SB_DSP_SPEAKER_ON);
 		snd_sb16dsp_command(codec, SB_DSP4_OUT8_AI);
 		snd_sb16dsp_command(codec, format);
 		snd_sb16dsp_command(codec, count & 0xff);
 		snd_sb16dsp_command(codec, count >> 8);
 		snd_sb16dsp_command(codec, SB_DSP_DMA8_OFF);
-		snd_spin_unlock(codec, reg, &flags);
+		spin_unlock_irqrestore(&codec->reg_lock, flags);
 	}
 }
 
-static void snd_sb16_playback_trigger(snd_pcm1_t * pcm1, int up)
+static void snd_sb16_playback_trigger(void *private_data,
+				      snd_pcm_subchn_t * subchn,
+				      int up)
 {
 	unsigned long flags;
-	sbdsp_t *codec;
+	sbdsp_t *codec = (sbdsp_t *) private_data;
 
-	codec = (sbdsp_t *) pcm1->private_data;
 #if 0
 	snd_printk("playback trigger - %i\n", up);
 #endif
-	snd_spin_lock(codec, reg, &flags);
+	spin_lock_irqsave(&codec->reg_lock, flags);
 	if (up) {
 		snd_sb16dsp_command(codec, SB_DSP_SPEAKER_ON);
 		snd_sb16dsp_command(codec, codec->mode16 & SB_MODE16_PLAYBACK16 ? SB_DSP_DMA16_ON : SB_DSP_DMA8_ON);
@@ -300,135 +319,132 @@ static void snd_sb16_playback_trigger(snd_pcm1_t * pcm1, int up)
 		/* next two lines are needed for some types of DSP4 (SB AWE 32 - 4.13) */
 		if (codec->mode16 & SB_MODE16_RATE_LOCK_R) {
 #if 0
-			snd_printk("record trigger!!!! (from playback)");
+			snd_printk("capture trigger!!!! (from playback)");
 #endif
-			snd_sb16dsp_command(codec, codec->mode16 & SB_MODE16_RECORD16 ? SB_DSP_DMA16_ON : SB_DSP_DMA8_ON);
+			snd_sb16dsp_command(codec, codec->mode16 & SB_MODE16_CAPTURE16 ? SB_DSP_DMA16_ON : SB_DSP_DMA8_ON);
 		}
 	}
-	snd_spin_unlock(codec, reg, &flags);
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
 }
 
-static void snd_sb16_record_prepare(snd_pcm1_t * pcm1,
+static void snd_sb16_capture_prepare(void *private_data,
+				    snd_pcm_subchn_t * subchn,
 				    unsigned char *buffer,
 				    unsigned int size,
 				    unsigned int offset,
 				    unsigned int count)
 {
 	unsigned long flags;
-	sbdsp_t *codec;
-	snd_pcm1_channel_t *pchn;
+	sbdsp_t *codec = (sbdsp_t *) private_data;
+	snd_pcm1_subchn_t *subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
 	unsigned char format;
 
-	pchn = &pcm1->record;
-	codec = (sbdsp_t *) pcm1->private_data;
 	format = SB_DSP4_MODE_UNS_MONO;
-	if (pchn->mode & SND_PCM1_MODE_U) {
-		format = pchn->voices > 1 ? SB_DSP4_MODE_UNS_STEREO : SB_DSP4_MODE_UNS_MONO;
+	if (subchn1->mode & SND_PCM1_MODE_U) {
+		format = subchn1->voices > 1 ? SB_DSP4_MODE_UNS_STEREO : SB_DSP4_MODE_UNS_MONO;
 	} else {
-		format = pchn->voices > 1 ? SB_DSP4_MODE_SIGN_STEREO : SB_DSP4_MODE_SIGN_MONO;
+		format = subchn1->voices > 1 ? SB_DSP4_MODE_SIGN_STEREO : SB_DSP4_MODE_SIGN_MONO;
 	}
-	snd_sb16_setup_rate(codec, pchn->real_rate, SND_PCM1_RECORD);
+	snd_sb16_setup_rate(codec, subchn1->real_rate, SND_PCM1_CAPTURE);
 #if 0
-	snd_printk("record - buffer = 0x%lx, size = %i (%i), count = %i\n", (long) buffer, size, pchn->size, count);
+	snd_printk("capture - buffer = 0x%lx, size = %i (%i), count = %i\n", (long) buffer, size, pchn->size, count);
 #endif
-	if (codec->mode16 & SB_MODE16_RECORD16) {	/* use 16-bit DMA */
-		if (!(pchn->mode & SND_PCM1_MODE_16)) {
+	if (codec->mode16 & SB_MODE16_CAPTURE16) {	/* use 16-bit DMA */
+		if (!(subchn1->mode & SND_PCM1_MODE_16)) {
 			size <<= 1;
 			count <<= 1;
 		}
-		snd_dma_program(codec->dma16, buffer, size, DMA_MODE_READ | DMA_MODE_AUTOINIT);
+		snd_dma_program(codec->dma16, buffer, size, DMA_MODE_READ | DMA_AUTOINIT);
 		count >>= 1;
 		count--;
-		snd_spin_lock(codec, reg, &flags);
+		spin_lock_irqsave(&codec->reg_lock, flags);
 		snd_sb16dsp_command(codec, SB_DSP4_IN16_AI);
 		snd_sb16dsp_command(codec, format);
 		snd_sb16dsp_command(codec, count & 0xff);
 		snd_sb16dsp_command(codec, count >> 8);
 		snd_sb16dsp_command(codec, SB_DSP_DMA16_OFF);
-		snd_spin_unlock(codec, reg, &flags);
+		spin_unlock_irqrestore(&codec->reg_lock, flags);
 	} else {
-		if (pchn->mode & SND_PCM1_MODE_16) {
+		if (subchn1->mode & SND_PCM1_MODE_16) {
 			size >>= 1;
 			count >>= 1;
 		}
-		snd_dma_program(codec->dma8, buffer, size, DMA_MODE_READ | DMA_MODE_AUTOINIT);
+		snd_dma_program(codec->dma8, buffer, size, DMA_MODE_READ | DMA_AUTOINIT);
 		count--;
-		snd_spin_lock(codec, reg, &flags);
+		spin_lock_irqsave(&codec->reg_lock, flags);
 		snd_sb16dsp_command(codec, SB_DSP4_IN8_AI);
 		snd_sb16dsp_command(codec, format);
 		snd_sb16dsp_command(codec, count & 0xff);
 		snd_sb16dsp_command(codec, count >> 8);
 		snd_sb16dsp_command(codec, SB_DSP_DMA8_OFF);
-		snd_spin_unlock(codec, reg, &flags);
+		spin_unlock_irqrestore(&codec->reg_lock, flags);
 	}
 }
 
-static void snd_sb16_record_trigger(snd_pcm1_t * pcm1, int up)
+static void snd_sb16_capture_trigger(void *private_data,
+				    snd_pcm_subchn_t * subchn,
+				    int up)
 {
 	unsigned long flags;
-	sbdsp_t *codec;
+	sbdsp_t *codec = (sbdsp_t *) private_data;
 
 #if 0
-	snd_printk("sb16: record trigger - %i\n", up);
+	snd_printk("sb16: capture trigger - %i\n", up);
 #endif
-	codec = (sbdsp_t *) pcm1->private_data;
-	snd_spin_lock(codec, reg, &flags);
+	spin_lock_irqsave(&codec->reg_lock, flags);
 	if (up) {
 		codec->mode16 |= SB_MODE16_RATE_LOCK_R;
-		snd_sb16dsp_command(codec, codec->mode16 & SB_MODE16_RECORD16 ? SB_DSP_DMA16_ON : SB_DSP_DMA8_ON);
+		snd_sb16dsp_command(codec, codec->mode16 & SB_MODE16_CAPTURE16 ? SB_DSP_DMA16_ON : SB_DSP_DMA8_ON);
 	} else {
 		codec->mode16 &= ~SB_MODE16_RATE_LOCK_R;
-		snd_sb16dsp_command(codec, codec->mode16 & SB_MODE16_RECORD16 ? SB_DSP_DMA16_OFF : SB_DSP_DMA8_OFF);
+		snd_sb16dsp_command(codec, codec->mode16 & SB_MODE16_CAPTURE16 ? SB_DSP_DMA16_OFF : SB_DSP_DMA8_OFF);
 		/* next two lines are needed for some types of DSP4 (SB AWE 32 - 4.13) */
 		if (codec->mode16 & SB_MODE16_RATE_LOCK_P)
 			snd_sb16dsp_command(codec, codec->mode16 & SB_MODE16_PLAYBACK16 ? SB_DSP_DMA16_ON : SB_DSP_DMA8_ON);
 	}
-	snd_spin_unlock(codec, reg, &flags);
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
 }
 
 void snd_sb16dsp_interrupt(snd_pcm_t * pcm, unsigned short status)
 {
 	int ok;
-	sbdsp_t *codec;
-	snd_pcm1_t *pcm1;
+	sbdsp_t *codec = (sbdsp_t *) pcm->private_data;
 	unsigned long flags;
 
-	pcm1 = (snd_pcm1_t *) pcm->private_data;
-	codec = (sbdsp_t *) pcm1->private_data;
 #if 0
 	PRINTK("sb16: interrupt - status = 0x%x\n", status);
 #endif
 	if (status & 0x01) {
 		ok = 0;
 		if ((codec->mode16 & (SB_MODE16_PLAYBACK16 | SB_MODE16_PLAYBACK)) == SB_MODE16_PLAYBACK) {
-			pcm1->playback.ack(pcm1);
+			codec->playback_subchn1->ack(codec->playback_subchn);
 			ok++;
 		}
-		if ((codec->mode16 & (SB_MODE16_RECORD16 | SB_MODE16_RECORD)) == SB_MODE16_RECORD) {
-			pcm1->record.ack(pcm1);
+		if ((codec->mode16 & (SB_MODE16_CAPTURE16 | SB_MODE16_CAPTURE)) == SB_MODE16_CAPTURE) {
+			codec->capture_subchn1->ack(codec->capture_subchn);
 			ok++;
 		}
-		snd_spin_lock(codec, reg, &flags);
+		spin_lock_irqsave(&codec->reg_lock, flags);
 		if (!ok)
 			snd_sb16dsp_command(codec, SB_DSP_DMA8_OFF);
 		snd_sb16_ack_8bit(codec);
-		snd_spin_unlock(codec, reg, &flags);
+		spin_unlock_irqrestore(&codec->reg_lock, flags);
 	}
 	if (status & 0x02) {
 		ok = 0;
 		if ((codec->mode16 & (SB_MODE16_PLAYBACK16 | SB_MODE16_PLAYBACK)) == (SB_MODE16_PLAYBACK16 | SB_MODE16_PLAYBACK)) {
-			pcm1->playback.ack(pcm1);
+			codec->playback_subchn1->ack(codec->playback_subchn);
 			ok++;
 		}
-		if ((codec->mode16 & (SB_MODE16_RECORD16 | SB_MODE16_RECORD)) == (SB_MODE16_RECORD16 | SB_MODE16_RECORD)) {
-			pcm1->record.ack(pcm1);
+		if ((codec->mode16 & (SB_MODE16_CAPTURE16 | SB_MODE16_CAPTURE)) == (SB_MODE16_CAPTURE16 | SB_MODE16_CAPTURE)) {
+			codec->capture_subchn1->ack(codec->capture_subchn);
 			ok++;
 		}
-		snd_spin_lock(codec, reg, &flags);
+		spin_lock_irqsave(&codec->reg_lock, flags);
 		if (!ok)
 			snd_sb16dsp_command(codec, SB_DSP_DMA16_OFF);
 		snd_sb16_ack_16bit(codec);
-		snd_spin_unlock(codec, reg, &flags);
+		spin_unlock_irqrestore(&codec->reg_lock, flags);
 	}
 }
 
@@ -436,22 +452,25 @@ void snd_sb16dsp_interrupt(snd_pcm_t * pcm, unsigned short status)
 
  */
 
-static int snd_sb16_playback_open(snd_pcm1_t * pcm1)
+static int snd_sb16_playback_open(void *private_data,
+				  snd_pcm_subchn_t * subchn)
 {
 	unsigned long flags;
-	sbdsp_t *codec;
+	sbdsp_t *codec = (sbdsp_t *) private_data;
+	snd_pcm1_channel_t *pchn1 = (snd_pcm1_channel_t *)subchn->pchn->private_data;
 	int err = -EBUSY;
 
-	codec = (sbdsp_t *) pcm1->private_data;
 	if (codec->dma16ptr &&
 	    (codec->force_mode16 & (SB_MODE16_PLAYBACK | SB_MODE16_AUTO))) {
-		if ((err = snd_pcm1_dma_alloc(pcm1, SND_PCM1_PLAYBACK, codec->dma16ptr, "Sound Blaster 16 DSP")) >= 0) {
-			snd_spin_lock(codec, open16, &flags);
+		if ((err = snd_pcm1_dma_alloc(subchn, codec->dma16ptr, "Sound Blaster 16 DSP")) >= 0) {
+			spin_lock_irqsave(&codec->open16_lock, flags);
 			codec->mode16 |= SB_MODE16_PLAYBACK16 | SB_MODE16_PLAYBACK;
-			pcm1->playback.hw.flags &= ~(SND_PCM1_HW_8BITONLY | SND_PCM1_HW_16BITONLY);
-			pcm1->playback.hw.flags |= SND_PCM1_HW_16BITONLY;
-			pcm1->record.hw.hw_formats = SND_PCM_FMT_S16_LE | SND_PCM_FMT_U16_LE;
-			snd_spin_unlock(codec, open16, &flags);
+			pchn1->hw.flags &= ~(SND_PCM1_HW_8BITONLY | SND_PCM1_HW_16BITONLY);
+			pchn1->hw.flags |= SND_PCM1_HW_16BITONLY;
+			pchn1->hw.hw_formats = SND_PCM_FMT_S16_LE | SND_PCM_FMT_U16_LE;
+			codec->playback_subchn = subchn;
+			codec->playback_subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
+			spin_unlock_irqrestore(&codec->open16_lock, flags);
 			return 0;
 		}
 	}
@@ -459,247 +478,276 @@ static int snd_sb16_playback_open(snd_pcm1_t * pcm1)
 	    ((codec->force_mode16 & (SB_MODE16_PLAYBACK | SB_MODE16_AUTO)) == SB_MODE16_AUTO ||
 	     (codec->force_mode16 & (SB_MODE16_PLAYBACK | SB_MODE16_AUTO)) == 0)
 	    ) {
-		if ((err = snd_pcm1_dma_alloc(pcm1, SND_PCM1_PLAYBACK, codec->dma8ptr, "Sound Blaster 16 DSP")) >= 0) {
-			snd_spin_lock(codec, open16, &flags);
+		if ((err = snd_pcm1_dma_alloc(subchn, codec->dma8ptr, "Sound Blaster 16 DSP")) >= 0) {
+			spin_lock_irqsave(&codec->open16_lock, flags);
 			codec->mode16 &= ~SB_MODE16_PLAYBACK16;
 			codec->mode16 |= SB_MODE16_PLAYBACK;
-			pcm1->playback.hw.flags &= ~(SND_PCM1_HW_8BITONLY | SND_PCM1_HW_16BITONLY);
-			pcm1->playback.hw.flags |= SND_PCM1_HW_8BITONLY;
-			pcm1->record.hw.hw_formats = SND_PCM_FMT_U8 | SND_PCM_FMT_S8;
-			snd_spin_unlock(codec, open16, &flags);
+			pchn1->hw.flags &= ~(SND_PCM1_HW_8BITONLY | SND_PCM1_HW_16BITONLY);
+			pchn1->hw.flags |= SND_PCM1_HW_8BITONLY;
+			pchn1->hw.hw_formats = SND_PCM_FMT_U8 | SND_PCM_FMT_S8;
+			codec->playback_subchn = subchn;
+			codec->playback_subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
+			spin_unlock_irqrestore(&codec->open16_lock, flags);
 			return 0;
 		}
 	}
 	return err;
 }
 
-static int snd_sb16_record_open(snd_pcm1_t * pcm1)
+static int snd_sb16_capture_open(void *private_data,
+				 snd_pcm_subchn_t * subchn)
 {
 	unsigned long flags;
-	sbdsp_t *codec;
+	sbdsp_t *codec = (sbdsp_t *) private_data;
+	snd_pcm1_channel_t *pchn1 = (snd_pcm1_channel_t *)subchn->pchn->private_data;
 	int err = -EBUSY;
 
-	codec = (sbdsp_t *) pcm1->private_data;
 	if (codec->dma16ptr &&
-	    (codec->force_mode16 & (SB_MODE16_RECORD | SB_MODE16_AUTO))) {
-		if ((err = snd_pcm1_dma_alloc(pcm1, SND_PCM1_RECORD, codec->dma16ptr, "Sound Blaster 16 DSP")) >= 0) {
-			snd_spin_lock(codec, open16, &flags);
-			codec->mode16 |= SB_MODE16_RECORD16 | SB_MODE16_RECORD;
-			pcm1->record.hw.flags &= ~(SND_PCM1_HW_8BITONLY | SND_PCM1_HW_16BITONLY);
-			pcm1->record.hw.flags |= SND_PCM1_HW_16BITONLY;
-			pcm1->record.hw.hw_formats = SND_PCM_FMT_S16_LE | SND_PCM_FMT_U16_LE;
-			snd_spin_unlock(codec, open16, &flags);
+	    (codec->force_mode16 & (SB_MODE16_CAPTURE | SB_MODE16_AUTO))) {
+		if ((err = snd_pcm1_dma_alloc(subchn, codec->dma16ptr, "Sound Blaster 16 DSP")) >= 0) {
+			spin_lock_irqsave(&codec->open16_lock, flags);
+			codec->mode16 |= SB_MODE16_CAPTURE16 | SB_MODE16_CAPTURE;
+			pchn1->hw.flags &= ~(SND_PCM1_HW_8BITONLY | SND_PCM1_HW_16BITONLY);
+			pchn1->hw.flags |= SND_PCM1_HW_16BITONLY;
+			pchn1->hw.hw_formats = SND_PCM_FMT_S16_LE | SND_PCM_FMT_U16_LE;
+			codec->capture_subchn = subchn;
+			codec->capture_subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
+			spin_unlock_irqrestore(&codec->open16_lock, flags);
 			return 0;
 		}
 	}
 	if (codec->dma8ptr &&
-	    ((codec->force_mode16 & (SB_MODE16_RECORD | SB_MODE16_AUTO)) == SB_MODE16_AUTO ||
-	    (codec->force_mode16 & (SB_MODE16_RECORD | SB_MODE16_AUTO)) == 0)) {
-		if ((err = snd_pcm1_dma_alloc(pcm1, SND_PCM1_RECORD, codec->dma8ptr, "Sound Blaster 16 DSP")) >= 0) {
-			snd_spin_lock(codec, open16, &flags);
-			codec->mode16 &= ~SB_MODE16_RECORD16;
-			codec->mode16 |= SB_MODE16_RECORD;
-			pcm1->record.hw.flags &= ~(SND_PCM1_HW_8BITONLY | SND_PCM1_HW_16BITONLY);
-			pcm1->record.hw.flags |= SND_PCM1_HW_8BITONLY;
-			pcm1->record.hw.hw_formats = SND_PCM_FMT_U8 | SND_PCM_FMT_S8;
-			snd_spin_unlock(codec, open16, &flags);
+	    ((codec->force_mode16 & (SB_MODE16_CAPTURE | SB_MODE16_AUTO)) == SB_MODE16_AUTO ||
+	    (codec->force_mode16 & (SB_MODE16_CAPTURE | SB_MODE16_AUTO)) == 0)) {
+		if ((err = snd_pcm1_dma_alloc(subchn, codec->dma8ptr, "Sound Blaster 16 DSP")) >= 0) {
+			spin_lock_irqsave(&codec->open16_lock, flags);
+			codec->mode16 &= ~SB_MODE16_CAPTURE16;
+			codec->mode16 |= SB_MODE16_CAPTURE;
+			pchn1->hw.flags &= ~(SND_PCM1_HW_8BITONLY | SND_PCM1_HW_16BITONLY);
+			pchn1->hw.flags |= SND_PCM1_HW_8BITONLY;
+			pchn1->hw.hw_formats = SND_PCM_FMT_U8 | SND_PCM_FMT_S8;
+			codec->capture_subchn = subchn;
+			codec->capture_subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
+			spin_unlock_irqrestore(&codec->open16_lock, flags);
 			return 0;
 		}
 	}
 	return err;
 }
 
-static void snd_sb16_playback_close(snd_pcm1_t * pcm1)
+static void snd_sb16_playback_close(void *private_data,
+				    snd_pcm_subchn_t * subchn)
 {
-	sbdsp_t *codec;
+	sbdsp_t *codec = (sbdsp_t *) private_data;
+	snd_pcm1_channel_t *pchn1 = (snd_pcm1_channel_t *)subchn->pchn->private_data;
 
-	codec = (sbdsp_t *) pcm1->private_data;
-	snd_pcm1_dma_free(pcm1, SND_PCM1_PLAYBACK,
-			  codec->mode16 & SB_MODE16_PLAYBACK16 ?
-			  codec->dma16ptr : codec->dma8ptr);
+	codec->playback_subchn = NULL;
+	codec->playback_subchn1 = NULL;
+	snd_pcm1_dma_free(subchn);
 	codec->mode16 &= ~(SB_MODE16_PLAYBACK | SB_MODE16_PLAYBACK16);
-	pcm1->playback.hw.flags &= ~(SND_PCM1_HW_8BITONLY | SND_PCM1_HW_16BITONLY);
+	pchn1->hw.flags &= ~(SND_PCM1_HW_8BITONLY | SND_PCM1_HW_16BITONLY);
 }
 
-static void snd_sb16_record_close(snd_pcm1_t * pcm1)
+static void snd_sb16_capture_close(void *private_data,
+				   snd_pcm_subchn_t * subchn)
 {
-	sbdsp_t *codec;
+	sbdsp_t *codec = (sbdsp_t *) private_data;
+	snd_pcm1_channel_t *pchn1 = (snd_pcm1_channel_t *)subchn->pchn->private_data;
 
-	codec = (sbdsp_t *) pcm1->private_data;
-	snd_pcm1_dma_free(pcm1, SND_PCM1_RECORD,
-			  codec->mode16 & SB_MODE16_RECORD16 ?
-			  codec->dma16ptr : codec->dma8ptr);
-	codec->mode16 &= ~(SB_MODE16_RECORD | SB_MODE16_RECORD16);
-	pcm1->record.hw.flags &= ~(SND_PCM1_HW_8BITONLY | SND_PCM1_HW_16BITONLY);
+	codec->capture_subchn = NULL;
+	codec->capture_subchn1 = NULL;
+	snd_pcm1_dma_free(subchn);
+	codec->mode16 &= ~(SB_MODE16_CAPTURE | SB_MODE16_CAPTURE16);
+	pchn1->hw.flags &= ~(SND_PCM1_HW_8BITONLY | SND_PCM1_HW_16BITONLY);
 }
 
-static unsigned int snd_sb16_playback_pointer(snd_pcm1_t * pcm1,
+static unsigned int snd_sb16_playback_pointer(void *private_data,
+					      snd_pcm_subchn_t * subchn,
 					      unsigned int used_size)
 {
-	sbdsp_t *codec;
-	snd_pcm1_channel_t *pchn;
+	sbdsp_t *codec = (sbdsp_t *) private_data;
+	snd_pcm1_subchn_t *subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
 	unsigned int result;
 
-	codec = (sbdsp_t *) pcm1->private_data;
-	pchn = &pcm1->playback;
 	if (codec->mode16 & SB_MODE16_PLAYBACK16) {
 		result = used_size - snd_dma_residue(codec->dma16);
-		if (!(pchn->mode & SND_PCM1_MODE_16))
+		if (!(subchn1->mode & SND_PCM1_MODE_16))
 			result >>= 1;
 	} else {
 		result = used_size - snd_dma_residue(codec->dma8);
-		if (pchn->mode & SND_PCM1_MODE_16)
+		if (subchn1->mode & SND_PCM1_MODE_16)
 			result <<= 1;
 	}
 	return result;
 }
 
-static unsigned int snd_sb16_record_pointer(snd_pcm1_t * pcm1,
-					    unsigned int used_size)
+static unsigned int snd_sb16_capture_pointer(void *private_data,
+					     snd_pcm_subchn_t * subchn,
+					     unsigned int used_size)
 {
-	sbdsp_t *codec;
-	snd_pcm1_channel_t *pchn;
+	sbdsp_t *codec = (sbdsp_t *) private_data;
+	snd_pcm1_subchn_t *subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
 	unsigned int result;
 
-	codec = (sbdsp_t *) pcm1->private_data;
-	pchn = &pcm1->record;
-	if (codec->mode16 & SB_MODE16_RECORD16) {
+	if (codec->mode16 & SB_MODE16_CAPTURE16) {
 		result = used_size - snd_dma_residue(codec->dma16);
-		if (!(pchn->mode & SND_PCM1_MODE_16))
+		if (!(subchn1->mode & SND_PCM1_MODE_16))
 			result >>= 1;
 	} else {
 		result = used_size - snd_dma_residue(codec->dma8);
-		if (pchn->mode & SND_PCM1_MODE_16)
+		if (subchn1->mode & SND_PCM1_MODE_16)
 			result <<= 1;
 	}
 	return result;
 }
 
-static void snd_sb16_playback_dma(snd_pcm1_t * pcm1,
-			      	  unsigned char *buffer, unsigned int offset,
-				  unsigned char *user, unsigned int count)
+static int snd_sb16_playback_dma(void *private_data,
+				 snd_pcm_subchn_t * subchn,
+			      	 unsigned char *buffer, unsigned int offset,
+				 unsigned char *user, unsigned int count)
 {
-	snd_pcm1_channel_t *pchn;
+	snd_pcm1_subchn_t *subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
+	snd_pcm1_channel_t *pchn1 = (snd_pcm1_channel_t *)subchn->pchn->private_data;
 
-	pchn = &pcm1->playback;
-	if (pchn->hw.flags & SND_PCM1_HW_8BITONLY) {
-		if (pchn->mode & SND_PCM1_MODE_16) {
-			snd_sb16_playback_16to8(buffer, offset, user, count);
-			return;
+	if (pchn1->hw.flags & SND_PCM1_HW_8BITONLY) {
+		if (subchn1->mode & SND_PCM1_MODE_16) {
+			if (snd_sb16_playback_16to8(buffer, offset, user, count))
+				return -EFAULT;
+			return 0;
 		}
 	}
-	if (pchn->hw.flags & SND_PCM1_HW_16BITONLY) {
-		if (!(pchn->mode & SND_PCM1_MODE_16)) {
-			snd_sb16_playback_8to16(buffer, offset, user, count, (pchn->mode & SND_PCM1_MODE_ULAW) ? 1 : 0);
-			return;
+	if (pchn1->hw.flags & SND_PCM1_HW_16BITONLY) {
+		if (!(subchn1->mode & SND_PCM1_MODE_16)) {
+			if (snd_sb16_playback_8to16(buffer, offset, user, count,
+				(subchn1->mode & SND_PCM1_MODE_ULAW) ? 1 : 0))
+				return -EFAULT;
+			return 0;
 		}
 	}
-	if (pcm1->playback.mode & SND_PCM1_MODE_ULAW)
-		snd_translate_from_user(snd_ulaw_dsp_loud, &buffer[offset], user, count);
-	else
-		copy_from_user(&buffer[offset], user, count);
+	if (subchn1->mode & SND_PCM1_MODE_ULAW) {
+		if (snd_translate_from_user(snd_ulaw_dsp_loud, &buffer[offset], user, count))
+			return -EFAULT;
+	} else {
+		if (copy_from_user(&buffer[offset], user, count))
+			return -EFAULT;
+	}
+	return 0;
 }
 
-static void snd_sb16_record_dma(snd_pcm1_t * pcm1,
+static int snd_sb16_capture_dma(void *private_data,
+				snd_pcm_subchn_t * subchn,
 			        unsigned char *buffer, unsigned int offset,
 				unsigned char *user, unsigned int count)
 {
-	snd_pcm1_channel_t *pchn;
+	snd_pcm1_subchn_t *subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
+	snd_pcm1_channel_t *pchn1 = (snd_pcm1_channel_t *)subchn->pchn->private_data;
 
-	pchn = &pcm1->record;
-	if (pchn->hw.flags & SND_PCM1_HW_8BITONLY) {
-		if (pchn->mode & SND_PCM1_MODE_16) {
-			snd_sb16_record_8to16(buffer, offset, user, count);
-			return;
+	if (pchn1->hw.flags & SND_PCM1_HW_8BITONLY) {
+		if (subchn1->mode & SND_PCM1_MODE_16) {
+			if (snd_sb16_capture_8to16(buffer, offset, user, count))
+				return -EFAULT;
+			return 0;
 		}
 	}
-	if (pchn->hw.flags & SND_PCM1_HW_16BITONLY) {
-		if (!(pchn->mode & SND_PCM1_MODE_16)) {
-			snd_sb16_record_16to8(buffer, offset, user, count, (pchn->mode & SND_PCM1_MODE_ULAW) ? 1 : 0);
-			return;
+	if (pchn1->hw.flags & SND_PCM1_HW_16BITONLY) {
+		if (!(subchn1->mode & SND_PCM1_MODE_16)) {
+			if (snd_sb16_capture_16to8(buffer, offset, user, count,
+				(subchn1->mode & SND_PCM1_MODE_ULAW) ? 1 : 0))
+				return -EFAULT;
+			return 0;
 		}
 	}
-	if (pcm1->record.mode & SND_PCM1_MODE_ULAW)
-		snd_translate_to_user(snd_dsp_ulaw_loud, user, &buffer[offset], count);
-	else
-		copy_to_user(user, &buffer[offset], count);
+	if (subchn1->mode & SND_PCM1_MODE_ULAW) {
+		if (snd_translate_to_user(snd_dsp_ulaw_loud, user, &buffer[offset], count))
+			return -EFAULT;
+	} else {
+		if (copy_to_user(user, &buffer[offset], count))
+			return -EFAULT;
+	}
+	return 0;
 }
 
-static void snd_sb16_playback_dma_move(snd_pcm1_t * pcm1,
-				       unsigned char *dbuffer,
-				       unsigned int dest_offset,
-			 	       unsigned char *sbuffer,
-			 	       unsigned int src_offset,
-				       unsigned int count)
+static int snd_sb16_playback_dma_move(void *private_data,
+				      snd_pcm_subchn_t * subchn,
+				      unsigned char *dbuffer,
+				      unsigned int dest_offset,
+			 	      unsigned char *sbuffer,
+			 	      unsigned int src_offset,
+				      unsigned int count)
 {
-	snd_pcm1_channel_t *pchn;
+	snd_pcm1_subchn_t *subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
+	snd_pcm1_channel_t *pchn1 = (snd_pcm1_channel_t *)subchn->pchn->private_data;
 
-	pchn = &pcm1->playback;
-	if (pchn->hw.flags & SND_PCM1_HW_8BITONLY) {
-		if (pchn->mode & SND_PCM1_MODE_16) {
+	if (pchn1->hw.flags & SND_PCM1_HW_8BITONLY) {
+		if (subchn1->mode & SND_PCM1_MODE_16) {
 			memcpy(&dbuffer[dest_offset >> 1], &sbuffer[src_offset >> 1], count >> 1);
-			return;
+			return 0;
 		}
 	}
-	if (pchn->hw.flags & SND_PCM1_HW_16BITONLY) {
-		if (!(pchn->mode & SND_PCM1_MODE_16)) {
+	if (pchn1->hw.flags & SND_PCM1_HW_16BITONLY) {
+		if (!(subchn1->mode & SND_PCM1_MODE_16)) {
 			memcpy(&dbuffer[dest_offset << 1], &sbuffer[src_offset << 1], count << 1);
-			return;
+			return 0;
 		}
 	}
-	snd_pcm1_dma_move(pcm1, dbuffer, dest_offset, sbuffer, src_offset, count);
+	return snd_pcm1_dma_move(private_data, subchn, dbuffer, dest_offset,
+				 sbuffer, src_offset, count);
 }
 
-static void snd_sb16_record_dma_move(snd_pcm1_t * pcm1,
+static int snd_sb16_capture_dma_move(void *private_data,
+				     snd_pcm_subchn_t * subchn,
 				     unsigned char *dbuffer,
 				     unsigned int dest_offset,
 			 	     unsigned char *sbuffer,
 			 	     unsigned int src_offset,
 				     unsigned int count)
 {
-	snd_pcm1_channel_t *pchn;
+	snd_pcm1_subchn_t *subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
+	snd_pcm1_channel_t *pchn1 = (snd_pcm1_channel_t *)subchn->pchn->private_data;
 
 #if 0
-	snd_printk("record dma move, buffer = 0x%lx, dest_offset = %i, src_offset = %i, count = %i\n", (long) buffer, dest_offset, src_offset, count);
+	snd_printk("capture dma move, buffer = 0x%lx, dest_offset = %i, src_offset = %i, count = %i\n", (long) buffer, dest_offset, src_offset, count);
 #endif
-	pchn = &pcm1->record;
-	if (pchn->hw.flags & SND_PCM1_HW_8BITONLY) {
-		if (pchn->mode & SND_PCM1_MODE_16) {
+	if (pchn1->hw.flags & SND_PCM1_HW_8BITONLY) {
+		if (subchn1->mode & SND_PCM1_MODE_16) {
 			memcpy(&dbuffer[dest_offset >> 1], &sbuffer[src_offset >> 1], count >> 1);
-			return;
+			return 0;
 		}
 	}
-	if (pchn->hw.flags & SND_PCM1_HW_16BITONLY) {
-		if (!(pchn->mode & SND_PCM1_MODE_16)) {
+	if (pchn1->hw.flags & SND_PCM1_HW_16BITONLY) {
+		if (!(subchn1->mode & SND_PCM1_MODE_16)) {
 			memcpy(&dbuffer[dest_offset << 1], &sbuffer[src_offset << 1], count << 1);
-			return;
+			return 0;
 		}
 	}
-	snd_pcm1_dma_move(pcm1, dbuffer, dest_offset, sbuffer, src_offset, count);
+	return snd_pcm1_dma_move(private_data, subchn, dbuffer, dest_offset,
+				 sbuffer, src_offset, count);
 }
 
-static void snd_sb16_playback_dma_neutral(snd_pcm1_t * pcm1,
-			      		  unsigned char *buffer,
-			      		  unsigned int offset,
-					  unsigned int count,
-					  unsigned char neutral_byte)
+static int snd_sb16_playback_dma_neutral(void *private_data,
+					 snd_pcm_subchn_t * subchn,
+			      		 unsigned char *buffer,
+			      		 unsigned int offset,
+					 unsigned int count,
+					 unsigned char neutral_byte)
 {
-	snd_pcm1_channel_t *pchn;
+	snd_pcm1_subchn_t *subchn1 = (snd_pcm1_subchn_t *)subchn->private_data;
+	snd_pcm1_channel_t *pchn1 = (snd_pcm1_channel_t *)subchn->pchn->private_data;
 
-	pchn = &pcm1->playback;
-	if (pchn->hw.flags & SND_PCM1_HW_8BITONLY) {
-		if (pchn->mode & SND_PCM1_MODE_16) {
+	if (pchn1->hw.flags & SND_PCM1_HW_8BITONLY) {
+		if (subchn1->mode & SND_PCM1_MODE_16) {
 			memset(&buffer[offset >> 1], neutral_byte, count >> 1);
-			return;
+			return 0;
 		}
 	}
-	if (pchn->hw.flags & SND_PCM1_HW_16BITONLY) {
-		if (!(pchn->mode & SND_PCM1_MODE_16)) {
+	if (pchn1->hw.flags & SND_PCM1_HW_16BITONLY) {
+		if (!(subchn1->mode & SND_PCM1_MODE_16)) {
 			memset(&buffer[offset << 1], neutral_byte, count << 1);
-			return;
+			return 0;
 		}
 	}
-	snd_pcm1_playback_dma_neutral(pcm1, buffer, offset, count, neutral_byte);
+	return snd_pcm1_playback_dma_neutral(private_data, subchn, buffer, offset,
+					     count, neutral_byte);
 }
 
 /*
@@ -708,8 +756,6 @@ static void snd_sb16_playback_dma_neutral(snd_pcm1_t * pcm1,
 
 struct snd_stru_pcm1_hardware snd_sb16_playback =
 {
-	NULL,			/* private data */
-	NULL,			/* private_free */
 	SND_PCM1_HW_AUTODMA,	/* flags */
 	SND_PCM_FMT_MU_LAW | SND_PCM_FMT_U8 | SND_PCM_FMT_S8 |
 	    SND_PCM_FMT_S16_LE | SND_PCM_FMT_U16_LE, /* formats */
@@ -731,10 +777,8 @@ struct snd_stru_pcm1_hardware snd_sb16_playback =
 	snd_sb16_playback_dma_neutral
 };
 
-struct snd_stru_pcm1_hardware snd_sb16_record =
+struct snd_stru_pcm1_hardware snd_sb16_capture =
 {
-	NULL,			/* private data */
-	NULL,			/* private free */
 	SND_PCM1_HW_AUTODMA,	/* flags */
 	SND_PCM_FMT_MU_LAW | SND_PCM_FMT_U8 | SND_PCM_FMT_S8 |
 	    SND_PCM_FMT_S16_LE | SND_PCM_FMT_U16_LE,	/* formats */
@@ -745,14 +789,14 @@ struct snd_stru_pcm1_hardware snd_sb16_record =
 	4000,			/* min. rate */
 	44100,			/* max. rate */
 	2,			/* max. voices */
-	snd_sb16_record_open,
-	snd_sb16_record_close,
-	snd_sb16_record_ioctl,
-	snd_sb16_record_prepare,
-	snd_sb16_record_trigger,
-	snd_sb16_record_pointer,
-	snd_sb16_record_dma,
-	snd_sb16_record_dma_move,
+	snd_sb16_capture_open,
+	snd_sb16_capture_close,
+	snd_sb16_capture_ioctl,
+	snd_sb16_capture_prepare,
+	snd_sb16_capture_trigger,
+	snd_sb16_capture_pointer,
+	snd_sb16_capture_dma,
+	snd_sb16_capture_dma_move,
 	NULL
 };
 
@@ -787,11 +831,11 @@ static void snd_sb16_proc_read(snd_info_buffer_t * buffer, void *private_data)
 	tmp = "8-bit";
 	if (codec->force_mode16 & SB_MODE16_AUTO)
 		tmp = "auto";
-	else if (codec->force_mode16 & SB_MODE16_RECORD)
+	else if (codec->force_mode16 & SB_MODE16_CAPTURE)
 		tmp = "16-bit";
 	tmp1 = "inactive";
-	if (codec->mode16 & SB_MODE16_RECORD) {
-		if (codec->mode16 & SB_MODE16_RECORD16)
+	if (codec->mode16 & SB_MODE16_CAPTURE) {
+		if (codec->mode16 & SB_MODE16_CAPTURE16)
 			tmp1 = "16-bit";
 		else
 			tmp1 = "8-bit";
@@ -808,7 +852,7 @@ static void snd_sb16_proc_setup_mode(sbdsp_t * codec, int what)
 	} else if (what == 1) {
 		codec->force_mode16 = SB_MODE16_PLAYBACK;
 	} else if (what == 2) {
-		codec->force_mode16 = SB_MODE16_RECORD;
+		codec->force_mode16 = SB_MODE16_CAPTURE;
 	}
 }
 
@@ -847,11 +891,9 @@ static void snd_sb16_proc_write(snd_info_buffer_t * buffer, void *private_data)
 void snd_sb16dsp_proc_init(snd_pcm_t * pcm)
 {
 	sbdsp_t *codec;
-	snd_pcm1_t *pcm1;
 	snd_info_entry_t *entry;
 
-	pcm1 = (snd_pcm1_t *) pcm->private_data;
-	codec = (sbdsp_t *) pcm1->private_data;
+	codec = (sbdsp_t *) pcm->private_data;
 	if ((entry = snd_info_create_entry(pcm->card, "sb16")) != NULL) {
 		entry->private_data = codec;
 		entry->mode = S_IFREG | S_IRUGO | S_IWUSR;
@@ -870,10 +912,8 @@ void snd_sb16dsp_proc_init(snd_pcm_t * pcm)
 void snd_sb16dsp_proc_done(snd_pcm_t * pcm)
 {
 	sbdsp_t *codec;
-	snd_pcm1_t *pcm1;
 
-	pcm1 = (snd_pcm1_t *) pcm->private_data;
-	codec = (sbdsp_t *) pcm1->private_data;
+	codec = (sbdsp_t *) pcm->private_data;
 	if (codec->proc_entry) {
 		snd_info_unregister(codec->proc_entry);
 		codec->proc_entry = NULL;
