@@ -24,14 +24,11 @@
 
 #define LinuxVersionCode(v, p, s) (((v)<<16)|((p)<<8)|(s))
 
-#if LinuxVersionCode(2, 0, 0) > LINUX_VERSION_CODE
-#error "This driver is designed only for Linux 2.0.0 and highter."
-#endif
-#if LinuxVersionCode(2, 1, 0) <= LINUX_VERSION_CODE
-#define LINUX_2_1
-#if LinuxVersionCode(2, 1, 127) > LINUX_VERSION_CODE
-#error "This driver requires Linux 2.1.127 and highter."
+#if LinuxVersionCode(2, 2, 0) > LINUX_VERSION_CODE
+#error "This driver is designed only for Linux 2.2.0 and highter."
 #endif
+#if LinuxVersionCode(2, 3, 11) <= LINUX_VERSION_CODE
+#define NEW_RESOURCE
 #endif
 
 #include <linux/module.h>
@@ -44,33 +41,17 @@
 #include <asm/io.h>
 #include <asm/dma.h>
 #include <asm/irq.h>
-#ifdef CONFIG_PCI
 #include <linux/pci.h>
-#ifdef LINUX_2_1
 #include <linux/vmalloc.h>
 #include <linux/poll.h>
+#include <linux/init.h>
 #include <asm/uaccess.h>
-#else
-#include <linux/bios32.h>
-#endif
-#endif
 #ifndef ALSA_BUILD
 #include <linux/isapnp.h>
 #else
 #include "isapnp.h"
 #endif
 
-#ifndef __initfunc
-#define __initfunc(__initarg) __initarg 
-#else
-#include <linux/init.h>
-#endif
-
-#ifndef LINUX_2_1
-#define copy_from_user memcpy_fromfs
-#define copy_to_user memcpy_tofs
-#endif 
-
 #ifdef CONFIG_PROC_FS
 #include "isapnp_proc.c"
 #endif
@@ -82,6 +63,12 @@
 #define ISAPNP_DEBUG
 #endif
 
+#ifdef NEW_RESOURCE
+struct resource *pidxr_res = NULL;
+struct resource *pnpwrp_res = NULL;
+struct resource *isapnp_rdp_res = NULL;
+#endif
+
 int isapnp_disable = 0;			/* Disable ISA PnP */
 int isapnp_rdp = 0;			/* Read Data Port */
 int isapnp_reset = 0;			/* reset all PnP cards (deactivate) */
@@ -91,7 +78,6 @@ int isapnp_reserve_irq[16] = { [0 ... 15] = -1 };	/* reserve (don't use) some IR
 int isapnp_reserve_dma[8] = { [0 ... 7] = -1 };		/* reserve (don't use) some DMA */
 int isapnp_reserve_io[16] = { [0 ... 15] = -1 };	/* reserve (don't use) some I/O region */
 int isapnp_reserve_mem[16] = { [0 ... 15] = -1 };	/* reserve (don't use) some memory region */
-#ifdef MODULE_PARM
 MODULE_AUTHOR("Jaroslav Kysela <perex@suse.cz>");
 MODULE_DESCRIPTION("Generic ISA Plug & Play support");
 MODULE_PARM(isapnp_disable, "i");
@@ -112,7 +98,6 @@ MODULE_PARM(isapnp_reserve_io, "1-16i");
 MODULE_PARM_DESC(isapnp_reserve_io, "ISA Plug & Play - reserve I/O region(s) - port,size");
 MODULE_PARM(isapnp_reserve_mem, "1-16i");
 MODULE_PARM_DESC(isapnp_reserve_mem, "ISA Plug & Play - reserve memory region(s) - address,size");
-#endif
 
 #define _PIDXR		0x279
 #define _PNPWRP		0xa79
@@ -1708,6 +1693,10 @@ static int isapnp_check_mem(unsigned int addr, unsigned int size)
 			return 1;
 		if (addr + size > raddr && addr + size < (raddr + rsize) - 1)
 			return 1;
+#ifdef NEW_RESOURCE
+		if (__check_region(&iomem_resource, addr, size))
+			return 1;
+#endif
 	}
 	return 0;
 }
@@ -1947,11 +1936,23 @@ static void isapnp_free_all_resources(void)
 #endif
 
 #ifdef ISAPNP_REGION_OK
+#ifdef NEW_RESOURCE
+	release_resource(pidxr_res);
+#else
 	release_region(_PIDXR, 1);
 #endif
+#endif
+#ifdef NEW_RESOURCE
+	release_resource(pnpwrp_res);
+#else
 	release_region(_PNPWRP, 1);
+#endif
 	if (isapnp_rdp >= 0x203 && isapnp_rdp <= 0x3ff)
+#ifdef NEW_RESOURCE
+		release_resource(isapnp_rdp_res);
+#else
 		release_region(isapnp_rdp, 1);
+#endif
 #ifdef MODULE
 	for (dev = isapnp_devices; dev; dev = devnext) {
 		devnext = dev->next;
@@ -1989,7 +1990,6 @@ __initfunc(static int isapnp_do_reserve_irq(int irq))
 }
 
 #ifdef CONFIG_PCI
-#ifdef LINUX_2_1
 
 __initfunc(static void isapnp_pci_init(void))
 {
@@ -2013,93 +2013,8 @@ __initfunc(static void isapnp_pci_init(void))
 	}
 }
 
-#else
-
-__initfunc(static void isapnp_pci_scan_bus(int bus))
-{
-	int devfn, is_multi = 0;
-	unsigned char hdr_type, irq;
-	unsigned int l, class_type, buses;
-	
-#ifdef ISAPNP_DEBUG
-	printk("PCI: scanning bus %i\n", bus);
-#endif
-	for (devfn = 0; devfn < 255; devfn++) {
-		if (PCI_FUNC(devfn) && !is_multi)
-			continue;
-		pcibios_read_config_byte(bus, devfn, PCI_HEADER_TYPE, &hdr_type);
-		if (!PCI_FUNC(devfn))
-			is_multi = hdr_type & 0x80;
-			
-		/* some broken boards return 0 if a slot is empty: */
-		pcibios_read_config_dword(bus, devfn, PCI_VENDOR_ID, &l);
-		if (l == 0xffffffff || l == 0x00000000)
-			continue;
-
-#ifdef ISAPNP_DEBUG
-		printk("PCI: devfn = %i, hdr_type = 0x%x, Vendor ID = 0x%x\n", devfn, (int)hdr_type, l);
-#endif
-		pcibios_read_config_dword(bus, devfn, PCI_CLASS_REVISION, &class_type);
-		class_type >>= 8;
-		
-		if ((class_type >> 8) == PCI_CLASS_BRIDGE_PCI) {
-			pcibios_read_config_dword(bus, devfn, 0x18, &buses);
-			if (buses)
-				isapnp_pci_scan_bus((buses >> 8) & 0xff);
-		}
-
-		pcibios_read_config_byte(bus, devfn, PCI_INTERRUPT_LINE, &irq);
-#ifdef ISAPNP_DEBUG
-		printk("PCI: reserved IRQ: %i\n", irq);
-#endif
-		if (irq > 0 && irq < 15)
-			isapnp_do_reserve_irq(irq);
-	}
-}
-
-__initfunc(static void isapnp_pci_init(void))
-{
-	isapnp_pci_scan_bus(0);
-}
-
-#endif
 #endif /* CONFIG_PCI */
 
-#ifndef LINUX_2_1
-static struct symbol_table isapnp_syms = {
-	#include <linux/symtab_begin.h>
-	X(isapnp_present),
-	X(isapnp_cfg_begin),
-	X(isapnp_cfg_end),
-	X(isapnp_cfg_get_byte),
-	X(isapnp_cfg_get_word),
-	X(isapnp_cfg_get_dword),
-	X(isapnp_cfg_set_byte),
-	X(isapnp_cfg_set_word),
-	X(isapnp_cfg_set_dword),
-	X(isapnp_wake),
-	X(isapnp_logdev),
-	X(isapnp_activate),
-	X(isapnp_deactivate),
-	X(isapnp_find_port),
-	X(isapnp_find_irq),
-	X(isapnp_find_dma),
-	X(isapnp_find_mem),
-	X(isapnp_find_mem32),
-	X(isapnp_verify_port),
-	X(isapnp_verify_irq),
-	X(isapnp_verify_dma),
-	X(isapnp_verify_mem),
-	X(isapnp_verify_mem32),
-	X(isapnp_find_device),
-	X(isapnp_find_logdev),
-	X(isapnp_config_init),
-	X(isapnp_configure),
-#include <linux/symtab_end.h>
-};
-#endif /* !LINUX_2_1 */
-
-#ifdef LINUX_2_1
 EXPORT_SYMBOL(isapnp_present);
 EXPORT_SYMBOL(isapnp_cfg_begin);
 EXPORT_SYMBOL(isapnp_cfg_end);
@@ -2127,7 +2042,6 @@ EXPORT_SYMBOL(isapnp_find_device);
 EXPORT_SYMBOL(isapnp_find_logdev);
 EXPORT_SYMBOL(isapnp_config_init);
 EXPORT_SYMBOL(isapnp_configure);
-#endif /* LINUX_2_1 */
 
 __initfunc(int isapnp_init(void))
 {
@@ -2141,27 +2055,46 @@ __initfunc(int isapnp_init(void))
 		return 0;
 	}
 #ifdef ISAPNP_REGION_OK
+#ifdef NEW_RESOURCE
+	pidxr_res=request_region(_PIDXR, 1, "isapnp index");
+	if(!pidxr_res) {
+#else
 	if (check_region(_PIDXR, 1)) {
+#endif
 		printk("isapnp: Index Register 0x%x already used\n", _PIDXR);
 		return -EBUSY;
 	}
 #endif
+#ifdef NEW_RESOURCE
+	pnpwrp_res=request_region(_PNPWRP, 1, "isapnp write");
+	if(!pnpwrp_res) {
+#else
 	if (check_region(_PNPWRP, 1)) {
+#endif
 		printk("isapnp: Write Data Register 0x%x already used\n", _PNPWRP);
 		return -EBUSY;
 	}
 	if (isapnp_rdp >= 0x203 && isapnp_rdp <= 0x3ff) {
+#ifdef NEW_RESOURCE
+		isapnp_rdp_res=request_region(isapnp_rdp, 1, "isapnp read");
+		if(!isapnp_rdp_res) {
+#else
 		if (check_region(isapnp_rdp, 1)) {
+#endif
 			printk("isapnp: Read Data Register 0x%x already used\n", isapnp_rdp);
 			return -EBUSY;
 		}
+#ifndef NEW_RESOURCE
 		request_region(isapnp_rdp, 1, "isapnp read");
+#endif
 	}
 	isapnp_detected = 1;
+#ifndef NEW_RESOURCE
 #ifdef ISAPNP_REGION_OK
 	request_region(_PIDXR, 1, "isapnp index");
 #endif
 	request_region(_PNPWRP, 1, "isapnp write");
+#endif
 	if (isapnp_rdp < 0x203 || isapnp_rdp > 0x3ff) {
 		devices = isapnp_isolate();
 		if (devices < 0 || 
@@ -2171,7 +2104,11 @@ __initfunc(int isapnp_init(void))
 			printk("isapnp: No Plug & Play device found\n");
 			return 0;
 		}
+#ifdef NEW_RESOURCE
+		isapnp_rdp_res=request_region(isapnp_rdp, 1, "isapnp read");
+#else
 		request_region(isapnp_rdp, 1, "isapnp read");
+#endif
 	}
 	isapnp_build_device_list();
 	devices = 0;
@@ -2201,10 +2138,6 @@ __initfunc(int isapnp_init(void))
 #endif
 #ifdef CONFIG_PROC_FS
 	isapnp_proc_init();
-#endif
-#ifndef LINUX_2_1
-	if (register_symtab(&isapnp_syms))
-		printk("isapnp: cannot register symtab!!!\n");
 #endif
 	return 0;
 }
